#-*-coding:utf8-*-
#############################################################################
#    Baram : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of Baram.                                            
#
#    Baram is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    Baram is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Baram.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for run Basic flow solver
#
# Author
#    NEXTFOAM Inc. 
##############################################################################
import os,math,glob,sys;import pygtk;pygtk.require('2.0');import gtk
import time

from writeFileBasicFlow import *
from generalSub import *
from changeDict import *
from misc import *

class basicFlowSolvingClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir,solver,installpath,terminal):
        self.caseDir=caseDir
        self.solver=solver
        self.installpath=installpath
        self.terminal=terminal

    def runSolver(self,inputDictList,mode,notebook,textbuffer):
    
        #inputDictList=[simDict,initialDict,runControlDict,bcDictList,fireDict]
    
        simDict=inputDictList[0];runControlDict=inputDictList[2];bcDictList=inputDictList[3]
        nCore=runControlDict['nCores'];startFrom=runControlDict['startFrom'];plotresidual=runControlDict['plotResidual']
        smpOrCluster=runControlDict['parallelType'];hostfile=runControlDict['hostfile'];startTime=runControlDict['startTime']

        GEN=generalClass(self.caseDir);WFILE=writeFileClassBasicFlow(self.caseDir);MISC=miscClass(self.caseDir)
	
        ss=glob.glob(self.caseDir+'/Running')
        if len(ss)==0:
            if mode=='initialize':
                if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                    MISC.UFileBasic();MISC.pFileBasic();MISC.kFileBasic(self.solver);MISC.epsilonFileBasic(self.solver);MISC.omegaFileBasic(self.solver);MISC.nutFileBasic()
                else:
                    MISC.UFileBasic();MISC.pFileBasicThermo();MISC.p_rghFileBasicThermo();MISC.tFileBasic()
                    MISC.kFileBasic(self.solver);MISC.epsilonFileBasic(self.solver);MISC.omegaFileBasic(self.solver);MISC.mutFileBasic();MISC.alphatFileBasic()
                savedResult=GEN.getResult()
                for ii in savedResult[0]:os.system('rm -r '+self.caseDir+'/'+ii)
                if len(savedResult[1])!=0:os.system('rm -r '+self.caseDir+'/processor*')
                self.solverSet(inputDictList,mode,notebook,textbuffer)
            else:
                self.solverSet(inputDictList,mode,notebook,textbuffer)
                self.solverRun(startFrom,startTime,nCore,plotresidual,smpOrCluster,hostfile,notebook,textbuffer)
        else:
            md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Solver is Running")
            md.run();md.destroy()
    #----------------------------------------------------------------------------------
    def solverSet(self,inputDictList,mode,notebook,textbuffer):

        GEN=generalClass(self.caseDir);WFILE=writeFileClassBasicFlow(self.caseDir);CHDICT=changeDictionary(self.caseDir,self.solver)	

        simDict=inputDictList[0];initialDict=inputDictList[1];runControlDict=inputDictList[2]
        bcDictList=inputDictList[3];fireDict=inputDictList[4]

        if simDict['densitymode']=='Constant':rho=simDict['rho']
        else:rho='perfectGas'

        gg='('+simDict['gravx']+' '+simDict['gravy']+' '+simDict['gravz']+')'
        
        operatingPressure=simDict['operatingPressure']

        nCore=runControlDict['nCores'];startFrom=runControlDict['startFrom'];plotresidual=runControlDict['plotResidual']
        smpOrCluster=runControlDict['parallelType'];hostfile=runControlDict['hostfile'];startTime=runControlDict['startTime']
      
        if mode=='initialize':
            CHDICT.changeInternalField(initialDict,notebook,textbuffer,operatingPressure)                       
            CHDICT.UFile(bcDictList,startFrom,startTime,nCore,initialDict,rho,notebook,textbuffer)                                                         
            CHDICT.kFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            if simDict['turmodel']=='kOmegaSST':CHDICT.omegaFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            elif simDict['turmodel']!='laminar':CHDICT.epsilonFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                CHDICT.pFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            else:
                CHDICT.p_rghFileThermo(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                CHDICT.TFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer) 
        else:            
            savedResult=GEN.getResult()
            if savedResult[0]!=[] or savedResult[1]!=[]:
                CHDICT.changeInternalField(initialDict,notebook,textbuffer,operatingPressure)                       
                CHDICT.UFile(bcDictList,startFrom,startTime,nCore,initialDict,rho,notebook,textbuffer)                                                         
                CHDICT.kFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                if simDict['turmodel']=='kOmegaSST':CHDICT.omegaFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                elif simDict['turmodel']!='laminar':CHDICT.epsilonFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                    CHDICT.pFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                else:
                    CHDICT.p_rghFileThermo(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                    CHDICT.TFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)             
            
        if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
            WFILE.transportProperties(simDict['mu'],simDict['rho'])
        else:
            WFILE.gravity(gg)
            WFILE.operatingConditions(operatingPressure)
            WFILE.thermophysicalProperties(simDict)
        
        WFILE.turbulenceProperties('RAS')
        WFILE.RASProperties(self.solver,simDict['turmodel'])  
                
        WFILE.controlDictFile(runControlDict,self.solver)        
        WFILE.decomposeParDict(nCore)

        po=glob.glob(self.caseDir+'/system/cellZoneConditions:*:Porous');mr=glob.glob(self.caseDir+'/system/cellZoneConditions:*:MRF')
        fu=glob.glob(self.caseDir+'/system/cellZoneConditions:*:Fixed.U');ms=glob.glob(self.caseDir+'/system/cellZoneConditions:*:Mom.Source')
        hs=glob.glob(self.caseDir+'/system/cellZoneConditions:*:None')
        WFILE.fvOptions(po,mr,fu,ms,hs)  
    #----------------------------------------------------------------------------------
    def solverRun(self,startFrom,startTime,nCore,plotresidual,smpOrCluster,hostfile,notebook,textbuffer):

        GEN=generalClass(self.caseDir);results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        notebook.set_current_page(2) 
        
        rf=open(self.caseDir+'/runSolver','w')
        rf.write('touch '+self.caseDir+'/Running \n')        

        aa=glob.glob(self.caseDir+'/postProcessing')
        if len(aa)!=0:rf.write('rm -r '+self.caseDir+'/postProcessing\n')

        aa=glob.glob(self.caseDir+'/PyFoam*')
        if len(aa)!=0:rf.write('rm -r '+self.caseDir+'/PyFoam*\n')
	
        if nCore=='1':#serial
            if len(parallelResult)==0:GEN.clearParallel()
            else:
                rf.write('reconstructPar -case /'+self.caseDir+' -latestTime\n')
                aa=glob.glob(self.caseDir+'/processor*')
                if len(aa)!=0:rf.write('rm -r '+self.caseDir+'/processor*\n')
                
            if plotresidual=='true':
                rf.write('pyFoamPlotRunner.py --no-continuity '+self.solver+' -case '+self.caseDir+'\n')
            else:
                rf.write('pyFoamRunner.py '+self.solver+' -case '+self.caseDir+'\n')

        else:#parallel
            aa=glob.glob(self.caseDir+'/processor*');nCorePrev=str(len(aa))
            if nCore==nCorePrev:
                GEN.clearResult(serialResult,'0')
                if smpOrCluster=='SMP':
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')
                    else:
                        rf.write('pyFoamRunner.py mpirun -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')
                else:
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')
                    else:
                        rf.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')

            else:#nCore!=nCorePrev
                if nCorePrev=='0':
                    rf.write('decomposePar -force -case '+self.caseDir+'\n')
                else:
                    GEN.clearResult(serialResult,'0')
                    rf.write('reconstructPar -latestTime -case '+self.caseDir+'\n')
                    rf.write('decomposePar -force -case '+self.caseDir+'\n')

                if smpOrCluster=='SMP':
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')
                    else:
                        rf.write('pyFoamRunner.py mpirun -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')
                else:
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')
                    else:
                        rf.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+self.solver+' -case '+self.caseDir+' -parallel \n')

        rf.write('rm '+self.caseDir+'/Running \n')
        rf.close()
        os.system('chmod +x '+self.caseDir+'/runSolver')
        cmd=self.caseDir+'/runSolver\n'
        length=len(cmd)
        self.terminal.feed_child(cmd,length)
            
        stime=time.asctime(time.localtime(time.time()))
        message='\n [  Solving is started  at '+stime +'  ]\n\n\n'         
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)






         

  


