#-*-coding:utf8-*-
#############################################################################
#    Baram : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of Baram.                                            
#
#    Baram is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    Baram is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Baram.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for writing various files basic flow solver
#
# Author
#    NEXTFOAM Inc. 
##############################################################################

import math,os,glob,sys

from generalSub import *

class writeFileClassBasicFlow:

    def __init__(self,caseDir):
        self.caseDir=caseDir
	
    #-------------------------------------------------------------------------------------
    def foamComment(self):
        fc=[]
        fc.append('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        fc.append('| =========                 |                                                 |\n')
        fc.append('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
        fc.append('|  \\    /   O peration     | Version:  2.3.0                                 |\n')
        fc.append('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      |\n')
        fc.append('|    \\/     M anipulation  |                                                 |\n')
        fc.append('\*---------------------------------------------------------------------------*/\n')	
        return fc
    #------------------------------------------------------------------------------
    def header(self,location,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       dictionary;\n')
        he.append('    location    "'+location+'";\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def BCheader(self,classtype,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       '+classtype+';\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def RASProperties(self,solver,turModel):
    
        if turModel=='standardKEpsilon':
            if solver=='simpleNFoam' or solver=='pimpleNFoam':model='NEXT::kEpsilon'
            else:model='standardKEpsilon'
        elif turModel=='realizableKE':model='NEXT::realizableKE'
        elif turModel=='RNGkEpsilon':model='NEXT::RNGkEpsilon'
        elif turModel=='kOmegaSST':model='NEXT::kOmegaSST'
        elif turModel=='laminar':model='laminar'        
        
        fcom=self.foamComment();head=self.header('constant','RASProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/RASProperties','w')
        for aaa in hhh:f.write(aaa)	
        f.write('RASModel        '+model+';\n')
        f.write('turbulence      on;\n')
        f.write('printCoeffs     on;\n')        
        f.close()
    #------------------------------------------------------------------------------
    def turbulenceProperties(self,turModel):
        fcom=self.foamComment();head=self.header('constant','turbulenceProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/turbulenceProperties','w')
        for aaa in hhh:f.write(aaa)	
        if turModel=='laminar' or turModel=='inviscid':f.write('simulationType        laminar;\n')
        else:f.write('simulationType        RASModel;\n')
        f.close()
    #------------------------------------------------------------------------------
    def transportProperties(self,viscosity,density):
        fcom=self.foamComment();head=self.header('constant','transportProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/transportProperties','w')
        for aaa in hhh:f.write(aaa)
        f.write('transportModel    Newtonian;\n')
        nu=float(viscosity)/float(density)
        f.write('nu    nu [ 0 2 -1 0 0 0 0 ] '+str(nu)+';\n')
        f.close()
    #------------------------------------------------------------------------------
    def dynamicMeshDict(self,sm):
        fcom=self.foamComment();head=self.header('constant','dynamicMeshDict');hhh=fcom+head
        f=open(self.caseDir+'/constant/dynamicMeshDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('dynamicFvMesh   multiSolidBodyMotionFvMesh;\n')
        f.write('motionSolverLibs ( "libfvMotionSolvers.so" );\n')
        f.write('\n')
        f.write('multiSolidBodyMotionFvMeshCoeffs\n')
        f.write('{\n')
        for ii in sm:f.write('    #include "'+ii+'"\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def decomposeParDict(self,ncore):
        fcom=self.foamComment();head=self.header('system','decomposeParDict');hhh=fcom+head
        f=open(self.caseDir+'/system/decomposeParDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('numberOfSubdomains     '+ncore+';\n')
        f.write('method                 scotch;\n')
        f.close()
    #------------------------------------------------------------------------------
    def controlDictFile(self,runControlDict,solver):
        f1=glob.glob(self.caseDir+'/system/probeFunction_*')
        f2=glob.glob(self.caseDir+'/system/surfaceProbeFunction')
        f3=glob.glob(self.caseDir+'/system/flowRateFunction')
        f4=glob.glob(self.caseDir+'/system/forces')
        f5=glob.glob(self.caseDir+'/system/forceCoeffs')

        fcom=self.foamComment();head=self.header('system','controlDict');hhh=fcom+head
        f=open(self.caseDir+'/system/controlDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('application        '+solver+';\n')
        f.write('startFrom          '+runControlDict['startFrom']+';\n')
        f.write('startTime          '+runControlDict['startTime']+';\n')
        f.write('stopAt             endTime;\n')
        f.write('endTime            '+runControlDict['endTime']+';\n')
        f.write('deltaT             '+runControlDict['timeStepSize']+';\n')
        f.write('writeControl       '+runControlDict['writeControl']+';\n')
        f.write('writeInterval      '+runControlDict['writeInterval']+';\n')
        f.write('purgeWrite         '+runControlDict['purgeWrite']+';\n')
        f.write('writeFormat        '+runControlDict['writeFormat']+';\n')
        f.write('writePrecision     '+runControlDict['writePrecision']+';\n')  
        f.write('writeCompression   '+runControlDict['dataCompression']+';\n')
        f.write('timeFormat         general;\n')
        f.write('timePrecision      6;\n')
        f.write('runTimeModifiable  yes;\n')
        f.write('adjustTimeStep     '+runControlDict['adjustTimeStep']+';\n')
        f.write('maxCo              '+runControlDict['maxCo']+';\n')                        
        f.write('functions\n')
        f.write('{\n')
        if len(f1)!=0:
            for ii in f1:f.write('    #include "'+ii+'"\n')
        if len(f2)!=0:f.write('    #include "surfaceProbeFunction"\n')
        if len(f3)!=0:f.write('    #include "flowRateFunction"\n')
        if len(f4)!=0:f.write('    #include "forces"\n')
        if len(f5)!=0:f.write('    #include "forceCoeffs"\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def fvOptions(self,po,mr,fu,ms,hs):
        fcom=self.foamComment();head=self.header('system','fvOptions');hhh=fcom+head
        f=open(self.caseDir+'/system/fvOptions','w')
        for aaa in hhh:f.write(aaa)
        if len(po)!=0:
            for ii in po:f.write('#include "'+ii+'"\n')
        if len(mr)!=0:
            for ii in mr:f.write('#include "'+ii+'"\n')
        if len(fu)!=0:
            for ii in fu:f.write('#include "'+ii+'"\n')
        if len(ms)!=0:
            for ii in ms:f.write('#include "'+ii+'"\n')
        if len(hs)!=0:
            for ii in hs:f.write('#include "'+ii+'"\n')
        f.close()
    #------------------------------------------------------------------------------
    def gravity(self,gg):
        fcom=self.foamComment();head=self.header('constant','g');hhh=fcom+head
        f=open(self.caseDir+'/constant/g','w')
        for aaa in hhh:f.write(aaa)
        f.write('dimensions      [0 1 -2 0 0 0 0];\n')
        f.write('value           '+gg+';\n')
        f.close()
    #------------------------------------------------------------------------------
    def thermophysicalProperties(self,simDict):
    
        GEN=generalClass(self.caseDir)
        
        hf='0';sf='0'

        if simDict['densitymode']=='Constant':
            equationOfState='rhoConst';transport='const';thermo='hConst'
            mw='28.96';mu=simDict['mu']            
            cp=simDict['cp'];kk=simDict['conductivity'];pr=float(cp)*float(mu)/float(kk)#;hf='0';sf='0'
        elif simDict['densitymode']=='Perfect gas':
            equationOfState='perfectGas'
            if simDict['transmode']=='Constant':transport='const'
            elif simDict['transmode']=='Sutherland':transport='sutherland'
            thermo='hConst'
            mw=simDict['mw'];mu=simDict['mu'];As=simDict['As'];Ts=simDict['Ts']
            cp=simDict['cp'];kk=simDict['conductivity'];pr=float(cp)*float(mu)/float(kk)
        elif simDict['densitymode']=='Perfect liquid':
            equationOfState='perfectFluid'
            if simDict['perfectliquidTransMode']=='const':transport='const'
            elif simDict['perfectliquidTransMode']=='sutherland':transport='sutherland'
            thermo='hConst'
            mw=simDict['perfectliquidM'];mu=simDict['perfectliquidMu']
            As=simDict['perfectliquidAs'];Ts=simDict['perfectliquidTs'];cp=simDict['perfectliquidCp'];kk=simDict['perfectliquidK']
            pr=float(cp)*float(mu)/float(kk)
        elif simDict['densitymode']=='Polynomial':
            aa=glob.glob(self.caseDir+'/system/settings/polynomialProperties')
            if len(aa)!=0:
                polyDict=GEN.makeDictFromFile(aa[0])
                mucoeff=polyDict['mucoeff'].split(' ');kappacoeff=polyDict['kappacoeff'].split(' ')
                cpcoeff=polyDict['cpcoeff'].split(' ');rhocoeff=polyDict['rhocoeff'].split(' ')
            else:
                md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Error!!! Coefficient of polynomial is not set")
                md.run();md.destroy()            
            equationOfState='icoPolynomial';transport='polynomial';thermo='hPolynomial';mw=simDict['polynomialM'];num=simDict['polynomialNo']
            
        fcom=self.foamComment();head=self.header('constant','thermophysicalProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/thermophysicalProperties','w')
        for aaa in hhh:f.write(aaa)           
        f.write('thermoType\n')
        f.write('{\n')
        f.write('    type             heRhoThermo;\n')
        f.write('    mixture          pureMixture;\n')
        f.write('    transport        '+transport+';\n')
        f.write('    thermo           '+thermo+';\n')
        f.write('    equationOfState  '+equationOfState+';\n')
        f.write('    specie           specie;\n')
        f.write('    energy           sensibleEnthalpy;\n')
        f.write('}\n')

        f.write('mixture\n')
        f.write('{\n')
        f.write('    specie\n')
        f.write('    {\n')
        f.write('        nMoles    1;\n')
        f.write('        molWeight '+mw+';\n')
        f.write('    }\n')

        f.write('    transport\n')
        f.write('    {\n')
        if transport=='const':
            f.write('        mu    '+mu+';\n')
            f.write('        Pr    '+str(pr)+';\n')
        elif transport=='sutherland':
            f.write('        As    '+As+';\n')
            f.write('        Ts    '+Ts+';\n')                       
        elif transport=='polynomial':
            f.write('        muCoeffs<'+num+'>    (')
            for ii in mucoeff:f.write(ii+' ')
            f.write(');\n')
            f.write('        kappaCoeffs<'+num+'>    (')
            for ii in kappacoeff:f.write(ii+' ')
            f.write(');\n')
        f.write('    }\n')

        f.write('    thermodynamics\n')
        f.write('    {\n')
        if thermo=='hConst':
            f.write('        Cp    '+cp+';\n')
            f.write('        Hf    '+hf+';\n')                       
        elif thermo=='hPolynomial':
            f.write('        CpCoeffs<'+num+'>    (')
            for ii in cpcoeff:f.write(ii+' ')
            f.write(');\n')
            f.write('        Hf    0;\n')
            f.write('        Sf    0;\n')
        f.write('    }\n')

        if simDict['densitymode']=='Polynomial':
            f.write('    equationOfState\n')
            f.write('    {\n')
            f.write('        rhoCoeffs<'+num+'>    (')
            for ii in rhocoeff:f.write(ii+' ')
            f.write(');\n')
            f.write('    }\n')
        elif simDict['densitymode']=='Constant':
            f.write('    equationOfState\n')
            f.write('    {\n')
            f.write('        rho    '+simDict['rho']+';\n')
            f.write('    }\n')
        f.write('}\n')
        f.close()          
    #------------------------------------------------------------------------------
    def operatingConditions(self,op):
        fcom=self.foamComment();head=self.header('constant','operatingConditions');hhh=fcom+head
        f=open(self.caseDir+'/constant/operatingConditions','w')
        for aaa in hhh:f.write(aaa)
        f.write('Op      Op [1 -1 -2 0 0 0 0]    '+op+';\n')
        f.close()   


