#-*-coding:utf8-*-
#############################################################################
#    Baram : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of Baram.                                            
#
#    Baram is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    Baram is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Baram.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for change dictionary of boundary file
#
# Author
#    NEXTFOAM Inc. 
##############################################################################
import math,os,glob,sys

from generalSub import *

class changeDictionary:

    def __init__(self,caseDir,solver):
        self.caseDir=caseDir
        self.solver=solver
    #------------------------------------------------------------------------------
    def changeInternalField(self,internalDict,notebook,textbuffer,operatingPressure):
    
        GEN=generalClass(self.caseDir)

        def change(field,inivalue):
            f=open(self.caseDir+'/0/'+field,'r');ff=f.readlines();f.close()
            for i in range(len(ff)):
                if ff[i][:13]=='internalField':
                    ff[i]='internalField    uniform '+inivalue+'; \n'
            g=open(self.caseDir+'/0/'+field,'w')
            for ii in ff:g.write(ii)
            g.close()
            
        abspressure=float(internalDict['pini'])+float(operatingPressure)
        
        if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
            fields=['U','p','k','epsilon','omega']
            inivalues=['('+internalDict['uxini']+' '+internalDict['uyini']+' '+internalDict['uzini']+')',internalDict['pini'],
                        internalDict['kini'],internalDict['eini'],internalDict['eini']]            
        elif self.solver=='buoyantSimpleNFoam' or self.solver=='buoyantPimpleNFoam':
            fields=['U','p','p_rgh','k','epsilon','omega','T']
            inivalues=['('+internalDict['uxini']+' '+internalDict['uyini']+' '+internalDict['uzini']+')',str(operatingPressure),internalDict['pini'],
                        internalDict['kini'],internalDict['eini'],internalDict['eini'],internalDict['Tini']]
                    
        for i in range(len(fields)):
            change(fields[i],inivalues[i])
            
        message='\n    - Internal Fields are modified \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0)
    #------------------------------------------------------------------------------
    def UFile(self,bcDictList,startFrom,startTime,nCore,initialDict,rho,notebook,textbuffer):

        uini='('+initialDict['uxini']+' '+initialDict['uyini']+' '+initialDict['uzini']+')'
        if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
            density=rho
        else:
            tini=initialDict['Tini'];pini=initialDict['pini']
            density=str(float(pini)/(287*float(tini)))

        GEN=generalClass(self.caseDir)
        results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if len(serialResult)==0:opt=' '
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if len(parallelResult)==0:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if len(serialResult)==0:opt=' '
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];Utype=dic['Utype']
            if patchtype=='wall' or patchtype=='patch' or patchtype=='mappedWall':
                if Utype=='no-slip':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+" -entry value -value '(0 0 0)'"+opt)
                elif Utype=='fixed-U':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value fixedValue'+opt)
                    valuestr="'("+dic['Ux']+' '+dic['Uy']+' '+dic['Uz']+")'"
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+valuestr+opt)
                elif Utype=='surfaceNormal-U':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value surfaceNormalFixedValue'+opt)
                    if float(dic['Umag']) > 0 :refv='-'+dic['Umag']
                    else:
                        aa=float(dic['Umag'])*-1;refv='-'+str(aa)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry refValue -value '+refv+opt)
                elif Utype=='volume-flowrate':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value flowRateInletVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry volumetricFlowRate -value '+dic['volflowrate']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+"'"+uini+"'"+opt)
                elif Utype=='mass-flowrate':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value flowRateInletVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry massFlowRate -value '+dic['massflowrate']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry rho -value rho'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry rhoInlet -value '+density+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+"'"+uini+"'"+opt)
                elif Utype=='pressureInletOutletVelocity':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value pressureInletOutletVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+"'"+uini+"'"+opt)
                elif Utype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value zeroGradient'+opt)
                elif Utype=='movingWallVelocity':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value movingWallVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+" -entry value -value '(0 0 0)'"+opt)
                elif Utype=='rotatingWallVelocity':
                    if dic['axis']=='x':axis='(1 0 0)'
                    elif dic['axis']=='y':axis='(0 1 0)'
                    elif dic['axis']=='z':axis='(0 0 1)'
                    elif dic['axis']=='-x':axis='(-1 0 0)'
                    elif dic['axis']=='-y':axis='(0 -1 0)'
                    elif dic['axis']=='-z':axis='(0 0 -1)'
                    origin='('+dic['orix']+' '+dic['oriy']+' '+dic['oriz']+')'
                    omega=float(dic['rpm'])*2*3.141592/60;omega=str(omega)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value rotatingWallVelocity'+opt)                  
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry origin -value '+"'"+origin+"'"+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry axis -value '+"'"+axis+"'"+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry omega -value '+omega+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+" -entry value -value '(0 0 0)'"+opt)
        if opt==' ':opt='0'

        message='\n    - U Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0)
    #------------------------------------------------------------------------------
    def pFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        pini=initialDict['pini']

        GEN=generalClass(self.caseDir)
        results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if len(serialResult)==0:opt=' '
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if len(parallelResult)==0:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if len(serialResult)==0:opt=' '
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];ptype=dic['ptype']
            if patchtype=='wall' or patchtype=='patch':
                if ptype=='totalPressure':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value totalPressure'+opt)                  
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry p0 -value '+dic['ptotal']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry value -value '+dic['ptotal']+opt)                    
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry gamma -value 1.4'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry rho -value none'+opt)                     
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry psi -value none'+opt)                                         
                elif ptype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry value -value '+dic['pstatic']+opt)
                elif ptype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value zeroGradient'+opt)
                elif ptype=='fixedFluxPressure':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value fixedFluxPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry rho -value rhok'+opt)                    
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value fixedFluxPressure'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry value -value '+pini+opt)
            elif patchtype=='cyclic':
                if ptype=='porousJump':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value porousJumpPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry patchType -value cyclic'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry value -value 0'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry I -value '+dic['jumpi']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry length -value '+dic['jumpl']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry D -value '+dic['jumpd']+opt)
            	else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value cyclic'+opt)

        if opt==' ':opt='0'

        message='\n    - p Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0)                                
    #------------------------------------------------------------------------------
    def p_rghFileThermo(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        pini=initialDict['pini']

        GEN=generalClass(self.caseDir)
        results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if len(serialResult)==0:opt=' '
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if len(parallelResult)==0:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if len(serialResult)==0:opt=' '
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];ptype=dic['ptype']
            if patchtype=='wall' or patchtype=='patch':
                if ptype=='totalPressure':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value totalPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry p0 -value '+dic['ptotal']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry value -value '+dic['ptotal']+opt) 
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry gamma -value 1.4'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry rho -value rho'+opt)                    
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry psi -value none'+opt)                     
                elif ptype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry value -value '+dic['pstatic']+opt)
                elif ptype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value zeroGradient'+opt)
                elif ptype=='fixedFluxPressure':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value fixedFluxPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry rho -value rhok'+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value fixedFluxPressure'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry value -value '+pini+opt)
            elif patchtype=='cyclic':
                if ptype=='porousJump':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value porousJumpPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry patchType -value cyclic'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry value -value 0'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry I -value '+dic['jumpi']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry length -value '+dic['jumpl']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry D -value '+dic['jumpd']+opt)
            	else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value cyclic'+opt)

        if opt==' ':opt='0'

        message='\n    - p_rgh Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0)          
    #------------------------------------------------------------------------------
    def TFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        Tini=initialDict['Tini']

        GEN=generalClass(self.caseDir)
        results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if len(serialResult)==0:opt=' '
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if len(parallelResult)==0:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if len(serialResult)==0:opt=' '
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];Ttype=dic['Ttype']
            if patchtype=='wall' or patchtype=='patch':
                if Ttype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry type -value zeroGradient'+opt)
                elif Ttype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry value -value '+dic['Tvalue']+opt)
                elif Ttype=='inletOutlet':
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry type -value inletOutlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry inletValue -value '+dic['Tinout']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry value -value '+Tini+opt)
                elif Ttype=='heatflux':
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry type -value externalWallHeatFluxTemperature'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry kappa -value fluidThermo'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry q -value '+dic['heatflux']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry kappaName -value none'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry value -value '+Tini+opt)
                elif Ttype=='convection':
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry type -value externalWallHeatFluxTemperature'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry kappa -value fluidThermo'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry Ta -value '+dic['Tinf']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry h -value '+dic['htcoeff']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry kappaName -value none'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry value -value '+Tini+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry type -value compressible::turbulentTemperatureCoupledBaffleMixed'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry Tnbr -value T'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry kappa -value fluidThermo'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry kappaName -value none'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field T -patch '+name+' -entry value -value '+Tini+opt)

        if opt==' ':opt='0'

        message='\n    - T Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0) 
    #------------------------------------------------------------------------------
    def kFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        kini=initialDict['kini']

        GEN=generalClass(self.caseDir)
        results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if len(serialResult)==0:opt=' '
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if len(parallelResult)==0:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if len(serialResult)==0:opt=' '
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];ktype=dic['ktype']
            if patchtype=='wall' or patchtype=='patch':
                if ktype=='wallFunction':
                    if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                        os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value kqRWallFunction'+opt)
                    else:
                        os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value compressible::kqRWallFunction'+opt)                    
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)
                elif ktype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+dic['kvalue']+opt)
                elif ktype=='intensity':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value turbulentIntensityKineticEnergyInlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry intensity -value '+dic['kintensity']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)
                elif ktype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value zeroGradient'+opt)
                elif ktype=='inletOutlet':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value inletOutlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry inletValue -value '+dic['kinout']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value compressible::kqRWallFunction'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)

        if opt==' ':opt='0'

        message='\n    - k Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0) 
    #------------------------------------------------------------------------------
    def epsilonFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        eini=initialDict['eini']

        GEN=generalClass(self.caseDir)
        results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if len(serialResult)==0:opt=' '
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if len(parallelResult)==0:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if len(serialResult)==0:opt=' '
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];etype=dic['etype']         
            if patchtype=='wall' or patchtype=='patch':
                if etype=='wallFunction':
                    if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                        os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value epsilonWallFunction'+opt)
                    else:
                        os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value compressible::epsilonWallFunction'+opt)                    
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)
                elif etype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+dic['evalue']+opt)
                elif etype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value zeroGradient'+opt)
                elif etype=='inletOutlet':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value inletOutlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry inletValue -value '+dic['einout']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)
                elif etype=='mixingLength':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value compressible::turbulentMixingLengthDissipationRateInlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry mixingLength -value '+dic['elength']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)                    
            elif patchtype=='mappedWall':
                if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value epsilonWallFunction'+opt)
                else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value compressible::epsilonWallFunction'+opt)                
                os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)

        if opt==' ':opt='0'

        message='\n    - epsilon Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0) 
    #------------------------------------------------------------------------------
    def omegaFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        eini=initialDict['eini']

        GEN=generalClass(self.caseDir)
        results=GEN.getResult();serialResult=results[0];parallelResult=results[1]
        
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if len(serialResult)==0:opt=' '
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if len(parallelResult)==0:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if len(serialResult)==0:opt=' '
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];etype=dic['etype']         
            if patchtype=='wall' or patchtype=='patch':
                if etype=='wallFunction':
                    if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                        os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value omegaWallFunction'+opt)
                    else:
                        os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value compressible::omegaWallFunction'+opt)                    
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+eini+opt)
                elif etype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+dic['evalue']+opt)
                elif etype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value zeroGradient'+opt)
                elif etype=='inletOutlet':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value inletOutlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry inletValue -value '+dic['einout']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+eini+opt)
                elif etype=='mixingLength':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value compressible::turbulentMixingLengthDissipationRateInlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry mixingLength -value '+dic['elength']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+eini+opt)                    
            elif patchtype=='mappedWall':
                if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value omegaWallFunction'+opt)
                else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value compressible::omegaWallFunction'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+eini+opt)

        if opt==' ':opt='0'

        message='\n    - omega Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0)        
    #------------------------------------------------------------------------------


