#-*-coding:utf8-*-
#############################################################################
#    DOPEN : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of DOPEN.                                            
#
#    DOPEN is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    DOPEN is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with DOPEN.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for writing various files 
#
# Author
#    NEXTFOAM Inc. 
##############################################################################

import math,os,glob,sys

from generalSub import *

class writeFileClass:

    def __init__(self,caseDir):
        self.caseDir=caseDir
	
    def mrfFile(self,dic):
        cellZone=dic['cellZoneName']
        aa=dic['nonRotPatchList'];nonRotPatch=aa.split(' ')
        orix=dic['orix'];oriy=dic['oriy'];oriz=dic['oriz']
        rpm=dic['rpm'];rotAxis=dic['axis']
	    
        if rotAxis=='x':axis='(1 0 0)'
        elif rotAxis=='y':axis='(0 1 0)'
        elif rotAxis=='z':axis='(0 0 1)'
        elif rotAxis=='-x':axis='(-1 0 0)'
        elif rotAxis=='-y':axis='(0 -1 0)'
        elif rotAxis=='-z':axis='(0 0 -1)'
        
        omega=float(rpm)*2*3.141592/60
	    
        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':MRF','w')
        f.write('MRFCellZone_'+cellZone+'\n')
        f.write('{\n')
        f.write('    type            MRFSource;\n')
        f.write('    active          yes;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    MRFSourceCoeffs\n')
        f.write('    {\n')
        f.write('        nonRotatingPatches    (')
        for jj in nonRotPatch:f.write(' '+jj+' ')
        f.write(' );\n')
        f.write('        origin     ('+orix+' '+oriy+' '+oriz+');\n')
        f.write('        axis       '+axis+';\n')
        f.write('        omega      '+str(omega)+';\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
        f.close()
    #-------------------------------------------------------------------------------------        
    def porousFile(self,dic):
    
        cellZone=dic['cellZoneName']
        potype=dic['porousType']    
        ax=dic['xDarcy'];ay=dic['yDarcy'];az=dic['zDarcy']
        bx=dic['xForch'];by=dic['yForch'];bz=dic['zForch']
        e1x=dic['e1x'];e1y=dic['e1y'];e1z=dic['e1z']
        e2x=dic['e2x'];e2y=dic['e2y'];e2z=dic['e2z']	
        c0=dic['powerC0'];c1=dic['powerC1']

        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':Porous','w')
        f.write('porous_'+cellZone+'\n')
        f.write('{\n')
        f.write('    type            explicitPorositySource;\n')
        f.write('    active          true;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    explicitPorositySourceCoeffs\n')
        f.write('    {\n')
        f.write('        type            '+potype+';\n')
        f.write('        active          yes;\n')
        f.write('\n')
        if potype=='DarcyForchheimer':
            f.write('        DarcyForchheimerCoeffs\n')
            f.write('        {\n')
            f.write('            d       d [0 -2 0 0 0 0 0] ('+ax+' '+ay+' '+az+');\n')
            f.write('            f       f [0 -1 0 0 0 0 0] ('+bx+' '+by+' '+bz+');\n')
            f.write('\n')
            f.write('            coordinateSystem\n')
            f.write('            {\n')
            f.write('                type    cartesian;\n')
            f.write('                origin  (0 0 0);\n')
            f.write('                coordinateRotation\n')
            f.write('                {\n')
            f.write('                    type    axesRotation;\n')
            f.write('                    e1  ('+e1x+' '+e1y+' '+e1z+');\n')
            f.write('                    e2  ('+e2x+' '+e2y+' '+e2z+');\n')
            f.write('                }\n')
            f.write('            }\n')
            f.write('        }\n')
        else:
            f.write('        powerLawCoeffs\n')
            f.write('        {\n')
            f.write('            C0      '+c0+';\n')
            f.write('            C1      '+c1+';\n')
            f.write('\n')
            f.write('            coordinateSystem\n')
            f.write('            {\n')
            f.write('                type    cartesian;\n')
            f.write('                origin  (0 0 0);\n')
            f.write('                coordinateRotation\n')
            f.write('                {\n')
            f.write('                    type    axesRotation;\n')
            f.write('                    e1  ('+e1x+' '+e1y+' '+e1z+');\n')
            f.write('                    e2  ('+e2x+' '+e2y+' '+e2z+');\n')
            f.write('                }\n')
            f.write('            }\n')
            f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')               
        f.close()       
    #-------------------------------------------------------------------------------------
    def momSourceFile(self,dic):
    
        cellZone=dic['cellZoneName']
        ux=dic['momSourceX'];uy=dic['momSourceY'];uz=dic['momSourceZ']

        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':Mom.Source','w')
        f.write('fixedU_'+cellZone+'\n')
        f.write('{\n')	
        f.write('    type            pressureGradientExplicitSource;\n')
        f.write('    active          yes;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    pressureGradientExplicitSourceCoeffs\n')
        f.write('    {\n')
        f.write('        fieldNames ( U );\n')
        f.write('        Ubar    ('+ux+' '+uy+' '+uz+');\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
        f.close()
    #-------------------------------------------------------------------------------------
    def fixedUFile(self,dic):
    
        cellZone=dic['cellZoneName']
        ux=dic['fixedUx'];uy=dic['fixedUy'];uz=dic['fixedUz']        
        
        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':Fixed.U','w')
        f.write('fixedU_'+cellZone+'\n')
        f.write('{\n')	
        f.write('    type            pressureGradientExplicitSource;\n')
        f.write('    active          yes;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    pressureGradientExplicitSourceCoeffs\n')
        f.write('    {\n')
        f.write('        fieldNames    (U);\n')
        f.write('        Ubar          ('+ux+' '+uy+' '+uz+');\n')
        f.write('        gradPini      gradPini [0 2 -2 0 0] 0;\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
        f.close()  
    #-------------------------------------------------------------------------------------
    def noneFile(self,dic):
    
        cellZone=dic['cellZoneName']        
       
        if dic['energySourceOnOff']=='true':
            f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':None','w')
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
            f.close()                                
    #-------------------------------------------------------------------------------------
    def foamComment(self):
        fc=[]
        fc.append('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        fc.append('| =========                 |                                                 |\n')
        fc.append('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
        fc.append('|  \\    /   O peration     | Version:  2.3.0                                 |\n')
        fc.append('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      |\n')
        fc.append('|    \\/     M anipulation  |                                                 |\n')
        fc.append('\*---------------------------------------------------------------------------*/\n')	
        return fc
    #------------------------------------------------------------------------------
    def header(self,location,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       dictionary;\n')
        he.append('    location    "'+location+'";\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def BCheader(self,classtype,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       '+classtype+';\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def topoSetDict(self,sets):
        fcom=self.foamComment();head=self.header('system','topoSetDict');hhh=fcom+head
        f=open(self.caseDir+'/system/topoSetDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('actions\n')
        f.write('(\n')
        for ss in sets:
            f.write('    {\n')
            f.write('        name    '+ss+';\n')
            f.write('        type    faceZoneSet;\n')
            f.write('        action  new;\n')
            f.write('        source  setToFaceZone;\n')
            f.write('        sourceInfo\n')
            f.write('        {\n')
            f.write('            faceSet '+ss+';\n')
            f.write('        }\n')
            f.write('    }\n')
        f.write(');\n')
        f.close()
    #------------------------------------------------------------------------------
    def createBafflesDict(self,sets):
        fcom=self.foamComment()
        head=self.header('system','createBafflesDict')
        hhh=fcom+head
        f=open(self.caseDir+'/system/createBafflesDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('internalFacesOnly true;\n')
        f.write('baffles\n')
        f.write('{\n')
        for ss in sets:                  
            f.write('    '+ss+'\n')
            f.write('    {\n')
            f.write('        type        faceZone;\n')
            f.write('        zoneName    '+ss+';\n')
            f.write('        patchPairs\n')
            f.write('        {\n')
            f.write('            type	mappedWall;\n')
            f.write('            sampleMode	nearestPatchFace;\n')
            f.write('	     }\n')
            f.write('    }\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def createInteriorDict(self,sets):
        fcom=self.foamComment()
        head=self.header('system','createBafflesDict')
        hhh=fcom+head
        f=open(self.caseDir+'/system/createBafflesDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('internalFacesOnly true;\n')
        f.write('baffles\n')
        f.write('{\n')
        for ss in sets:                  
            f.write('    '+ss+'\n')
            f.write('    {\n')
            f.write('        type        faceZone;\n')
            f.write('        zoneName    '+ss+';\n')
            f.write('        patchPairs\n')
            f.write('        {\n')
            f.write('            type        cyclic;\n')
            f.write('	     }\n')
            f.write('    }\n')
        f.write('}\n')
        f.close()                   
    #------------------------------------------------------------------------------
    def changeDictionaryDictNoCyclicAMI(self,patch,newtype):
        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDict')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        if newtype!='cyclicAMI':
            f.write('        '+patch+'\n')
            f.write('        {\n')
            f.write('            type    '+newtype+';\n')
            f.write('        }\n')			
        f.write('    }\n')
        f.write('}\n')
        f.close() 
    #------------------------------------------------------------------------------
    def changeDictionaryDictInterface(self,selectedFaces,weight,weightValue):        
        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDictInterface')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictInterface','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')

        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+selectedFaces[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            inGroups          1(cyclicAMI);\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         noOrdering;\n')
        f.write('            neighbourPatch    '+selectedFaces[1]+';\n')
        if weight==True:f.write('            lowWeightCorrection    '+weightValue+';\n')
        f.write('        }\n')
        f.write('        '+selectedFaces[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            inGroups          1(cyclicAMI);\n')        
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         noOrdering;\n')
        f.write('            neighbourPatch    '+selectedFaces[0]+';\n')
        if weight==True:f.write('            lowWeightCorrection    '+weightValue+';\n')
        f.write('        }\n')			
        f.write('    }\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def changeDictionaryDictRotCyclic(self,sets,cc1,cc2,cc3,axis):          
        
        if axis=='x':raxis='(1 0 0)'
        elif axis=='y':raxis='(0 1 0)'
        elif axis=='z':raxis='(0 0 1)'

        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDictRotCyclic')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictRotCyclic','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')     
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+sets[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         rotational;\n')
        f.write('            neighbourPatch    '+sets[1]+';\n')
        f.write('            rotationAxis      '+raxis+';\n')
        f.write('            rotationCentre    ('+cc1+' '+cc2+' '+cc3+');\n')
        f.write('        }\n')
        f.write('        '+sets[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         rotational;\n')
        f.write('            neighbourPatch    '+sets[0]+';\n')
        f.write('            rotationAxis      '+raxis+';\n')
        f.write('            rotationCentre    ('+cc1+' '+cc2+' '+cc3+');\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        f.close()                      
    #------------------------------------------------------------------------------
    def changeDictionaryDictTranslateCyclic(self,sets,cc1,cc2,cc3):         

        vector1='('+cc1+' '+cc2+' '+cc3+')'
        xx=-1*float(cc1);yy=-1*float(cc2);zz=-1*float(cc3)
        vector2='('+str(xx)+' '+str(yy)+' '+str(zz)+')'
	
        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDictTranslateCyclic')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictTranslateCyclic','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')          
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+sets[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         translational;\n')
        f.write('            neighbourPatch    '+sets[1]+';\n')
        f.write('            separationVector  '+vector1+';\n')
        f.write('        }\n')
        f.write('        '+sets[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         translational;\n')
        f.write('            neighbourPatch    '+sets[0]+';\n')
        f.write('            separationVector  '+vector2+';\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        f.close()          
    #------------------------------------------------------------------------------
    def viewFactorsDict(self,bcname,dic):         
	
        fcom=self.foamComment()
        head=self.BCheader('dictionary','viewFactorsDict')
        hhh=fcom+head
        f=open(self.caseDir+'/constant/viewFactorsDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')
        f.write('writeViewFactorMatrix     true;\n')
        f.write('writeFacesAgglomeration   true;\n')
        f.write('writePatchViewFactors     false;\n')
        for ii in bcname:
            f.write(ii+'\n')
            f.write('{\n')
            f.write('    nFacesInCoarsestLevel    '+dic['nFace:'+ii]+';\n')
            f.write('    featureAngle             '+dic['feature:'+ii]+';\n')
            f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def radiationProperties(self,dic,installpath):

        GEN=generalClass(self.caseDir)
        
        fcom=self.foamComment();head=self.header('constant','radiationProperties');hhh=fcom+head        
        if dic['radOnOff']=='true':
            f=open(self.caseDir+'/constant/radiationProperties','w')
            for aaa in hhh:f.write(aaa)
            f.write('radiation    on;\n')
            f.write('radiationModel    '+dic['radModel']+';\n')
            f.write('solverFreq        '+dic['radFreq']+';\n')
            
            if dic['radModel']=='fvDOM':
                f.write('fvDOMCoeffs\n')
                f.write('{\n')
                f.write('    nPhi        '+dic['radDONPhi']+';\n')
                f.write('    nTheta      '+dic['radDONTheta']+';\n')
                f.write('    convergence '+dic['radDOConv']+';\n')
                f.write('    maxIter     '+dic['radDOMaxiter']+';\n')
                f.write('    cacheDiv    '+dic['radDOCache']+';\n')
                f.write('}\n')
            elif dic['radModel']=='viewFactor':
                f.write('viewFactorCoeffs\n')
                f.write('{\n')
                f.write('    smoothing   false;\n')
                f.write('    constantEmissivity    true;\n')
                f.write('}\n')                
            
            # A-E Model            
            if dic['radAEModel']=='constant':
                f.write('absorptionEmissionModel        constantAbsorptionEmission;\n')
                f.write('constantAbsorptionEmissionCoeffs\n')
                f.write('{\n')
                f.write('    absorptivity    absorptivity    [ m^-1 ]         '+dic['radAEConstAbs']+';\n')
                f.write('    emissivity      emissivity      [ m^-1 ]         '+dic['radAEConstEmi']+';\n')
                f.write('    E               E               [ kg m^-1 s^-3 ] '+dic['radAEConstE']+';\n')
                f.write('}\n')
                
            elif dic['radAEModel']=='greyMean':
                self.greyMeanAbsorptionEmission(installpath)###
                f.write('absorptionEmissionModel        greyMeanAbsorptionEmission;\n')
                f.write('greyMeanAbsorptionEmissionCoeffs\n')
                f.write('{\n')
                f.write('    lookUpTableFileName    "'+self.caseDir+'/constant/greyMeanLookupTable";\n')
                f.write('    EhrrCoeff       '+dic['radAEGreyEhrr']+';\n')
                f.write('}\n')
                
            elif dic['radAEModel']=='binary':
                aa=glob.glob(self.caseDir+'/system/settings/binaryAbsorptionCondition')
                if len(aa)==0:
                    GEN.makeDialog('Error! Binary Absorption Emission is not set properly')
                else:
                    bidic=GEN.makeDictFromFile(aa[0])
                f.write('absorptionEmissionModel        binaryAbsorptionEmission;\n') 
                f.write('binaryAbsorptionEmissionCoeffs\n')
                f.write('{\n')
                f.write('    model1;\n')
                f.write('    {\n')
                f.write('        absorptionEmissionModel    '+bidic['model1']+';\n')
                if bidic['model1']=='constant':
                    f.write('        constantAbsorptionEmissionCoeffs\n')
                    f.write('        {\n')
                    f.write('            absorptivity    absorptivity    [ m^-1 ]         '+bidic['model1constAb']+';\n')
                    f.write('            emissivity      emissivity      [ m^-1 ]         '+bidic['model1constEm']+';\n')
                    f.write('            E               E               [ kg m^-1 s^-3 ] '+bidic['model1constE']+';\n')
                    f.write('        {\n')
                elif bidic['model1']=='greyMean':
                    f.write('        greyMeanAbsorptionEmissionCoeffs\n')
                    f.write('        {\n')
                    f.write('            lookUpTableFileName    "'+self.caseDir+'/constant/greyMeanLookupTable";\n')
                    f.write('            EhrrCoeff       '+bidic['model1greyEhrr']+';\n')
                    f.write('        {\n') 
                elif bidic['model1']=='cloud':
                    cc=glob.glob(self.caseDir+'/system/settings/cloudCoal-absorption1')
                    if len(cc)==0:
                        GEN.makeDialog('Error! Cloud name for model1 is not set properly')
                        return
                    else:
                        g=open(cc[0],'r');gg=g.readlines();g.close()                        
                    f.write('        cloudAbsorptionEmissionCoeffs\n')
                    f.write('        {\n')
                    for jj in gg:f.write('            '+gg[:-1]+';\n')
                    f.write('        }\n') 
                                  
                f.write('    model2;\n')
                f.write('    {\n')
                f.write('        absorptionEmissionModel    '+bidic['model2']+';\n')
                if bidic['model2']=='constant':
                    f.write('        constantAbsorptionEmissionCoeffs\n')
                    f.write('        {\n')
                    f.write('            absorptivity    absorptivity    [ m^-1 ]         '+bidic['model2constAb']+';\n')
                    f.write('            emissivity      emissivity      [ m^-1 ]         '+bidic['model2constEm']+';\n')
                    f.write('            E               E               [ kg m^-1 s^-3 ] '+bidic['model2constE']+';\n')
                    f.write('        {\n')
                elif bidic['model2']=='greyMean':
                    f.write('        greyMeanAbsorptionEmissionCoeffs\n')
                    f.write('        {\n')
                    f.write('            lookUpTableFileName    "'+self.caseDir+'/constant/greyMeanLookupTable";\n')
                    f.write('            EhrrCoeff       '+bidic['model2greyEhrr']+';\n')
                    f.write('        {\n') 
                elif bidic['model2']=='cloud':
                    cc=glob.glob(self.caseDir+'/system/settings/cloudCoal-absorption2')
                    if len(cc)==0:
                        GEN.makeDialog('Error! Cloud name for model2 is not set properly')
                        return
                    else:
                        g=open(cc[0],'r');gg=g.readlines();g.close()                        
                        f.write('        cloudAbsorptionEmissionCoeffs\n')
                        f.write('        {\n')
                        for jj in gg:f.write('            '+jj[:-1]+';\n')
                        f.write('        }\n')
                f.write('}\n')                      
                    
            # scattering           
            f.write('scatterModel        '+dic['radScatterModel']+';\n')
            if dic['radScatterModel']=='cloud':
                cc=glob.glob(self.caseDir+'/system/settings/cloudCoal-scattering')
                if len(cc)==0:
                    GEN.makeDialog('Error! Cloud name for scattering is not set properly')
                    return
                else:
                    g=open(cc[0],'r');gg=g.readlines();g.close()
                    f.write('    cloudScatterCoeffs\n')
                    f.write('    {\n')
                    for jj in gg:f.write('        '+jj[:-1]+';\n')
                    f.write('    }\n')            
            
            f.write('sootModel         none;\n')
            f.close()
    #------------------------------------------------------------------------------
    def greyMeanAbsorptionEmission(self,installpath):
    
        GEN=generalClass(self.caseDir)
        speciesDBList=os.listdir(installpath+'/speciesPropertiesDB')
        f=open(self.caseDir+'/constant/greyMeanLookupTable','w')
        for ii in speciesDBList:
            dic=GEN.makeDictFromFile(installpath+'/speciesPropertiesDB/'+ii)
            f.write(ii+'\n')
            f.write('    {\n')
            f.write('        Tcommon         '+dic['radTcommon']+';\n')
            f.write('        invTemp         true;\n')
            f.write('        Tlow            '+dic['radTlow']+';\n')
            f.write('        Thigh           '+dic['radThigh']+';\n')
            low=dic['loTCoeffsList'].split(' ')
            high=dic['hiTCoeffsList'].split(' ')
            f.write('        loTcoeffs\n')
            f.write('        (\n')
            f.write('            '+low[0]+'\n')
            f.write('            '+low[1]+'\n')
            f.write('            '+low[2]+'\n')
            f.write('            '+low[3]+'\n')
            f.write('            '+low[4]+'\n')
            f.write('            '+low[5]+'\n')
            f.write('        );\n')
            f.write('        hiTcoeffs\n')
            f.write('        (\n')
            f.write('            '+high[0]+'\n')
            f.write('            '+high[1]+'\n')
            f.write('            '+high[2]+'\n')
            f.write('            '+high[3]+'\n')
            f.write('            '+high[4]+'\n')
            f.write('            '+high[5]+'\n')
            f.write('        );\n')
            f.write('    }\n')
            f.close()
        
        
        
        

               
        
        
        
