#-*-coding:utf8-*-
#############################################################################
#    Baram : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of Baram.                                            
#
#    Baram is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    Baram is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Baram.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for boundary conditions
#
# Author
#    NEXTFOAM Inc. 
##############################################################################

import os,math,sys;import pygtk;pygtk.require('2.0');import gtk;import glob

from generalSub import *
from boundaryConditionSub import *
from writeFileSub import *

class freepanelBoundaryConditionClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir,mainwindow,solver,installpath,rr,gg,bb):
        self.caseDir=caseDir
        self.mainwindow=mainwindow
        self.solver=solver
        self.installpath=installpath
        self.rr=rr;self.gg=gg;self.bb=bb

    def boundaryCondition(self,gvtk,notebook,textbuffer):
    
        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)

        def copyBC(widget):self.copyBoundaryCondition(bcitembox,self.boundarybox,gvtk)

        self.boundarybox=gtk.VBox(False,5)
        label=gtk.Label();label.set_markup('<b>  Boundary Conditions  </b>')
        ebox=gtk.EventBox();self.boundarybox.pack_start(ebox,True,True,0)
        label.set_alignment(0,0.5);label.set_size_request(50,45)
        ebox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.color_parse('white'));ebox.add(label)

        #separator=gtk.HSeparator();separator.set_size_request(285,5);self.boundarybox.pack_start(separator,False,True,10)

        hbox=gtk.HBox(False,5);self.boundarybox.pack_start(hbox,False,False,5)
        bcswin=gtk.ScrolledWindow();bcswin.set_border_width(0);bcswin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        hbox.pack_start(bcswin,True,True,0);bcitembox=gtk.VBox(False,0);bcswin.add_with_viewport(bcitembox)
        bcswin.set_size_request(200,680)
        
        self.copybcB=gtk.Button('Copy Boundary Condition');self.copybcB.connect('clicked',copyBC);self.copybcB.set_size_request(140,28)
        hbox=gtk.HBox(False,5);self.boundarybox.pack_start(hbox,False,False,0)       
        hbox.pack_end(self.copybcB,True,True,5)        
            
        self.makeBCBox(bcitembox,self.boundarybox,gvtk,notebook,textbuffer)

        return self.boundarybox
    #---------------------------------------------------------------------------------------------------------        
    def makeBCBox(self,bcitembox,boundarybox,gvtk,notebook,textbuffer):

        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)
        
        if glob.glob(self.caseDir+'/constant/polyMesh/boundary')==[]:
            GEN.makeDialog('Error!!! There might be some problem with mesh')
            return
            
        bcnameList=GEN.getBCName();bctypeList=GEN.getBCType()

        def detail(widget,path,column):
            model=self.treeview.get_model()
            iter=model.get_iter(path[0])
            namepatch=model.get_value(iter,0)
            typepatch=model.get_value(iter,1) 
            
            if typepatch=='wall' or typepatch=='patch':
                BCS=boundaryConditionClass(self.caseDir,self.solver,self.installpath,self.rr,self.gg,self.bb)
                BCS.openBCWindow(namepatch,typepatch,gvtk,self.mainwindow,notebook,textbuffer,self.liststore_patchname)            
                f=open(self.caseDir+'/system/settings/openSubWindow','w');f.close()
            else:GEN.makeDialog('There is nothing to set with '+typepatch+' patch')

        def changebctype(widget, path, text):        
            bcnameList=GEN.getBCName();bctypeList=GEN.getBCType()
            
            if bctypeList[int(path)]!=text:
                self.liststore_patchname[path][1] = text
                patch=bcnameList[int(path)]
                newtype=text
                curtype=bctypeList[int(path)]
                if newtype=='cyclic' or newtype=='mappedWall' or newtype=='empty':
                    GEN.makeDialog('B.C. type cannot be changed to '+newtype)
                    self.liststore_patchname[path][1] = curtype
                else:            
                    self.changepatchtype(bcnameList,bctypeList,patch,newtype,curtype,bcitembox,boundarybox,gvtk,notebook,textbuffer)

        self.liststore_types = gtk.ListStore(str)
        self.btypeList=['wall','patch','symmetry','symmetryPlane','empty','cyclicAMI','mappedWall']
        for ii in self.btypeList:self.liststore_types.append([ii])

        self.liststore_patchname = gtk.ListStore(str, str)
        for i in range(len(bcnameList)): 
            self.liststore_patchname.append([bcnameList[i],bctypeList[i]])

        self.treeview = gtk.TreeView(model=self.liststore_patchname)       
        self.treeview.connect('row-activated',detail)

        renderer_text = gtk.CellRendererText()
        
        column_text = gtk.TreeViewColumn("patch name                ", renderer_text, text=0)
        self.treeview.append_column(column_text)

        renderer_combo = gtk.CellRendererCombo()
        renderer_combo.set_property("editable", True)
        renderer_combo.set_property("model", self.liststore_types)
        renderer_combo.set_property("text-column", 0)
        renderer_combo.set_property("has-entry", False)
        renderer_combo.connect("edited", changebctype)

        column_combo = gtk.TreeViewColumn("type", renderer_combo, text=1)

        column_text.set_sort_column_id(0)
        column_combo.set_sort_column_id(1)

        self.treeview.append_column(column_combo)
        
        bcitembox.pack_start(self.treeview,True,True,2)  
    #--------------------------------------------------------------------------------------------------------- 
    def changepatchtype(self,bcnameList,bctypeList,patch,newtype,curtype,bcitembox,boundarybox,gvtk,notebook,textbuffer):
        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)
        if newtype=='cyclicAMI':                            
            if curtype!='cyclicAMI':self.selectAMIType(bcnameList,bctypeList,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer)
        else:
            if curtype!='cyclicAMI':
                WFILE.changeDictionaryDictNoCyclicAMI(patch,newtype)
                os.system('changeDictionary -disablePatchGroups -case '+self.caseDir+' > '+self.caseDir+'/logfiles/log.changeDictionary')
                message='\n [  Patch type of "'+patch+'" is changed to '+newtype+'  ]\n\n'                     
                enditer=textbuffer.get_end_iter()
                textbuffer.insert(enditer,message)
                
            else:self.changeAMIToOthers(patch,newtype)

        aa=glob.glob(self.caseDir+'/system/settings/BCDict:'+patch)
        if len(aa)!=0:
            for ii in aa:os.system('rm '+ii)
                            
        self.refresh(bcitembox,boundarybox,gvtk,notebook,textbuffer)
    #--------------------------------------------------------------------------------------------------------- 
    def changepatchname(self,oldname,newname,bcitembox,boundarybox,gvtk,notebook):
        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)

        f=open(self.caseDir+'/constant/polyMesh/boundary','r');ff=f.readlines();f.close()
        for i in range(len(ff)):
            if ff[i][4:-1]==oldname:ff[i]='    '+newname+'\n'
        g=open(self.caseDir+'/constant/polyMesh/boundary','w')
        for ii in ff:g.write(ii)
        g.close()
        
        os.system('mv '+self.caseDir+'/VTK/'+oldname+' '+self.caseDir+'/VTK/'+newname)
        os.system('mv '+self.caseDir+'/VTK/'+newname+'/'+oldname+'_0.vtk '+self.caseDir+'/VTK/'+newname+'/'+newname+'_0.vtk')

        aa=glob.glob(self.caseDir+'/system/settings/BCDict:'+oldname)
        if len(aa)!=0:
            for ii in aa:os.system('rm '+ii)
    #---------------------------------------------------------------------------------------------------------                        
    def refresh(self,bcitembox,boundarybox,gvtk,notebook,textbuffer):
        aa=bcitembox.get_children()
        for ii in aa:bcitembox.remove(ii)
        self.makeBCBox(bcitembox,boundarybox,gvtk,notebook,textbuffer)
        boundarybox.show_all()                          
    #---------------------------------------------------------------------------------------------------------
    def selectAMIType(self,bcname,bctype,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer):    
   
        def applythis(widget):
            if interfaceRadio.get_active()==1:self.createInterface(bcname,bctype,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer)
            elif rotcyclicRadio.get_active()==1:self.createCyclicAMIRotation(bcname,bctype,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer)
            elif transcyclicRadio.get_active()==1:self.createCyclicAMITranslate(bcname,bctype,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer)                     
            cwin.destroy()            
        
        def closethis(widget):cwin.destroy()

        cwin=gtk.Window(gtk.WINDOW_TOPLEVEL);cwin.set_title('Select AMI Type')
        cwin.set_border_width(5);cwin.set_position(gtk.WIN_POS_CENTER);cwin.set_transient_for(self.mainwindow)
        cmainbox=gtk.VBox(False,0);cwin.add(cmainbox)

        label=gtk.Label();label.set_markup('<b>Select cyclicAMI Type   </b>')
        hbox=gtk.HBox(False,5);cmainbox.pack_start(hbox,False,False,5);hbox.pack_start(label,False,False,15)            
        
        cscwin=gtk.ScrolledWindow();cscwin.set_border_width(0);cscwin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        cscwin.set_size_request(250,200)
        hbox=gtk.HBox(False,5);cmainbox.pack_start(hbox,False,False,5);hbox.pack_start(cscwin,True,True,0)
        vbox=gtk.VBox(False,0);cscwin.add_with_viewport(vbox)

        interfaceRadio=gtk.RadioButton(None,'non-conformal interface');vbox.pack_start(interfaceRadio,False,False,5)
        rotcyclicRadio=gtk.RadioButton(interfaceRadio,'rotational cyclic');vbox.pack_start(rotcyclicRadio,False,False,5)
        transcyclicRadio=gtk.RadioButton(interfaceRadio,'translational cyclic');vbox.pack_start(transcyclicRadio,False,False,5)
        interfaceRadio.set_active(1)
      
        appB=gtk.Button('Apply');appB.set_size_request(100,28);appB.connect('clicked',applythis)
        clsB=gtk.Button('Close');clsB.set_size_request(100,28);clsB.connect('clicked',closethis)
        hbox=gtk.HBox(False,5);cmainbox.pack_start(hbox,False,False,5)
        hbox.pack_end(clsB,False,False,5);hbox.pack_end(appB,False,False,5)        
        
        cwin.show_all()        
    #---------------------------------------------------------------------------------------------------------
    def createInterface(self,bcname,bctype,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer):
    
        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)
            
        def closethis(widget,data=None):window.destroy()

        def makeInterface(widget,data=None):
            weight=checkWeight.get_active()
            weightValue=weightEntry.get_text()
            selectedFaces=[];tf=[]
            for i in range(len(bcname)):
                aa=setName[i].get_active()		   
                if aa==True:selectedFaces.append(bcname[i])
            selectedFaces.append(patch)		
            WFILE.changeDictionaryDictInterface(selectedFaces,weight,weightValue)
            os.system('changeDictionary -case '+self.caseDir+' -disablePatchGroups -dict '+self.caseDir+'/system/changeDictionaryDictInterface'+' > '+self.caseDir+'/logfiles/log.changeDictionary')
            message='\n [  "'+selectedFaces[0]+'" and "'+selectedFaces[1]+'" are made to Interface  ]\n\n'                     
            enditer=textbuffer.get_end_iter()
            textbuffer.insert(enditer,message)
            
            nn=bcname.index(selectedFaces[0])
            self.liststore_patchname[nn][1] = 'cyclicAMI'
            
            window.destroy()
            
            self.refresh(bcitembox,boundarybox,gvtk,notebook,textbuffer)

        def chchk(widget):
            check=widget.get_active()
            if check==False:weightEntry.set_sensitive(0)
            else:weightEntry.set_sensitive(1)

        nametype={}
        for i in range(len(bcname)):nametype[bcname[i]]=bctype[i]

        window=gtk.Window(gtk.WINDOW_TOPLEVEL);window.set_title('Select Interface')
        window.set_border_width(5);window.set_position(gtk.WIN_POS_CENTER);window.set_transient_for(self.mainwindow)
        mainbox=gtk.VBox(False,5);window.add(mainbox)

        swin=gtk.ScrolledWindow();swin.set_border_width(5);swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        mainbox.pack_start(swin,True,True,5)
        itembox=gtk.VBox(False,0);swin.add_with_viewport(itembox)
        scheight=len(bcname)*38
        if scheight > 750:scheight=750        
        swin.set_size_request(300,scheight)

        setName=[]
        for i in range(len(bcname)):
            hbox1=gtk.HBox(False,5);itembox.pack_start(hbox1,False,False,2)                
            setName.append(gtk.CheckButton(bcname[i],use_underline=False));setName[i].set_size_request(180,28)
            if nametype[bcname[i]]!='cyclicAMI' and bcname[i]!=patch:
                hbox1.pack_start(setName[i],False,False,0)

        weightbox=gtk.HBox(False,5)
        mainbox.pack_start(weightbox,False,False,5)
        checkWeight=gtk.CheckButton('lowWeightCorrection');checkWeight.set_active(0)
        checkWeight.connect('toggled',chchk)
        weightEntry=gtk.Entry();weightEntry.set_size_request(80,28);weightEntry.set_text('0.1')
        weightbox.pack_start(checkWeight,False,False,5);weightbox.pack_start(weightEntry,False,False,5)

        hbox2=gtk.HBox(False,5)
        mainbox.pack_start(hbox2,False,False,5)
        app=gtk.Button(label='Apply');app.set_size_request(100,28)
        cll=gtk.Button(label='Close');cll.set_size_request(100,28)
        app.connect('clicked',makeInterface);cll.connect('clicked',closethis)
        hbox2.pack_end(cll,False,False,5);hbox2.pack_end(app,False,False,5)  

        window.show_all() 
    #---------------------------------------------------------------------------------------------------------
    def createCyclicAMIRotation(self,bcname,bctype,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer):

        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)

        def closethis(widget,data=None):window.destroy()

        def makeAMI(widget,data=None):
            selectedpatch=[]
            for i in range(len(bcname)):
                aa=setName[i].get_active()		   
                if aa==True:selectedpatch.append(bcname[i])
            selectedpatch.append(patch)		
            if len(selectedpatch)!=2:
                GEN.makeDialog('Error!!! Selected patch is not 2')
                return
            else:
                rr1=r1Entry.get_text();rr2=r2Entry.get_text();rr3=r3Entry.get_text()		
                dd=axisCombo.child;axis=dd.get_text()
                WFILE.changeDictionaryDictRotCyclic(selectedpatch,rr1,rr2,rr3,axis)
                os.system('changeDictionary -disablePatchGroups -dict '+self.caseDir+'/system/changeDictionaryDictRotCyclic -case '+self.caseDir+' > '+self.caseDir+'/logfiles/log.changeDictionary')
                message='\n [  "'+selectedpatch[0]+'" and "'+selectedpatch[1]+'" are made to Rotational cyclicAMI  ]\n\n'                     
                enditer=textbuffer.get_end_iter()
                textbuffer.insert(enditer,message)
           
            nn=bcname.index(selectedpatch[0])
            self.liststore_patchname[nn][1] = 'cyclicAMI'
                    
            window.destroy()                
            
            self.refresh(bcitembox,boundarybox,gvtk,notebook,textbuffer)

        window=gtk.Window(gtk.WINDOW_TOPLEVEL);window.set_title('Select Rotational Cyclic pair')
        window.set_border_width(5);window.set_position(gtk.WIN_POS_CENTER);window.set_transient_for(self.mainwindow)
        mainbox=gtk.VBox(False,5);window.add(mainbox)
        

        swin=gtk.ScrolledWindow();swin.set_border_width(5);swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        mainbox.pack_start(swin,True,True,5);itembox=gtk.VBox(False,0);swin.add_with_viewport(itembox)
        scheight=len(bcname)*38
        if scheight > 750:scheight=750        
        swin.set_size_request(300,scheight)

        setName=[]
        for i in range(len(bcname)):
            hbox1=gtk.HBox(False,5);itembox.pack_start(hbox1,False,False,2)                
            setName.append(gtk.CheckButton(bcname[i],use_underline=False));setName[i].set_size_request(180,28)
            if bctype[i]!='cyclicAMI' and bcname[i]!=patch:
                hbox1.pack_start(setName[i],False,False,0)

        label1=gtk.Label('rotationCenter')
        r1Entry=gtk.Entry();r1Entry.set_size_request(60,28);r1Entry.set_text('0')
        r2Entry=gtk.Entry();r2Entry.set_size_request(60,28);r2Entry.set_text('0')
        r3Entry=gtk.Entry();r3Entry.set_size_request(60,28);r3Entry.set_text('0')
        hbox=gtk.HBox(False,5);mainbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label1,False,False,5);hbox.pack_end(r3Entry,False,False,5)
        hbox.pack_end(r2Entry,False,False,5);hbox.pack_end(r1Entry,False,False,5)

        label2=gtk.Label('rotationAxis')
        axisCombo=gtk.combo_box_entry_new_text();axisCombo.set_size_request(120,28)
        axisCombo.append_text('x');axisCombo.append_text('y');axisCombo.append_text('z')
        axisCombo.set_active(2)
        hbox=gtk.HBox(False,5);mainbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label2,False,False,5);hbox.pack_end(axisCombo,False,False,5)

        hbox2=gtk.HBox(False,5)
        mainbox.pack_start(hbox2,False,False,5)
        app=gtk.Button(label='Apply');app.set_size_request(100,28)
        cll=gtk.Button(label='Close');cll.set_size_request(100,28)
        app.connect('clicked',makeAMI);cll.connect('clicked',closethis)
        hbox2.pack_end(cll,False,False,5);hbox2.pack_end(app,False,False,5)  

        window.show_all()   
    #---------------------------------------------------------------------------------------------------------
    def createCyclicAMITranslate(self,bcname,bctype,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer):

        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)

        def closethis(widget,data=None):window.destroy()

        def makeAMI(widget,data=None):        
            selectedpatch=[]
            for i in range(len(bcname)):
                aa=setName[i].get_active()		   
                if aa==True:selectedpatch.append(bcname[i])
            selectedpatch.append(patch)		
            if len(selectedpatch)!=2:
                GEN.makeDialog('Error!!! Selected patch is not 2')
                return
            else:            
                cc1=c1Entry.get_text();cc2=c2Entry.get_text();cc3=c3Entry.get_text()
                WFILE.changeDictionaryDictTranslateCyclic(selectedpatch,cc1,cc2,cc3)
                os.system('changeDictionary -disablePatchGroups -dict '+self.caseDir+'/system/changeDictionaryDictTranslateCyclic -case '+self.caseDir+' > '+self.caseDir+'/logfiles/log.changeDictionary')
                message='\n [  "'+selectedpatch[0]+'" and "'+selectedpatch[1]+'" are made to Translational cyclicAMI  ]\n\n'                     
                enditer=textbuffer.get_end_iter()
                textbuffer.insert(enditer,message)
            
            nn=bcname.index(selectedpatch[0])
            self.liststore_patchname[nn][1] = 'cyclicAMI'
            
            window.destroy()                
            
            self.refresh(bcitembox,boundarybox,gvtk,notebook,textbuffer)

        window=gtk.Window(gtk.WINDOW_TOPLEVEL);window.set_title('Select Translational Cyclic pair')
        window.set_border_width(5);window.set_position(gtk.WIN_POS_CENTER);window.set_transient_for(self.mainwindow)
        mainbox=gtk.VBox(False,5);window.add(mainbox)

        swin=gtk.ScrolledWindow();swin.set_border_width(5);swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        mainbox.pack_start(swin,True,True,5);itembox=gtk.VBox(False,0);swin.add_with_viewport(itembox)
        scheight=len(bcname)*38
        if scheight > 750:scheight=750        
        swin.set_size_request(300,scheight)

        setName=[]
        for i in range(len(bcname)):
            hbox1=gtk.HBox(False,5);itembox.pack_start(hbox1,False,False,2)                
            setName.append(gtk.CheckButton(bcname[i],use_underline=False));setName[i].set_size_request(180,28)
            if bctype[i]!='cyclicAMI' and bcname[i]!=patch:
                hbox1.pack_start(setName[i],False,False,0)
        
        label=gtk.Label('vector, 1st to 2nd')
        c1Entry=gtk.Entry();c1Entry.set_size_request(60,28);c1Entry.set_text('0')
        c2Entry=gtk.Entry();c2Entry.set_size_request(60,28);c2Entry.set_text('0')
        c3Entry=gtk.Entry();c3Entry.set_size_request(60,28);c3Entry.set_text('0')
        hbox=gtk.HBox(False,5);mainbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5);hbox.pack_end(c3Entry,False,False,5)
        hbox.pack_start(c2Entry,False,False,0);hbox.pack_end(c1Entry,False,False,5)

        hbox=gtk.HBox(False,5);mainbox.pack_start(hbox,False,False,5)
        app=gtk.Button(label='Apply');app.set_size_request(100,28)
        cll=gtk.Button(label='Close');cll.set_size_request(100,28)
        app.connect('clicked',makeAMI);cll.connect('clicked',closethis)
        hbox.pack_end(cll,False,False,5);hbox.pack_end(app,False,False,5)  

        window.show_all() 
    #---------------------------------------------------------------------------------------------------------
    def changeAMIToOthers(self,patch,newtype):
        aa=glob.glob(self.caseDir+'/constant/polyMesh/boundary')
        if len(aa)!=0:
            nnn=[]
            f=open(aa[0],'r');ff=f.readlines();f.close()
            for i in range(len(ff)):                
                if ff[i][:-1]==patch:
                    ff[i+2]='    type            '+newtype+';\n'
                    ff[i+6]='del'
                    ff[i+7]='del'
                    ff[i+8]='del'
            for ii in ff:
                if ii!='del':nnn.append(ii)
            f=open(aa[0],'w')
            for ii in nnn:f.write(ii)
            f.close()
    #---------------------------------------------------------------------------------------------------------
    def boundarySort(self):
        GEN=generalClass(self.caseDir)
        bn=GEN.getBCName();bt=GEN.getBCType()
        bdict={}
        for i in range(len(bn)):bdict[bn[i]]=bt[i]
        
        patches=[];walls=[];amis=[];etcs=[]
        for ii in bn:
            if bdict[ii]=='patch':patches.append(ii)
            elif bdict[ii]=='wall':walls.append(ii)
            elif bdict[ii]=='cyclicAMI':amis.append(ii)
            else:etcs.append(ii)
        
        patches.sort();walls.sort();amis.sort();etcs.sort()    
        names=[];types=[]
        for ii in patches:names.append(ii);types.append(bdict[ii])
        for ii in walls:names.append(ii);types.append(bdict[ii])
        for ii in amis:names.append(ii);types.append(bdict[ii])
        for ii in etcs:names.append(ii);types.append(bdict[ii])

        nametypelist=[names,types]       
        return nametypelist                
    #---------------------------------------------------------------------------------------------------------
    def copyBoundaryCondition(self,bcitembox,boundarybox,gvtk):
    
        GEN=generalClass(self.caseDir)
    
        def selectSource(widget):
            aa=widget.child;sourcename=aa.get_text()
            
            aa=glob.glob(self.caseDir+'/system/settings/BCDict:'+sourcename)
            if len(aa)==0:
                text='"'+sourcename+'" is not set yet'
                GEN.makeDialog(text)
            else:            
                aa=itembox.get_children()
                for ii in aa:itembox.remove(ii)
            
                self.setName=[]
                if bcdict[sourcename]=='wall':dest=walls
                elif bcdict[sourcename]=='patch':dest=patchs
            
                for i in range(len(dest)):
                    hbox1=gtk.HBox(False,5);itembox.pack_start(hbox1,False,False,2)                
                    self.setName.append(gtk.CheckButton(dest[i],use_underline=False));self.setName[i].set_size_request(180,28)
                    if dest[i]!=sourcename:
                        hbox1.pack_start(self.setName[i],False,False,0)
                
            window.show_all()
        
        def applythis(widget):
            aa=sourceCombo.child;sourcename=aa.get_text()
            targets=[]
            for i in range(len(self.setName)):
                dest=self.setName[i].get_label()
                if self.setName[i].get_active()==1:targets.append(dest)
                
            for ii in targets:
                targetdic=GEN.makeDictFromFile(self.caseDir+'/system/settings/BCDict:'+sourcename)
                targetdic['name']=ii
                targetkeys=targetdic.keys()
                f=open(self.caseDir+'/system/settings/BCDict:'+ii,'w')
                for ii in targetkeys:f.write(ii+';'+ targetdic[ii]+'\n')
                f.close()
            
            window.destroy()
        
        def closethis(widget):window.destroy()
        
        names=GEN.getBCName();types=GEN.getBCType()
        
        bcdict={};walls=[];patchs=[];wallpatchs=[]
        for i in range(len(names)):
            bcdict[names[i]]=types[i]
            if types[i]=='wall':walls.append(names[i])
            elif types[i]=='patch':patchs.append(names[i])
        wallpatchs=patchs+walls
        
        window=gtk.Window(gtk.WINDOW_TOPLEVEL);window.set_title('Copy Boundary Conditions')
        window.set_border_width(5);window.set_position(gtk.WIN_POS_CENTER);window.set_transient_for(self.mainwindow)
        mainbox=gtk.VBox(False,5);window.add(mainbox)
        
        hhbox=gtk.HBox(False,5);mainbox.pack_start(hhbox,False,False,0)
        leftbox=gtk.VBox(False,5);hhbox.pack_start(leftbox,False,False,5)
        rightbox=gtk.VBox(False,5);hhbox.pack_start(rightbox,False,False,5)        
        
        label=gtk.Label('Select Source Patch')
        hbox=gtk.HBox(False,5);leftbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5)
        
        sourceCombo=gtk.combo_box_entry_new_text();sourceCombo.set_size_request(200,28)
        for ii in wallpatchs:sourceCombo.append_text(ii)
        sourceCombo.connect('changed',selectSource)
        hbox=gtk.HBox(False,5);leftbox.pack_start(hbox,False,False,5)
        hbox.pack_start(sourceCombo,False,False,5)
        
        
        label=gtk.Label('Select Target Patches')
        hbox=gtk.HBox(False,5);rightbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5)
        swin=gtk.ScrolledWindow();swin.set_border_width(5);swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        rightbox.pack_start(swin,True,True,5);itembox=gtk.VBox(False,0);swin.add_with_viewport(itembox) 
        swin.set_size_request(250,400)


        hbox2=gtk.HBox(False,5)
        mainbox.pack_start(hbox2,False,False,5)
        app=gtk.Button(label='Copy');app.set_size_request(100,28)
        cll=gtk.Button(label='Close');cll.set_size_request(100,28)
        app.connect('clicked',applythis);cll.connect('clicked',closethis)
        hbox2.pack_end(cll,False,False,5);hbox2.pack_end(app,False,False,5)  

        window.show_all()       

    
    

