#-*-coding:utf8-*-
#############################################################################
#    Baram : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of Baram.                                            
#
#    Baram is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    Baram is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Baram.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for postprocessing functions
#
# Author
#    NEXTFOAM Inc. 
##############################################################################

import os,math,glob,sys,time;import pygtk;pygtk.require('2.0');import gtk
import pickle

from generalSub import * 
from VTKStuffClass import *

class freepanelMonitoringClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()
                
    def __init__(self,caseDir,mainwindow,solver,installpath,rr,gg,bb):
        self.caseDir=caseDir
        self.installpath=installpath
        self.mainwindow=mainwindow
        self.solver=solver
        self.openSubWindow=False
        self.rr=rr;self.gg=gg;self.bb=bb
        
        aa=glob.glob(self.caseDir+'/system/settings/simulationConditions')
        if aa!=[]:
            GEN=generalClass(self.caseDir);dic=GEN.makeDictFromFile(aa[0])
            self.turmodel=dic['turmodel']
        else:self.turmodel='standardKEpsilon'
    #------------------------------------------------------------------------------------------
    def monitoring(self,gvtk,notebook,maintextbuffer):
    
        def pointSet(widget):
            if self.openSubWindow==False:self.pointProbeThermal(self.mainwindow,gvtk,self.pointPlotB,notebook,maintextbuffer)

        def pointPlot(widget):
            self.plotprobeThermal()
            self.closePointB.set_sensitive(1)
            self.pointPlotB.set_sensitive(0)

        def closePointPlot(widget):
            self.closeplotprobeThermal()
            self.pointPlotB.set_sensitive(1)
            self.closePointB.set_sensitive(0)
            
        def surfSet(widget):
            if self.openSubWindow==False:self.surfaceProbe(self.mainwindow,self.surfPlotB,maintextbuffer)

        def surfPlot(widget):
            self.plotsurface()
            self.closeSurfB.set_sensitive(1)
            self.surfPlotB.set_sensitive(0)

        def closeSurfPlot(widget):
            self.closeplotprobesurface()
            self.closeSurfB.set_sensitive(0)
            self.surfPlotB.set_sensitive(1)            

        def flowSet(widget):
            if self.openSubWindow==False:self.flowrate(self.mainwindow,self.flowPlotB,maintextbuffer)

        def flowPlot(widget):
            self.plotflowrate()
            self.closeFlowB.set_sensitive(1)
            self.flowPlotB.set_sensitive(0)

        def closeFlowPlot(widget):
            self.closeplotflowrate()
            self.closeFlowB.set_sensitive(0)
            self.flowPlotB.set_sensitive(1)
            
        def forceSet(widget):
            if self.openSubWindow==False:self.forceSet(self.mainwindow,self.forcePlotB,maintextbuffer)

        def forcePlot(widget):
            self.plotforce()
            self.closeForceB.set_sensitive(1)
            self.forcePlotB.set_sensitive(0)
            
        def closeForcePlot(widget):
            self.closeplotforce()
            self.closeForceB.set_sensitive(0)
            self.forcePlotB.set_sensitive(1)            

        def probeLocation(widget):
            self.probeLocation(self.mainwindow,gvtk,notebook)
            
        def patchIntegrate(widget):
            self.patchIntegrate(self.mainwindow)            

        def patchAverage(widget):
            self.patchAverage(self.mainwindow) 
            
        def forceReport(widget):
            self.openForce(self.mainwindow) 
            
        def plotResidual(widget):
            bb=glob.glob(self.caseDir+'/PyFoamRunner.mpirun.logfile')
            if len(bb)!=0:aa='pyFoamPlotWatcher.py PyFoamRunner.mpirun.logfile &'
            else:aa='pyFoamPlotWatcher.py PyFoamRunner.'+self.solver+'.logfile &'
            os.chdir(self.caseDir);os.system(aa)
        
        def closeResidual(widget):                
            os.chdir(self.caseDir);os.system('pkill -9 pyFoamPlotWatch')        
                                    
        self.monitorbox=gtk.VBox(False,5)
        label=gtk.Label();label.set_markup('<b>  Monitoring / Post-processing  </b>')
        ebox=gtk.EventBox();self.monitorbox.pack_start(ebox,True,True,0)
        label.set_alignment(0,0.5);label.set_size_request(50,45)
        ebox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.color_parse('white'));ebox.add(label)
                
        #separator=gtk.HSeparator();separator.set_size_request(285,5);self.monitorbox.pack_start(separator,True,True,10)

        label=gtk.Label('   Point value monitoring');self.monitorbox.pack_start(label,False,False,5);label.set_alignment(0,0)
        
        self.pointSetB=gtk.Button('Set');self.pointSetB.set_size_request(70,28);self.pointSetB.connect('clicked',pointSet)
        self.pointPlotB=gtk.Button('Plot');self.pointPlotB.set_size_request(70,28);self.pointPlotB.connect('clicked',pointPlot)
        self.closePointB=gtk.Button('Close');self.closePointB.set_size_request(70,28);self.closePointB.connect('clicked',closePointPlot)
        aa=glob.glob(self.caseDir+'/system/settings/probepoint')
        if len(aa)==0:self.pointPlotB.set_sensitive(0)
        self.closePointB.set_sensitive(0)
        self.pointbox=gtk.HBox(False,5);self.monitorbox.pack_start(self.pointbox,False,False,5)                
        self.pointbox.pack_end(self.closePointB,False,False,5);self.pointbox.pack_end(self.pointPlotB,False,False,0)
        self.pointbox.pack_end(self.pointSetB,False,False,5)

        label=gtk.Label('   Surface average monitoring');self.monitorbox.pack_start(label,False,False,5);label.set_alignment(0,0)

        self.surfSetB=gtk.Button('Set');self.surfSetB.set_size_request(70,28);self.surfSetB.connect('clicked',surfSet)
        self.surfPlotB=gtk.Button('Plot');self.surfPlotB.set_size_request(70,28);self.surfPlotB.connect('clicked',surfPlot)
        self.closeSurfB=gtk.Button('Close');self.closeSurfB.set_size_request(70,28);self.closeSurfB.connect('clicked',closeSurfPlot)
        aa=glob.glob(self.caseDir+'/system/settings/probesurface')
        if len(aa)==0:self.surfPlotB.set_sensitive(0)
        self.closeSurfB.set_sensitive(0)
        self.surfbox=gtk.HBox(False,5);self.monitorbox.pack_start(self.surfbox,False,False,5)
        self.surfbox.pack_end(self.closeSurfB,False,False,5)
        self.surfbox.pack_end(self.surfPlotB,False,False,0);self.surfbox.pack_end(self.surfSetB,False,False,5)

        label=gtk.Label('   Flowrate monitoring');self.monitorbox.pack_start(label,False,False,5);label.set_alignment(0,0)

        self.flowSetB=gtk.Button('Set');self.flowSetB.set_size_request(70,28);self.flowSetB.connect('clicked',flowSet)
        self.flowPlotB=gtk.Button('Plot');self.flowPlotB.set_size_request(70,28);self.flowPlotB.connect('clicked',flowPlot)
        self.closeFlowB=gtk.Button('Close');self.closeFlowB.set_size_request(70,28);self.closeFlowB.connect('clicked',closeFlowPlot)
        aa=glob.glob(self.caseDir+'/system/settings/flowrate')
        if len(aa)==0:self.flowPlotB.set_sensitive(0)
        self.closeFlowB.set_sensitive(0)
        self.flowbox=gtk.HBox(False,5);self.monitorbox.pack_start(self.flowbox,False,False,5)
        self.flowbox.pack_end(self.closeFlowB,False,False,5)
        self.flowbox.pack_end(self.flowPlotB,False,False,0);self.flowbox.pack_end(self.flowSetB,False,False,5)

        label=gtk.Label('   Force monitoring');self.monitorbox.pack_start(label,False,False,5);label.set_alignment(0,0)

        self.forceSetB=gtk.Button('Set');self.forceSetB.set_size_request(70,28);self.forceSetB.connect('clicked',forceSet)
        self.forcePlotB=gtk.Button('Plot');self.forcePlotB.set_size_request(70,28);self.forcePlotB.connect('clicked',forcePlot)
        self.closeForceB=gtk.Button('Close');self.closeForceB.set_size_request(70,28);self.closeForceB.connect('clicked',closeForcePlot)
        aa=glob.glob(self.caseDir+'/system/settings/forceMonitor')
        if len(aa)==0:self.forcePlotB.set_sensitive(0)
        self.closeForceB.set_sensitive(0)
        self.forcebox=gtk.HBox(False,5);self.monitorbox.pack_start(self.forcebox,False,False,5)
        self.forcebox.pack_end(self.closeForceB,False,False,5)
        self.forcebox.pack_end(self.forcePlotB,False,False,0);self.forcebox.pack_end(self.forceSetB,False,False,5)
    
        separator=gtk.HSeparator();separator.set_size_request(285,5);self.monitorbox.pack_start(separator,True,True,10)

        self.probeLocationB=gtk.Button('Probe Location');self.probeLocationB.connect('clicked',probeLocation)
        self.forceReportB=gtk.Button('Force Report');self.forceReportB.connect('clicked',forceReport)
        self.probeLocationB.set_size_request(120,28);self.forceReportB.set_size_request(120,28)
        hbox=gtk.HBox(False,5);self.monitorbox.pack_start(hbox,False,False,2)
        hbox.pack_start(self.forceReportB,True,True,5);hbox.pack_end(self.probeLocationB,True,True,5)
        
        self.patchIntegrateB=gtk.Button('Patch Integrate');self.patchIntegrateB.connect('clicked',patchIntegrate)
        self.patchAverageB=gtk.Button('Patch Average');self.patchAverageB.connect('clicked',patchAverage)
        self.patchIntegrateB.set_size_request(120,28);self.patchAverageB.set_size_request(120,28)
        hbox=gtk.HBox(False,5);self.monitorbox.pack_start(hbox,False,False,2)
        hbox.pack_start(self.patchIntegrateB,True,True,5);hbox.pack_end(self.patchAverageB,True,True,5)
        
        self.plotResidualB=gtk.Button('Plot Residual');self.plotResidualB.connect('clicked',plotResidual)
        self.closeResidualB=gtk.Button('Close Residual');self.closeResidualB.connect('clicked',closeResidual)
        self.plotResidualB.set_size_request(120,28);self.closeResidualB.set_size_request(120,28)
        hbox=gtk.HBox(False,5);self.monitorbox.pack_start(hbox,False,False,2)
        hbox.pack_start(self.plotResidualB,True,True,5);hbox.pack_end(self.closeResidualB,True,True,5)
    
        return self.monitorbox      
    #------------------------------------------------------------------------------------------
    def pointProbeThermal(self,mainwindow,gvtk,plotButton,notebook,maintextbuffer):

        def applythis(widget,data=None):
            self.inter=self.interval.get_text();num=len(self.addhbox)
            names=[]
            if num!=0:                
                ff=open(self.caseDir+'/system/settings/probepoint','w')
                ff.write(self.inter+'\n')
                for i in range(num):
                    nn=self.nameEntry[i].get_text();names.append(nn)
                    xx=self.xEntry[i].get_text();yy=self.yEntry[i].get_text();zz=self.zEntry[i].get_text()
                    pp=self.pcheck[i].get_active();uu=self.ucheck[i].get_active()
                    kk=self.kcheck[i].get_active();ee=self.echeck[i].get_active();oo=self.ocheck[i].get_active()
                    
                    prgh=self.prghcheck[i].get_active();tt=self.Tcheck[i].get_active()
                    
                    if pp==False and uu==False and kk==False and ee==False and oo==False and prgh==False and tt==False:
                        GEN=generalClass(self.caseDir);GEN.makeDialog('Warning! No Field is selected')
                        return
                    else:                    
                        f=open(self.caseDir+'/system/probeFunction_'+nn,'w')
                        f.write('    probes_'+nn+'\n')
                        f.write('    {\n')
                        f.write('        functionObjectLibs ( "libsampling.so" );\n')
                        f.write('        type            probes;\n')
                        f.write('        name            probes;\n')
                        f.write('        outputControl   timeStep;\n')
                        f.write('        outputInterval  '+self.inter+';\n')
                        f.write('\n')
                        f.write('        fields\n')
                        f.write('        (\n')
                        if pp==True:f.write('            p\n')
                        if prgh==True:f.write('            p_rgh\n')
                        if uu==True:f.write('            Umag\n')
                        if tt==True:f.write('            T\n')
                        if kk==True:f.write('            k\n')
                        if ee==True:f.write('            epsilon\n')
                        if oo==True:f.write('            omega\n')
                        f.write('        );\n')
                        f.write('\n')
                        f.write('        probeLocations\n')
                        f.write('        (\n')
                        f.write('            ('+xx+' '+yy+' '+zz+') \n')
                        f.write('        );\n')
                        f.write('    }\n') 
                        f.close()
                        
                        ff.write(nn+' '+xx+' '+yy+' '+zz+' ')
                        if pp==0:ff.write('False ')
                        else:ff.write('True ')
                        if prgh==0:ff.write('False ')
                        else:ff.write('True ')
                        if uu==0:ff.write('False ')
                        else:ff.write('True ')
                        if tt==0:ff.write('False ')
                        else:ff.write('True ')
                        if kk==0:ff.write('False ')
                        else:ff.write('True ')
                        if ee==0:ff.write('False ')
                        else:ff.write('True ')
                        if oo==0:ff.write('False\n')
                        else:ff.write('True\n')
                        
                        plotButton.set_sensitive(1)
                        
                ff.close()
                
                message='\n [  Point monitoring is set  ]\n'
                for i in range(num):
                    xx=self.xEntry[i].get_text();yy=self.yEntry[i].get_text();zz=self.zEntry[i].get_text()
                    message=message+'\n    - ( '+xx+' '+yy+' '+zz+' ) for '  
                    if pp==True:message=message+'  p  '
                    if prgh==True:message=message+'  p_rgh  '
                    if uu==True:message=message+'  Umag  '
                    if tt==True:message=message+'  T  '
                    if kk==True:message=message+'  k  '
                    if ee==True:message=message+'  epsilon  '
                    if oo==True:message=message+'  omega  '
                    message=message+'\n'                    
                    
                enditer=maintextbuffer.get_end_iter()
                maintextbuffer.insert(enditer,message)
                
            else:
                aa=glob.glob(self.caseDir+'/system/settings/probepoint')
                if len(aa)!=0:os.system('rm '+aa[0])
                plotButton.set_sensitive(0)
                
            sameName=False
            for ii in names:
                if names.count(ii)>1:sameName=True
            if sameName==True:
                GEN=generalClass(self.caseDir);GEN.makeDialog('Warning! Ther are same point name')
            else:
                self.Win.destroy();self.openSubWindow=False

        def closethis(widget,data=None):
            self.Win.destroy();self.openSubWindow=False

        def delpoint(widget):
            i=self.delButton.index(widget)
            os.system('rm '+self.caseDir+'/system/probeFunction_'+self.nameEntry[i].get_text())
            self.itembox.remove(self.addhbox[i])
            del self.addhbox[i];del self.nameEntry[i];del self.xEntry[i];del self.yEntry[i];del self.zEntry[i]
            del self.pcheck[i];del self.ucheck[i];del self.kcheck[i];del self.echeck[i];del self.ocheck[i]
            del self.prghcheck[i];del self.Tcheck[i]
            del self.delButton[i];del self.displayButton[i]
            self.Win.show_all()
            
        def displaypoint(widget):
            i=self.displayButton.index(widget)
            xc=self.xEntry[i].get_text();yc=self.yEntry[i].get_text();zc=self.zEntry[i].get_text()            
            VTK=VTKStuff(self.caseDir,self.rr,self.gg,self.bb);renderer=VTK.showPoint(xc,yc,zc)
            gvtk.GetRenderWindow().AddRenderer(renderer)
            gvtk.Initialize();gvtk.Start()
            
            notebook.set_current_page(1)

        def addpoint(widget):
            aa=self.itembox.get_children();np=int(len(aa))-2
            self.addhbox.append(gtk.HBox(False,5))
            self.nameEntry.append(gtk.Entry());self.xEntry.append(gtk.Entry());self.yEntry.append(gtk.Entry());self.zEntry.append(gtk.Entry())
            self.pcheck.append(gtk.CheckButton());self.ucheck.append(gtk.CheckButton())
            self.prghcheck.append(gtk.CheckButton())
            self.Tcheck.append(gtk.CheckButton())
            self.kcheck.append(gtk.CheckButton())
            self.echeck.append(gtk.CheckButton());self.ocheck.append(gtk.CheckButton())
            self.delButton.append(gtk.Button('Delete'))
            self.displayButton.append(gtk.Button('Display'))
            self.nameEntry[np].set_size_request(80,28);self.nameEntry[np].set_text('point-'+str(np+1))
            self.xEntry[np].set_size_request(60,28);self.yEntry[np].set_size_request(60,28);self.zEntry[np].set_size_request(60,28)
            self.pcheck[np].set_size_request(40,28)
            
            self.prghcheck[np].set_size_request(40,28)
            self.Tcheck[np].set_size_request(40,28)
            
            self.ucheck[np].set_size_request(40,28)
            self.kcheck[np].set_size_request(40,28);self.echeck[np].set_size_request(40,28)
            self.ocheck[np].set_size_request(40,28)
            self.delButton[np].set_size_request(70,28);self.delButton[np].connect('clicked',delpoint)
            self.displayButton[np].set_size_request(80,28);self.displayButton[np].connect('clicked',displaypoint)
            self.addhbox[np].pack_start(self.nameEntry[np],False,False,2);self.addhbox[np].pack_start(self.xEntry[np],False,False,2)
            self.addhbox[np].pack_start(self.yEntry[np],False,False,0);self.addhbox[np].pack_start(self.zEntry[np],False,False,2)
            self.addhbox[np].pack_start(self.pcheck[np],False,False,2)
            
            self.addhbox[np].pack_start(self.prghcheck[np],False,False,2)
            
            self.addhbox[np].pack_start(self.ucheck[np],False,False,2)
            
            self.addhbox[np].pack_start(self.Tcheck[np],False,False,2)
            
            self.addhbox[np].pack_start(self.kcheck[np],False,False,2)
            self.addhbox[np].pack_start(self.echeck[np],False,False,2);self.addhbox[np].pack_start(self.ocheck[np],False,False,2)

            self.addhbox[np].pack_start(self.delButton[np],False,False,2)
            self.addhbox[np].pack_start(self.displayButton[np],False,False,2)
            self.itembox.pack_start(self.addhbox[np],False,False,5)
            if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                self.prghcheck[np].set_sensitive(0);self.Tcheck[np].set_sensitive(0)
                
            self.Win.show_all()

        if self.openSubWindow==False:
        
            self.Win=gtk.Window(gtk.WINDOW_TOPLEVEL)
            self.Win.connect('delete_event',self.delete_event);self.Win.connect('destroy',self.destroy)
            self.Win.set_border_width(10);self.Win.set_title('Set Points to Monitor')
            self.Win.set_transient_for(mainwindow)
            self.Win.set_position(gtk.WIN_POS_CENTER)
            self.vbox=gtk.VBox(False,5);self.Win.add(self.vbox)
            #--------------------------------------------------------------------------------------------------------
            self.interBox=gtk.HBox(False,5)
            self.label=gtk.Label('Output interval');self.label.set_size_request(120,28);self.label.set_alignment(0,0.5)
            self.interval=gtk.Entry();self.interval.set_text('1');self.interval.set_size_request(100,28)

            self.vbox.pack_start(self.interBox,False,False,5)
            self.interBox.pack_start(self.label,False,False,5);self.interBox.pack_start(self.interval,False,False,5)

            self.locFrame=gtk.Frame();self.locFrame.set_label('Probe Locations');self.locFrame.set_label_align(0.0,1.0)
            self.locFrame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)

            self.locBox=gtk.VBox(False,5);self.vbox.pack_start(self.locFrame,False,False,5);self.locFrame.add(self.locBox)
            
            self.addButton=gtk.Button('Add Point');self.addButton.set_size_request(120,28);self.addButton.connect('clicked',addpoint)
            self.npointbox=gtk.HBox(False,5);self.locBox.pack_start(self.npointbox,False,False,5)
            self.npointbox.pack_start(self.addButton,False,False,5)

            self.swin=gtk.ScrolledWindow();self.swin.set_border_width(5);self.swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
            self.locBox.pack_start(self.swin);self.itembox=gtk.VBox(False,0);self.swin.add_with_viewport(self.itembox)
            self.swin.set_size_request(870,300)

            name=gtk.Label('point name');name.set_size_request(80,28)
            xx=gtk.Label('x');xx.set_size_request(60,28)
            yy=gtk.Label('y');yy.set_size_request(60,28)
            zz=gtk.Label('z');zz.set_size_request(60,28)
            pp=gtk.Label('  p');pp.set_size_request(40,28);pp.set_alignment(0,0.5)
            ppp=gtk.Label('p_rgh');ppp.set_size_request(40,28);ppp.set_alignment(0,0.5)
            uu=gtk.Label('  U');uu.set_size_request(40,28);uu.set_alignment(0,0.5)
            tt=gtk.Label('  T');tt.set_size_request(40,28);tt.set_alignment(0,0.5)
            kk=gtk.Label('  k');kk.set_size_request(40,28);kk.set_alignment(0,0.5)
            ee=gtk.Label('  ε');ee.set_size_request(40,28);ee.set_alignment(0,0.5)
            oo=gtk.Label('  ω');oo.set_size_request(40,28);oo.set_alignment(0,0.5)
            
            self.titlebox=gtk.HBox(False,5);self.itembox.pack_start(self.titlebox,False,False,5)
            self.titlebox.pack_start(name,False,False,2)
            self.titlebox.pack_start(xx,False,False,2);self.titlebox.pack_start(yy,False,False,0);self.titlebox.pack_start(zz,False,False,2)
            
            self.titlebox.pack_start(pp,False,False,2)
            self.titlebox.pack_start(ppp,False,False,2)
            self.titlebox.pack_start(uu,False,False,2)
            self.titlebox.pack_start(tt,False,False,2)
            self.titlebox.pack_start(kk,False,False,2);self.titlebox.pack_start(ee,False,False,2);self.titlebox.pack_start(oo,False,False,2)
            separator=gtk.HSeparator();separator.set_size_request(400,5)
            self.itembox.pack_start(separator,False,False,5)

            self.addhbox=[];self.nameEntry=[];self.xEntry=[];self.yEntry=[];self.zEntry=[];self.pcheck=[];self.ucheck=[]
            self.kcheck=[];self.echeck=[];self.ocheck=[];self.delButton=[];self.displayButton=[]
            self.prghcheck=[];self.Tcheck=[]
            #-----------------------------------
            self.buttonBox=gtk.HBox(False,5);self.vbox.pack_start(self.buttonBox,False,False,5)
            self.appB=gtk.Button('Apply');self.closeB=gtk.Button('Close')
            self.appB.set_size_request(100,28);self.closeB.set_size_request(100,28)
            self.appB.connect('clicked',applythis);self.closeB.connect('clicked',closethis)
            self.buttonBox.pack_end(self.closeB,False,False,5);self.buttonBox.pack_end(self.appB,False,False,5)
            
            if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                ppp.set_sensitive(0);tt.set_sensitive(0)
            #-----------------------------------
            ggg=glob.glob(self.caseDir+'/system/settings/probepoint')
            if len(ggg)!=0:
                f=open(ggg[0],'r');ff=f.readlines();f.close()
                self.interval.set_text(ff[0][:-1])
                np=int(len(ff))-1
                for i in range(np):
                    setlist=ff[i+1][:-1].split(' ')
                    self.addhbox.append(gtk.HBox(False,5))
                    self.nameEntry.append(gtk.Entry());self.xEntry.append(gtk.Entry());self.yEntry.append(gtk.Entry());self.zEntry.append(gtk.Entry())
                    self.pcheck.append(gtk.CheckButton());self.prghcheck.append(gtk.CheckButton());self.ucheck.append(gtk.CheckButton())
                    self.Tcheck.append(gtk.CheckButton());self.kcheck.append(gtk.CheckButton())
                    self.echeck.append(gtk.CheckButton());self.ocheck.append(gtk.CheckButton())
                    self.delButton.append(gtk.Button('Delete'))
                    self.displayButton.append(gtk.Button('Display'))
                    self.nameEntry[i].set_size_request(80,28)
                    self.xEntry[i].set_size_request(60,28);self.yEntry[i].set_size_request(60,28);self.zEntry[i].set_size_request(60,28)
                    self.pcheck[i].set_size_request(40,28);self.prghcheck[i].set_size_request(40,28);self.ucheck[i].set_size_request(40,28)
                    self.Tcheck[i].set_size_request(40,28);self.kcheck[i].set_size_request(40,28)
                    self.echeck[i].set_size_request(40,28);self.ocheck[i].set_size_request(40,28)
                    self.delButton[i].set_size_request(70,28);self.delButton[i].connect('clicked',delpoint)
                    self.displayButton[i].set_size_request(80,28);self.displayButton[i].connect('clicked',displaypoint)
                    self.nameEntry[i].set_text(setlist[0])
                    self.xEntry[i].set_text(setlist[1]);self.yEntry[i].set_text(setlist[2]);self.zEntry[i].set_text(setlist[3])
                    if setlist[4]=='True':self.pcheck[i].set_active(1)
                    if setlist[5]=='True':self.prghcheck[i].set_active(1)
                    if setlist[6]=='True':self.ucheck[i].set_active(1)
                    if setlist[7]=='True':self.Tcheck[i].set_active(1)
                    if setlist[8]=='True':self.kcheck[i].set_active(1)
                    if setlist[9]=='True':self.echeck[i].set_active(1)
                    if setlist[10]=='True':self.ocheck[i].set_active(1)
                    self.addhbox[i].pack_start(self.nameEntry[i],False,False,2);self.addhbox[i].pack_start(self.xEntry[i],False,False,2)
                    self.addhbox[i].pack_start(self.yEntry[i],False,False,0);self.addhbox[i].pack_start(self.zEntry[i],False,False,2)
                    self.addhbox[i].pack_start(self.pcheck[i],False,False,2)
                    self.addhbox[i].pack_start(self.prghcheck[i],False,False,2)
                    self.addhbox[i].pack_start(self.ucheck[i],False,False,2)
                    self.addhbox[i].pack_start(self.Tcheck[i],False,False,2)
                    self.addhbox[i].pack_start(self.kcheck[i],False,False,2)
                    self.addhbox[i].pack_start(self.echeck[i],False,False,2)
                    self.addhbox[i].pack_start(self.ocheck[i],False,False,2)
                    self.addhbox[i].pack_start(self.delButton[i],False,False,2)
                    self.addhbox[i].pack_start(self.displayButton[i],False,False,2)
                    self.itembox.pack_start(self.addhbox[i],False,False,5)
                    
                    if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
                        self.prghcheck[i].set_sensitive(0);self.Tcheck[i].set_sensitive(0)
                    
            self.Win.show_all()
            
            self.openSubWindow=True
    #--------------------------------------------------
    def plotprobeThermal(self):

        a1=glob.glob(self.caseDir+'/system/settings/probepoint')
        if len(a1)!=0:
            f=open(a1[0],'r');ff=f.readlines();f.close()
            npoint=len(ff)-1

            for i in range(npoint):
                aa=ff[i+1][:-1].split(' ')
                name=aa[0];xx=aa[1];yy=aa[2];zz=aa[3];pp=aa[4];prgh=aa[5];uu=aa[6]
                tt=aa[7];kk=aa[8];ee=aa[9];oo=aa[10]
                aa=glob.glob(self.caseDir+'/postProcessing/probes_'+name+'/*')
                times=[]
                for ii in aa:
                    bb=ii.split('/');times.append(bb[-1]);times.sort()
                lastTime=times[-1]

                postdir=self.caseDir+'/postProcessing/probes_'+name
                plotp="plot '"+postdir+'/'+lastTime+"/p' u 1:2 w l title 'p'\n"
                plotprgh="plot '"+postdir+'/'+lastTime+"/p_rgh' u 1:2 w l title 'p_rgh'\n"
                plotu="plot '"+postdir+'/'+lastTime+"/Umag' u 1:2 w l title 'Umag'\n"
                plott="plot '"+postdir+'/'+lastTime+"/T' u 1:2 w l title 'T'\n"
                plotk="plot '"+postdir+'/'+lastTime+"/k' u 1:2 w l title 'k'\n"
                plote="plot '"+postdir+'/'+lastTime+"/epsilon' u 1:2 w l title 'epsilon'\n"
                ploto="plot '"+postdir+'/'+lastTime+"/omega' u 1:2 w l title 'omega'\n"
                gpstate="state=system(\"cat "+self.caseDir+"/PyFoamState.TheState\")\n"
                gpreread="if(state ne \"Finished\") pause 1; reread;\n"

                if pp=='True':
                    f=open(self.caseDir+'/system/settings/plotcmdp_'+name,'w')
                    f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Point Probe : "+name+"'\n")
                    f.write(gpstate);f.write(plotp);f.write(gpreread);f.close()
                    pidfile = self.caseDir+'/system/settings/plotprobecmdp_'+name+'_pid'
                    os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmdp_'+name+' |echo $! > '+pidfile+' &')
                    time.sleep(0.5)
                if prgh=='True':
                    f=open(self.caseDir+'/system/settings/plotcmdprgh_'+name,'w')
                    f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Point Probe : "+name+"'\n")
                    f.write(gpstate);f.write(plotprgh);f.write(gpreread);f.close()
                    pidfile = self.caseDir+'/system/settings/plotprobecmdprgh_'+name+'_pid'
                    os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmdprgh_'+name+' |echo $! > '+pidfile+' &')
                    time.sleep(0.5)                    
                if uu=='True':
                    f=open(self.caseDir+'/system/settings/plotcmdU_'+name,'w')
                    f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Point Probe : "+name+"'\n")
                    f.write(gpstate);f.write(plotu);f.write(gpreread);f.close()
                    pidfile = self.caseDir+'/system/settings/plotprobecmdU_'+name+'_pid'
                    os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmdU_'+name+' |echo $! > '+pidfile+' &')
                    time.sleep(0.5)
                if tt=='True':
                    f=open(self.caseDir+'/system/settings/plotcmdT_'+name,'w')
                    f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Point Probe : "+name+"'\n")
                    f.write(gpstate);f.write(plott);f.write(gpreread);f.close()
                    pidfile = self.caseDir+'/system/settings/plotprobecmdT_'+name+'_pid'
                    os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmdT_'+name+' |echo $! > '+pidfile+' &')
                    time.sleep(0.5)
                if kk=='True':
                    f=open(self.caseDir+'/system/settings/plotcmdk_'+name,'w')
                    f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Point Probe : "+name+"'\n")
                    f.write(gpstate);f.write(plotk);f.write(gpreread);f.close()
                    pidfile = self.caseDir+'/system/settings/plotprobecmdk_'+name+'_pid'
                    os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmdk_'+name+' |echo $! > '+pidfile+' &')
                    time.sleep(0.5)
                if ee=='True':
                    f=open(self.caseDir+'/system/settings/plotcmde_'+name,'w')
                    f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Point Probe : "+name+"'\n")
                    f.write(gpstate);f.write(plote);f.write(gpreread);f.close()
                    pidfile = self.caseDir+'/system/settings/plotprobecmde_'+name+'_pid'
                    os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmde_'+name+' |echo $! > '+pidfile+' &')
                    time.sleep(0.5)
                if oo=='True':
                    f=open(self.caseDir+'/system/settings/plotcmdo_'+name,'w')
                    f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Point Probe : "+name+"'\n")
                    f.write(gpstate);f.write(ploto);f.write(gpreread);f.close()
                    pidfile = self.caseDir+'/system/settings/plotprobecmdo_'+name+'_pid'
                    os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmdo_'+name+' |echo $! > '+pidfile+' &')
                    time.sleep(0.5)
        else:
            md=gtk.MessageDialog(None,gtk.DIALOG_MODAL,gtk.MESSAGE_INFO,gtk.BUTTONS_CLOSE,"Error! Probe point is not set")
            md.run();md.destroy()
    #--------------------------------------------------
    def closeplotprobeThermal(self):

        a1=glob.glob(self.caseDir+'/system/settings/probepoint')
        statefile = open(self.caseDir+'/PyFoamState.TheState');state = statefile.read();statefile.close()

        if len(a1)!=0:
            f=open(a1[0],'r');ff=f.readlines();f.close()

            npoint=len(ff)-1
            for i in range(npoint):
                aa=ff[i+1][:-1].split(' ')
                name=aa[0];xx=aa[1];yy=aa[2];zz=aa[3];pp=aa[4];prgh=aa[5];uu=aa[6]
                tt=aa[7];kk=aa[8];ee=aa[9];oo=aa[10]
                aa=glob.glob(self.caseDir+'/postProcessing/probes_'+name+'/*')
                times=[]
                for ii in aa:
                    bb=ii.split('/');times.append(bb[-1]);times.sort()
                lastTime=times[-1]
                if pp=='True':
                    pidfile = open(self.caseDir+'/system/settings/plotprobecmdp_'+name+'_pid')
                    ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                    if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)
                    else: os.system('kill -9 '+pid)
                if prgh=='True':
                    pidfile = open(self.caseDir+'/system/settings/plotprobecmdprgh_'+name+'_pid')
                    ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                    if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)                    
                    else: os.system('kill -9 '+pid)
                if uu=='True':
                    pidfile = open(self.caseDir+'/system/settings/plotprobecmdU_'+name+'_pid')
                    ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                    if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)                    
                    else: os.system('kill -9 '+pid)
                if tt=='True':
                    pidfile = open(self.caseDir+'/system/settings/plotprobecmdT_'+name+'_pid')
                    ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                    if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)                    
                    else: os.system('kill -9 '+pid)
                if kk=='True':
                    pidfile = open(self.caseDir+'/system/settings/plotprobecmdk_'+name+'_pid')
                    ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                    if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)                    
                    else: os.system('kill -9 '+pid)
                if ee=='True':
                    pidfile = open(self.caseDir+'/system/settings/plotprobecmde_'+name+'_pid')
                    ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                    if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)                    
                    else: os.system('kill -9 '+pid)
                if oo=='True':
                    pidfile = open(self.caseDir+'/system/settings/plotprobecmdo_'+name+'_pid')
                    ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                    if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)                    
                    else: os.system('kill -9 '+pid)
        else:
            md=gtk.MessageDialog(None,gtk.DIALOG_MODAL,gtk.MESSAGE_INFO,gtk.BUTTONS_CLOSE,"Error! Probe point is not set")
            md.run();md.destroy()
    #------------------------------------------------------------------------------------------
    def plotsurface(self):

        a1=glob.glob(self.caseDir+'/system/settings/probesurface')
        if len(a1)!=0:
            f=open(self.caseDir+'/system/settings/probesurface','r');ff=f.readlines();f.close()
            if ff[0][:-1]=='True':pCheck='yes'
            else:pCheck='no'
            if ff[1][:-1]=='True':prghCheck='yes'
            else:prghCheck='no'
            if ff[2][:-1]=='True':UCheck='yes'
            else:UCheck='no'
            if ff[3][:-1]=='True':TCheck='yes'
            else:TCheck='no'
            surfaces=ff[5][:-1].split(',')
            surfaces.sort()
        else:
            print 'Error!! Probe surface is not set'

        aa=glob.glob(self.caseDir+'/postProcessing/patchAverage_'+surfaces[0]+'/*')
        times=[]
        for ii in aa:
            bb=ii.split('/');times.append(bb[-1])
        times.sort()
        lastTime=times[-1]

        postdir=glob.glob(self.caseDir+'/postProcessing/patchAverage_*')
        postdir.sort()
        j=0
        for ii in postdir:
            j=j+1
            plotp="plot '"+ii+'/'+lastTime+"/p' u 1:2 w l title 'p'\n"
            plotprgh="plot '"+ii+'/'+lastTime+"/p_rgh' u 1:2 w l title 'p_rgh'\n"
            plott="plot '"+ii+'/'+lastTime+"/T' u 1:2 w l title 'T'\n"
            plotu="plot '"+ii+'/'+lastTime+"/Umag' u 1:2 w l title 'Umag'\n"
            gpstate="state=system(\"cat "+self.caseDir+"/PyFoamState.TheState\")\n"
            gpreread="if(state ne \"Finished\") pause 1; reread;\n"

            if pCheck=='yes':
                f=open(self.caseDir+'/system/settings/plotcmd_psurf'+str(j),'w')
                f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Surface Probe - "+surfaces[j-1]+"' \n")
                f.write(gpstate);f.write(plotp);f.write(gpreread);f.close()
                pidfile = self.caseDir+'/system/settings/surfprobecmdp_pid'+str(j)
                os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmd_psurf'+str(j)+' |echo $! > '+pidfile+' &')
                time.sleep(0.5)
            if prghCheck=='yes':
                f=open(self.caseDir+'/system/settings/plotcmd_prghsurf'+str(j),'w')
                f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Surface Probe - "+surfaces[j-1]+"' \n")
                f.write(gpstate);f.write(plotprgh);f.write(gpreread);f.close()
                pidfile = self.caseDir+'/system/settings/surfprobecmdprgh_pid'+str(j)
                os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmd_prghsurf'+str(j)+' |echo $! > '+pidfile+' &')
                time.sleep(0.5)                
            if UCheck=='yes':
                f=open(self.caseDir+'/system/settings/plotcmd_usurf'+str(j),'w')
                f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Surface Probe - "+surfaces[j-1]+"' \n")
                f.write(gpstate);f.write(plotu);f.write(gpreread);f.close()
                pidfile = self.caseDir+'/system/settings/surfprobecmdu_pid'+str(j)
                os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmd_usurf'+str(j)+' |echo $! > '+pidfile+' &')
                time.sleep(0.5)
            if TCheck=='yes':
                f=open(self.caseDir+'/system/settings/plotcmd_tsurf'+str(j),'w')
                f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Surface Probe - "+surfaces[j-1]+"' \n")
                f.write(gpstate);f.write(plott);f.write(gpreread);f.close()
                pidfile = self.caseDir+'/system/settings/surfprobecmdt_pid'+str(j)
                os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmd_tsurf'+str(j)+' |echo $! > '+pidfile+' &')
                time.sleep(0.5)             
    #------------------------------------------------------------------------------------------
    def closeplotprobesurface(self):
        a1=glob.glob(self.caseDir+'/system/settings/probesurface')
        statefile = open(self.caseDir+'/PyFoamState.TheState');state = statefile.read();statefile.close()

        a1=glob.glob(self.caseDir+'/system/settings/probesurface')
        if len(a1)!=0:
            f=open(self.caseDir+'/system/settings/probesurface','r');ff=f.readlines();f.close()
            if ff[0][:-1]=='True':pCheck='yes'
            else:pCheck='no'
            if ff[1][:-1]=='True':prghCheck='yes'
            else:prghCheck='no'
            if ff[2][:-1]=='True':UCheck='yes'
            else:UCheck='no'
            if ff[3][:-1]=='True':TCheck='yes'
            else:TCheck='no'
            surfaceNo=len(ff[5][:-1].split(','))
        else:
            print 'Error!! Probe surface is not set'

        if pCheck=='yes':
            for i in range(surfaceNo):
                pidfile = open(self.caseDir+'/system/settings/surfprobecmdp_pid'+str(i+1))
                ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)
                else: os.system('kill -9 '+pid)
        if prghCheck=='yes':
            for i in range(surfaceNo):
                pidfile = open(self.caseDir+'/system/settings/surfprobecmdprgh_pid'+str(i+1))
                ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)
                else: os.system('kill -9 '+pid)                
        if UCheck=='yes':
            for i in range(surfaceNo):
                pidfile = open(self.caseDir+'/system/settings/surfprobecmdu_pid'+str(i+1))
                ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)
                else: os.system('kill -9 '+pid)
        if TCheck=='yes':
            for i in range(surfaceNo):
                pidfile = open(self.caseDir+'/system/settings/surfprobecmdt_pid'+str(i+1))
                ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
                if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)
                else: os.system('kill -9 '+pid)
    #------------------------------------------------------------------------------------------
    def surfaceProbe(self,mainwindow,plotButton,maintextbuffer):
    
        GEN=generalClass(self.caseDir)

        def start(widget,data=None):

            selp=self.pCheck.get_active();selprgh=self.prghCheck.get_active()
            selu=self.uCheck.get_active();selT=self.TCheck.get_active()
            inter=self.interval.get_text()
	    
            selectedSet=[];tf=[]
            for i in range(len(bcName)):
                aa=setName[i].get_active()		   
                if aa==True:selectedSet.append(bcName[i])	

            if len(selectedSet)==0:
                aa=glob.glob(self.caseDir+'/system/surfaceProbeFunction')
                if len(aa)!=0:os.system('rm '+self.caseDir+'/system/surfaceProbeFunction')
                bb=glob.glob(self.caseDir+'/system/settings/probesurface')
                if len(bb)!=0:os.system('rm '+self.caseDir+'/system/settings/probesurface')                
                self.Win.destroy();self.openSubWindow=False
                GEN.makeDialog('Surface average monitoring is canceled')
                plotButton.set_sensitive(0)
                
            else:
                if selp==0 and selprgh==0 and selu==0 and selT==0:
                    GEN.makeDialog('Error! No field is selected.')
                else:                
                    f=open(self.caseDir+'/system/surfaceProbeFunction','w')
                    for i in range(len(selectedSet)):
                        f.write('    '+selectedSet[i]+'\n')
                        f.write('    {\n')
                        f.write('        functionObjectLibs ( "libsimpleFunctionObjects.so" );\n')
                        f.write('        type            patchAverage;\n')
                        f.write('        verbose         true;\n')	
                        f.write('        outputControlMode   timeStep;\n')
                        f.write('        outputInterval  '+inter+';\n')
                        f.write('        factor          1;\n')
                        f.write('\n')
                        f.write('        fields\n')
                        f.write('        (\n')
                        if selp==True:f.write('            p\n')
                        if selprgh==True:f.write('            p_rgh\n')
                        if selu==True:f.write('            U\n')
                        if selT==True:f.write('            T\n')
                        f.write('        );\n')
                        f.write('\n')
                        f.write('        patches\n')
                        f.write('        (\n')
                        f.write('            '+selectedSet[i]+'\n')
                        f.write('        );\n')
                        f.write('    }\n') 
                        f.write('\n')
                    f.close()

                    ff=open(self.caseDir+'/system/settings/probesurface','w')
                    if selp==True:ff.write('True'+'\n')
                    else:ff.write('False'+'\n')
                    if selprgh==True:ff.write('True'+'\n')
                    else:ff.write('False'+'\n')
                    if selu==True:ff.write('True'+'\n')
                    else:ff.write('False'+'\n')
                    if selT==True:ff.write('True'+'\n')
                    else:ff.write('False'+'\n')
                    ff.write(inter+'\n')
                    for ii in selectedSet:ff.write(ii+',')
                    ff.close()
                    plotButton.set_sensitive(1)	

                    self.Win.destroy();self.openSubWindow=False
                    
                    message='\n [  Surface average monitoring is set  ]\n'
                    for i in range(len(selectedSet)):
                        message=message+'\n    - '+selectedSet[i]+' for '
                        if selp==True:message=message+'  p  '
                        if selprgh==True:message=message+'  p_rgh  '
                        if selu==True:message=message+'  U  '
                        if selT==True:message=message+'  T  '
                        message=message+'\n'
                    
                    enditer=maintextbuffer.get_end_iter()
                    maintextbuffer.insert(enditer,message)

        def stop(widget,data=None):
            self.Win.destroy();self.openSubWindow=False

        self.openSubWindow=True
        
        self.Win=gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.Win.connect('delete_event',self.delete_event);self.Win.connect('destroy',self.destroy)
        self.Win.set_border_width(10);self.Win.set_title('Set Surface to probe')
        self.Win.set_position(gtk.WIN_POS_CENTER);self.Win.set_transient_for(mainwindow)

        GEN=generalClass(self.caseDir);bcName=GEN.getBCName()

        self.vbox=gtk.VBox(False,5);self.varBox=gtk.VBox(False,5)
        self.regbox=gtk.HBox(False,5);self.vbox.pack_start(self.regbox,False,False,5)	
        self.varFrame=gtk.Frame()
        self.varFrame.set_label('Probe variables');self.varFrame.set_label_align(0.0,1.0)
        self.varFrame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
        
        self.Win.add(self.vbox);self.vbox.pack_start(self.varFrame,False,False,5)
        self.varFrame.add(self.varBox)
        self.pCheck=gtk.CheckButton('p')
        self.prghCheck=gtk.CheckButton('p_rgh',use_underline=False)
        self.uCheck=gtk.CheckButton('U')
        self.TCheck=gtk.CheckButton('T')
        self.varBox.pack_start(self.pCheck,False,False,5);self.varBox.pack_start(self.prghCheck,False,False,5)
        self.varBox.pack_start(self.uCheck,False,False,5);self.varBox.pack_start(self.TCheck,False,False,5)
        if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':
            self.TCheck.set_sensitive(0);self.prghCheck.set_sensitive(0)
	    #------------------------------
        self.interBox=gtk.HBox(False,5);self.label=gtk.Label('Output interval')
        self.interval=gtk.Entry();self.interval.set_text('1');self.interval.set_size_request(100,28)

        self.vbox.pack_start(self.interBox,False,False,5)
        self.interBox.pack_start(self.label,False,False,5);self.interBox.pack_start(self.interval,False,False,5)

        self.locFrame=gtk.Frame();self.locFrame.set_label('Probe Surfaces');self.locFrame.set_label_align(0.0,1.0)
        self.locFrame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)

        self.vbox.pack_start(self.locFrame,False,False,5)
        self.swin=gtk.ScrolledWindow();self.swin.set_border_width(5);self.swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        self.locFrame.add(self.swin);itembox=gtk.VBox(False,0);self.swin.add_with_viewport(itembox)
        scheight=len(bcName)*38
        if scheight > 750:scheight=750        
        self.swin.set_size_request(300,500)

        setName=[];hbox1=[]
	    
        for i in range(len(bcName)):
            hbox1.append(gtk.HBox(False,5));itembox.pack_start(hbox1[i],False,False,2)                
            setName.append(gtk.CheckButton(bcName[i]));setName[i].set_size_request(180,28)	    
            hbox1[i].pack_start(setName[i],False,False,0)
        #-----------------------------------
        self.buttonBox=gtk.HBox(False,5)
        self.vbox.pack_start(self.buttonBox,False,False,5)

        self.appB=gtk.Button('Apply');self.closeB=gtk.Button('Cancel')
        self.appB.set_size_request(100,28);self.closeB.set_size_request(100,28)
        self.appB.connect('clicked',start);self.closeB.connect('clicked',stop)
        self.buttonBox.pack_end(self.closeB,False,False,5);self.buttonBox.pack_end(self.appB,False,False,5)
        #-----------------------------------
        ggg=glob.glob(self.caseDir+'/system/settings/probesurface')
        if len(ggg)!=0:
            f=open(self.caseDir+'/system/settings/probesurface','r');ff=f.readlines();f.close()
            if len(ff)>2:
                if ff[0][:-1]=='True':self.pCheck.set_active(1)
                else:self.pCheck.set_active(0)
                if ff[1][:-1]=='True':self.prghCheck.set_active(1)
                else:self.prghCheck.set_active(0)
                if ff[2][:-1]=='True':self.uCheck.set_active(1)
                else:self.uCheck.set_active(0)
                if ff[3][:-1]=='True':self.TCheck.set_active(1)
                else:self.TCheck.set_active(0)
                self.interval.set_text(ff[4][:-1])
                q1=ff[5][:-1];patches=q1.split(',')
                activepatches=[]
                for ii in patches:activepatches.append(bcName.index(ii))
                for i in range(len(activepatches)):setName[activepatches[i]].set_active(True)
            else:
               	self.Win.show_all()
                print '!!!! previous point probe setting has some problem'
                md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Error loading previous setting")
                md.run()
                md.destroy()

        self.Win.show_all()        
    #------------------------------------------------------------------------------------------
    def plotflowrate(self):

        a1=glob.glob(self.caseDir+'/system/settings/flowrate')
        if len(a1)!=0:
            f=open(self.caseDir+'/system/settings/flowrate','r');ff=f.readlines();f.close()
            aa=ff[0][:-1];patches=aa.split(',')
        else:
            print 'Error!! Probe point is not set'

        aa=glob.glob(self.caseDir+'/postProcessing/patchMassFlows_flowrate/*')
        times=[]
        for ii in aa:
            bb=ii.split('/')
            times.append(bb[-1])
        times.sort()
        lastTime=times[-1]

        gpstate="state=system(\"cat "+self.caseDir+"/PyFoamState.TheState\")\n"
        gpreread="if(state ne \"Finished\") pause 1; reread;\n"

        f=open(self.caseDir+'/system/settings/plotcmd_fr','w')
        f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Flow rate [kg/s]'\n")
        f.write(gpstate)
        path=self.caseDir+'/postProcessing/patchMassFlows_flowrate/'
        f.write("plot ")
        npatch=len(patches)
        for i in range(len(patches)):
            if i+1!=len(patches):
                f.write("'"+path+lastTime+"/massFlow' u 1:"+str(i+2)+" w l title '"+patches[npatch-i-1]+"',")#160617
            else:
                f.write("'"+path+lastTime+"/massFlow' u 1:"+str(i+2)+" w l title '"+patches[npatch-i-1]+"'")#
        f.write('\n');f.write(gpreread);f.close()

        pidfile = self.caseDir+'/system/settings/flowratecmd_pid'
        os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmd_fr |echo $! > '+pidfile+' &')
        time.sleep(0.5)
    #------------------------------------------------------------------------------------------
    def closeplotflowrate(self):
        statefile = open(self.caseDir+'/PyFoamState.TheState');state = statefile.read();statefile.close()
        pidfile = open(self.caseDir+'/system/settings/flowratecmd_pid')
        ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
        if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)
        else: os.system('kill -9 '+pid)
    #------------------------------------------------------------------------------------------
    def flowrate(self,mainwindow,plotButton,maintextbuffer):

        def start(widget,data=None):    
            selectedSet=[];tf=[]
            for i in range(len(bcName)):
        	aa=setName[i].get_active()		   
        	if aa==True:selectedSet.append(bcName[i])		

            if len(selectedSet)==0:
                aa=glob.glob(self.caseDir+'/system/flowRateFunction')
                if len(aa)!=0:os.system('rm '+self.caseDir+'/system/flowRateFunction')
                bb=glob.glob(self.caseDir+'/system/settings/flowrate')
                if len(bb)!=0:os.system('rm '+self.caseDir+'/system/settings/flowrate')
                plotButton.set_sensitive(0)
            else:
                f=open(self.caseDir+'/system/flowRateFunction','w')
                f.write('    flowrate\n')
                f.write('    {\n')
                f.write('        functionObjectLibs ( "libsimpleFunctionObjects.so" );\n')
                f.write('        type            patchMassFlow;\n')
                f.write('        verbose         true;\n')	
                f.write('        outputControlMode   timeStep;\n')
                f.write('        outputInterval  1;\n')
                f.write('        factor          1;\n')
                f.write('\n')
                f.write('\n')
                f.write('        patches\n')
                f.write('        (\n')
                for i in range(len(selectedSet)):f.write('            '+selectedSet[i]+'\n')
                f.write('        );\n')
                f.write('    }\n') 
                f.close()

                ff=open(self.caseDir+'/system/settings/flowrate','w')
                for ii in selectedSet:ff.write(ii+',')
                ff.close()
                plotButton.set_sensitive(1)
               
                message='\n [  Flowrate monitoring is set for the patch(es) ]\n'
                for i in range(len(selectedSet)):
                    message=message+'\n    - '+selectedSet[i]+'  '
                message=message+'\n'
                    
                enditer=maintextbuffer.get_end_iter()
                maintextbuffer.insert(enditer,message)
                	
            self.Win.destroy();self.openSubWindow=False

        def stop(widget,data=None):
            self.Win.destroy();self.openSubWindow=False

        self.openSubWindow=True
        
        self.Win=gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.Win.connect('delete_event',self.delete_event);self.Win.connect('destroy',self.destroy)
        self.Win.set_border_width(10);self.Win.set_title('Set Surface to check flowrate')
        self.Win.set_position(gtk.WIN_POS_CENTER);self.Win.set_transient_for(mainwindow)

        GEN=generalClass(self.caseDir);bcName=GEN.getBCName()

        self.vbox=gtk.VBox(False,5)
        self.Win.add(self.vbox)

        self.locFrame=gtk.Frame()
        self.locFrame.set_label('patches to check flowrate');self.locFrame.set_label_align(0.0,1.0)
        self.locFrame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)

        self.vbox.pack_start(self.locFrame,False,False,5)

        self.swin=gtk.ScrolledWindow();self.swin.set_border_width(5);self.swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        self.locFrame.add(self.swin)
        itembox=gtk.VBox(False,0)
        self.swin.add_with_viewport(itembox)
        scheight=len(bcName)*38
        if scheight > 750:scheight=750        
        self.swin.set_size_request(300,500)

        setName=[];hbox1=[]
	    
        for i in range(len(bcName)):
            hbox1.append(gtk.HBox(False,5));itembox.pack_start(hbox1[i],False,False,2)      
            setName.append(gtk.CheckButton(bcName[i]));setName[i].set_size_request(180,28)
            hbox1[i].pack_start(setName[i],False,False,0)
        #-----------------------------------
        self.buttonBox=gtk.HBox(False,5)
        self.vbox.pack_start(self.buttonBox,False,False,5)

        self.appB=gtk.Button('Apply');self.closeB=gtk.Button('Cancel')
        self.appB.set_size_request(100,28);self.closeB.set_size_request(100,28)
        self.appB.connect('clicked',start);self.closeB.connect('clicked',stop)
        self.buttonBox.pack_end(self.closeB,False,False,5);self.buttonBox.pack_end(self.appB,False,False,5)
        #-----------------------------------
        ggg=glob.glob(self.caseDir+'/system/settings/flowrate')
        if len(ggg)!=0:
            f=open(self.caseDir+'/system/settings/flowrate','r');ff=f.readlines();f.close()
            if len(ff)>0:
                q1=ff[0][:-1];patches=q1.split(',')
                activepatches=[]
                for ii in patches:activepatches.append(bcName.index(ii))
                for i in range(len(activepatches)):setName[activepatches[i]].set_active(True)
            else:
                self.Win.show_all()
                print '!!!! previous point probe setting has some problem'
                md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Error loading previous setting")
                md.run()
                md.destroy()

        self.Win.show_all() 
    #------------------------------------------------------------------------------------------
    def plotforce(self):
        aa=glob.glob(self.caseDir+'/postProcessing/forceCoeffs/*')
        times=[]
        for ii in aa:
            bb=ii.split('/');times.append(bb[-1])
        times.sort();lastTime=times[-1]

        gpstate="state=system(\"cat "+self.caseDir+"/PyFoamState.TheState\")\n"
        gpreread="if(state ne \"Finished\") pause 1; reread;\n"
        bb=glob.glob(self.caseDir+'/postProcessing/forceCoeffs/'+lastTime+'/forceCoeff*')
        bb.sort();forcefile=bb[-1]
        f=open(self.caseDir+'/system/settings/plotcmd_force','w')
        f.write('set terminal x11 1 noraise\n');f.write('set xlabel "time"\n');f.write("set title 'Force coeffs'\n")
        #f.write('set terminal x11 \n');f.write('set xlabel "time"\n');f.write("set title 'Force coeffs'\n")
        #f.write('set terminal wxt\n');f.write('set xlabel "time"\n');f.write("set title 'Force coeffs'\n")
        f.write(gpstate)
        f.write("plot '"+forcefile+"' u 1:3 w l title 'Cd'\n")
        f.write(gpreread);f.close()

        pidfile = self.caseDir+'/system/settings/forcecmd_pid'
        os.system('gnuplot -persist '+self.caseDir+'/system/settings/plotcmd_force |echo $! > '+pidfile+' &')
        time.sleep(0.5)
    #------------------------------------------------------------------------------------------
    def closeplotforce(self):
        statefile = open(self.caseDir+'/PyFoamState.TheState');state = statefile.read();statefile.close()
        pidfile = open(self.caseDir+'/system/settings/forcecmd_pid')
        ppid = pidfile.read();pidfile.close();pid = str(int(ppid)+2)
        if state.strip()=='Running': os.system('pkill -TERM -P '+ppid.strip()+' && kill -9 '+ppid)
        else: os.system('kill -9 '+pid)
    #------------------------------------------------------------------------------------------
    def forceSet(self,mainwindow,plotButton,maintextbuffer):
        GEN=generalClass(self.caseDir);bcnameori=GEN.getBCName();bctype=GEN.getBCType()
        bcname=[]
        for i in range(len(bctype)):
            if bctype[i]=='wall' or bctype[i]=='mappedWall':bcname.append(bcnameori[i])

        def makeforcefile(faces,rho,aref,uref,cofr,dragdir,liftdir):
            f=open(self.caseDir+'/system/forces','w')
            f.write('forces\n')
            f.write('{\n')
            f.write('    type        	forces;\n')
            f.write('    functionObjectLibs ( "libforces.so" );\n')
            f.write('    outputControl   	timeStep;\n')
            f.write('    timeInterval   	1;\n')
            f.write('    log         	yes;\n')
            f.write('    patches \n')
            f.write('    ( \n')
            for ii in faces:f.write('        '+ii+' \n')
            f.write('    ); \n')
            f.write('    pName       	p;\n')
            f.write('    UName       	U;\n')
            if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':f.write('    rhoName     	rhoInf; \n')
            else:f.write('    rhoName     	rho; \n')
            f.write('    log         	true;\n')
            f.write('    rhoInf      	'+rho+';  \n')
            f.write('    liftDir     	('+liftdir[0]+' '+liftdir[1]+' '+liftdir[2]+');\n')
            f.write('    dragDir     	('+dragdir[0]+' '+dragdir[1]+' '+dragdir[2]+');\n')
            f.write('    CofR        	('+cofr[0]+' '+cofr[1]+' '+cofr[2]+');\n')
            f.write('    pitchAxis   	(0 0 1);\n')
            f.write('    magUInf     	'+uref+';\n')
            f.write('    lRef        	1;\n')    
            f.write('    Aref        	'+aref+';\n')   
            f.write('}\n')
            f.close()
            f=open(self.caseDir+'/system/forceCoeffs','w')
            f.write('forceCoeffs\n')
            f.write('{\n')
            f.write('    type        	forceCoeffs;\n')
            f.write('    functionObjectLibs ( "libforces.so" );\n')
            f.write('    outputControl   	timeStep;\n')
            f.write('    timeInterval   	1;\n')
            f.write('    log         	yes;\n')
            f.write('    patches \n')
            f.write('    ( \n')
            for ii in faces:f.write('        '+ii+' \n')
            f.write('    ); \n')
            f.write('    pName       	p;\n')
            f.write('    UName       	U;\n')
            f.write('    rhoName     	rhoInf; \n')
            f.write('    log         	true;\n')
            f.write('    rhoInf      	'+rho+';  \n')
            f.write('    liftDir     	('+liftdir[0]+' '+liftdir[1]+' '+liftdir[2]+');\n')
            f.write('    dragDir     	('+dragdir[0]+' '+dragdir[1]+' '+dragdir[2]+');\n')
            f.write('    CofR        	('+cofr[0]+' '+cofr[1]+' '+cofr[2]+');\n')
            f.write('    pitchAxis   	(0 0 1);\n')
            f.write('    magUInf     	'+uref+';\n')
            f.write('    lRef        	1;        \n')
            f.write('    Aref        	'+aref+';\n')   
            f.write('}\n')
            f.close()

        def applythis(widget):
            rho=rhoEntry.get_text();aref=arefEntry.get_text();uref=urefEntry.get_text()
            cofrx=cxEntry.get_text();cofry=cyEntry.get_text();cofrz=czEntry.get_text()
            dragx=dragxEntry.get_text();dragy=dragyEntry.get_text();dragz=dragzEntry.get_text()
            liftx=liftxEntry.get_text();lifty=liftyEntry.get_text();liftz=liftzEntry.get_text()
            if GEN.isNumber(rho)==False:GEN.makeDialog('Error!!! rho is not number')            
            if GEN.isNumber(aref)==False:GEN.makeDialog('Error!!! aref is not number')                        
            if GEN.isNumber(uref)==False:GEN.makeDialog('Error!!! uref is not number')                        
            if GEN.isNumber(cofrx)==False:GEN.makeDialog('Error!!! cofrx is not number')                        
            if GEN.isNumber(cofry)==False:GEN.makeDialog('Error!!! cofry is not number')                        
            if GEN.isNumber(cofrz)==False:GEN.makeDialog('Error!!! cofrz is not number')                        
            if GEN.isNumber(dragx)==False:GEN.makeDialog('Error!!! dragx is not number')                        
            if GEN.isNumber(dragy)==False:GEN.makeDialog('Error!!! dragy is not number')                        
            if GEN.isNumber(dragz)==False:GEN.makeDialog('Error!!! dragz is not number')                        
            if GEN.isNumber(liftx)==False:GEN.makeDialog('Error!!! liftx is not number')                        
            if GEN.isNumber(lifty)==False:GEN.makeDialog('Error!!! lifty is not number')                        
            if GEN.isNumber(liftz)==False:GEN.makeDialog('Error!!! liftz is not number')
            
            cofr=[cofrx,cofry,cofrz];dragdir=[dragx,dragy,dragz];liftdir=[liftx,lifty,liftz]
            onOff=[]
            for i in range(len(bcname)):
                vvv=checkBCs[i].get_active();onOff.append(vvv)
            faces=[]
            for i in range(len(bcname)):
                if onOff[i]==1:faces.append(bcname[i])
            if len(faces)!=0:
                f=open(self.caseDir+'/system/settings/forceMonitor','w')
                for ii in faces:f.write(ii+' ')
                f.write('\n')
                f.write(rho+' '+aref+' '+uref+' '+cofr[0]+' '+cofr[1]+' '+cofr[2]+' '+dragdir[0]+' '+dragdir[1]+' '+dragdir[2]+' '+liftdir[0]+' '+liftdir[1]+' '+liftdir[2]+'\n')
                f.close()
                makeforcefile(faces,rho,aref,uref,cofr,dragdir,liftdir)
                plotButton.set_sensitive(1)
                
                message='\n [  Force monitoring is set ]\n'
                message=message+'\n    - patch(es) : '
                for ii in faces:message=message+'  '+ii
                message=message+'\n'
                message=message+'\n    - Reference Area is '+aref+'\n'
                message=message+'\n    - Reference Velocity is '+uref+'\n'
                message=message+'\n    - Center of Rotation is ( '+cofr[0]+' '+cofr[1]+' '+cofr[2]+' )\n'
                message=message+'\n    - Drag Direction is ( '+dragdir[0]+' '+dragdir[1]+' '+dragdir[2]+' )\n'
                message=message+'\n    - Lift Direction is ( '+liftdir[0]+' '+liftdir[1]+' '+liftdir[2]+' )\n'
                    
                enditer=maintextbuffer.get_end_iter()
                maintextbuffer.insert(enditer,message)               
                
            else:
                os.system('rm '+self.caseDir+'/system/settings/forceMonitor');os.system('rm '+self.caseDir+'/system/force*')
                plotButton.set_sensitive(0)
            mwin.destroy();self.openSubWindow=False
        
        def closethis(widget):
            mwin.destroy();self.openSubWindow=False

        self.openSubWindow=True
        onOff=[]
        for i in range(len(bcname)):onOff.append(0)

        mwin=gtk.Window(gtk.WINDOW_TOPLEVEL);mwin.set_border_width(10)
        mwin.set_title('Force');mwin.set_position(gtk.WIN_POS_CENTER);mwin.set_transient_for(mainwindow)
        mainbox=gtk.VBox(False,0);mwin.add(mainbox)

        vbox=gtk.VBox(False,5);mainbox.pack_start(vbox,False,False,5)
        swbc = gtk.ScrolledWindow();swbc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        swbc.set_size_request(300,300);bcbox=gtk.VBox(False,5);swbc.add_with_viewport(bcbox)
        vbox.pack_start(swbc,True,True,5)

        bclabel=gtk.Label('Select patches for forces');bcbox.pack_start(bclabel,False,False,5)
        checkBCs=[]
        for i in range(len(bcname)):
            checkBCs.append(gtk.CheckButton(bcname[i]))
            bcbox.pack_start(checkBCs[i],False,False,5)

        frame=gtk.Frame();frame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
        vbox.pack_start(frame,False,False,5);vvbox=gtk.VBox(False,5);frame.add(vvbox)

        rhobox=gtk.HBox(False,5);vvbox.pack_start(rhobox,True,True,5)
        label=gtk.Label('Density [kg/m3]')
        rhoEntry=gtk.Entry();rhoEntry.set_size_request(60,28);rhoEntry.set_text('1.2')
        rhobox.pack_start(label,False,False,5);rhobox.pack_end(rhoEntry,False,False,5)
        if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':rhoEntry.set_sensitive(1)
        else:rhoEntry.set_sensitive(0)

        arefbox=gtk.HBox(False,5);vvbox.pack_start(arefbox,False,False,5)
        label=gtk.Label('Reference area [m2]')
        arefEntry=gtk.Entry();arefEntry.set_size_request(60,28);arefEntry.set_text('1')
        arefbox.pack_start(label,False,False,5);arefbox.pack_end(arefEntry,False,False,5)

        urefbox=gtk.HBox(False,5);vvbox.pack_start(urefbox,False,False,5)
        label=gtk.Label('Reference velocity [m/s]')
        urefEntry=gtk.Entry();urefEntry.set_size_request(60,28);urefEntry.set_text('1')
        urefbox.pack_start(label,False,False,5);urefbox.pack_end(urefEntry,False,False,5)

        cofrbox=gtk.HBox(False,5);vvbox.pack_start(cofrbox,False,False,5)
        label=gtk.Label('CofR')
        cxEntry=gtk.Entry();cxEntry.set_size_request(60,28);cxEntry.set_text('0')
        cyEntry=gtk.Entry();cyEntry.set_size_request(60,28);cyEntry.set_text('0')
        czEntry=gtk.Entry();czEntry.set_size_request(60,28);czEntry.set_text('0')
        cofrbox.pack_start(label,False,False,5);cofrbox.pack_end(czEntry,False,False,5)
        cofrbox.pack_end(cyEntry,False,False,0);cofrbox.pack_end(cxEntry,False,False,5)

        dragdirbox=gtk.HBox(False,5);vvbox.pack_start(dragdirbox,False,False,5)
        label=gtk.Label('Drag direction')
        dragxEntry=gtk.Entry();dragxEntry.set_size_request(60,28);dragxEntry.set_text('1')
        dragyEntry=gtk.Entry();dragyEntry.set_size_request(60,28);dragyEntry.set_text('0')
        dragzEntry=gtk.Entry();dragzEntry.set_size_request(60,28);dragzEntry.set_text('0')
        dragdirbox.pack_start(label,False,False,5);dragdirbox.pack_end(dragzEntry,False,False,5)
        dragdirbox.pack_end(dragyEntry,False,False,0);dragdirbox.pack_end(dragxEntry,False,False,5)

        liftdirbox=gtk.HBox(False,5);vvbox.pack_start(liftdirbox,False,False,5)
        label=gtk.Label('Lift direction')
        liftxEntry=gtk.Entry();liftxEntry.set_size_request(60,28);liftxEntry.set_text('0')
        liftyEntry=gtk.Entry();liftyEntry.set_size_request(60,28);liftyEntry.set_text('0')
        liftzEntry=gtk.Entry();liftzEntry.set_size_request(60,28);liftzEntry.set_text('1')
        liftdirbox.pack_start(label,False,False,5);liftdirbox.pack_end(liftzEntry,False,False,5)
        liftdirbox.pack_end(liftyEntry,False,False,0);liftdirbox.pack_end(liftxEntry,False,False,5)

        clsB=gtk.Button(label='Close');clsB.connect('clicked',closethis);clsB.set_size_request(100,28)
        appB=gtk.Button(label='Apply');appB.connect('clicked',applythis);appB.set_size_request(100,28)
        buttonbox=gtk.HBox(False,5);mainbox.pack_start(buttonbox,False,False,5)
        buttonbox.pack_end(clsB,False,False,5);buttonbox.pack_end(appB,False,False,5)

        aa=glob.glob(self.caseDir+'/system/settings/forceMonitor')
        if len(aa)!=0:
            f=open(aa[0],'r');ff=f.readlines();f.close()
            patchs=ff[0][:-2].split(' ')
            for i in range(len(patchs)):
                bb=bcname.index(patchs[i]);checkBCs[bb].set_active(1)
            rr=ff[1][:-1].split(' ')
            rhoEntry.set_text(rr[0]);arefEntry.set_text(rr[1]);urefEntry.set_text(rr[2])
            cxEntry.set_text(rr[3]);cyEntry.set_text(rr[4]);czEntry.set_text(rr[5])
            dragxEntry.set_text(rr[6]);dragyEntry.set_text(rr[7]);dragzEntry.set_text(rr[8])
            liftxEntry.set_text(rr[9]);liftyEntry.set_text(rr[10]);liftzEntry.set_text(rr[11])

        mwin.show_all() 
    #------------------------------------------------------------------------------------------
    def openForce(self,mainwindow):
        GEN=generalClass(self.caseDir);bcnameori=GEN.getBCName();bctype=GEN.getBCType()
        bcname=[]
        for i in range(len(bctype)):
            if bctype[i]=='wall' or bctype[i]=='mappedWall':bcname.append(bcnameori[i])

        def force(widget):
            rho=rhoEntry.get_text();aref=arefEntry.get_text();uref=urefEntry.get_text()
            cofrx=cxEntry.get_text();cofry=cyEntry.get_text();cofrz=czEntry.get_text()
            dragx=dragxEntry.get_text();dragy=dragyEntry.get_text();dragz=dragzEntry.get_text()
            liftx=liftxEntry.get_text();lifty=liftyEntry.get_text();liftz=liftzEntry.get_text()
            if GEN.isNumber(rho)==False:GEN.makeDialog('Error!!! rho is not number')            
            if GEN.isNumber(aref)==False:GEN.makeDialog('Error!!! aref is not number')            
            if GEN.isNumber(uref)==False:GEN.makeDialog('Error!!! uref is not number')            
            if GEN.isNumber(cofrx)==False:GEN.makeDialog('Error!!! cofrx is not number')            
            if GEN.isNumber(cofry)==False:GEN.makeDialog('Error!!! cofry is not number')            
            if GEN.isNumber(cofrz)==False:GEN.makeDialog('Error!!! cofrz is not number')            
            if GEN.isNumber(dragx)==False:GEN.makeDialog('Error!!! dragx is not number')            
            if GEN.isNumber(dragy)==False:GEN.makeDialog('Error!!! dragy is not number')                        
            if GEN.isNumber(dragz)==False:GEN.makeDialog('Error!!! dragz is not number')                        
            if GEN.isNumber(liftx)==False:GEN.makeDialog('Error!!! liftx is not number')                        
            if GEN.isNumber(lifty)==False:GEN.makeDialog('Error!!! lifty is not number')                        
            if GEN.isNumber(liftz)==False:GEN.makeDialog('Error!!! liftz is not number')
            
            cofr=[cofrx,cofry,cofrz];dragdir=[dragx,dragy,dragz];liftdir=[liftx,lifty,liftz]

            onOff=[]
            for i in range(len(bcname)):
                vvv=checkBCs[i].get_active();onOff.append(vvv)

            forceDic=self.reportForce(bcname,onOff,rho,cofr,dragdir,liftdir,uref,aref)

            faces=forceDic.keys()
            strings=[];stringf=''
            string0='# -- Pressure Force x,y,z --- \n'
            ssx=0;ssy=0;ssz=0;pressureForce=[]
            for i in range(len(faces)):
                ss=forceDic[faces[i]];pf=ss[0]
                pressureForce.append([pf[0],pf[1],pf[2]])
                string=faces[i]+' : '+pf[0]+'   '+pf[1]+'   '+pf[2]+' \n'
                strings.append(string)
                ssx=ssx+float(pf[0]);ssy=ssy+float(pf[1]);ssz=ssz+float(pf[2])		
            for i in range(len(faces)):stringf=stringf+strings[i]
            stringsum='total : '+str(ssx)+'    '+str(ssy)+'    '+str(ssz)+' \n'		
            stringpf=string0+stringf+stringsum
            pressureForceTotal=[ssx,ssy,ssz]

            strings=[];stringf=''
            string0='# -- Viscous Force x,y,z --- \n'
            ssx=0;ssy=0;ssz=0;viscousForce=[]
            for i in range(len(faces)):
                ss=forceDic[faces[i]];vf=ss[1]
                viscousForce.append([vf[0],vf[1],vf[2]])
                string=faces[i]+' : '+vf[0]+'   '+vf[1]+'   '+vf[2]+' \n'
                strings.append(string)
                ssx=ssx+float(vf[0]);ssy=ssy+float(vf[1]);ssz=ssz+float(vf[2])
            for i in range(len(faces)):stringf=stringf+strings[i]
            stringsum='total : '+str(ssx)+'    '+str(ssy)+'    '+str(ssz)+' \n'
            stringvf=string0+stringf+stringsum
            viscousForceTotal=[ssx,ssy,ssz]

            strings=[];stringf=''
            string0='# -- Pressure Moment x,y,z --- \n'
            ssx=0;ssy=0;ssz=0;pressureMoment=[]
            for i in range(len(faces)):
                ss=forceDic[faces[i]];pm=ss[2]
                pressureMoment.append([pm[0],pm[1],pm[2]])
                string=faces[i]+' : '+pm[0]+'   '+pm[1]+'   '+pm[2]+' \n'
                strings.append(string)
                ssx=ssx+float(pm[0]);ssy=ssy+float(pm[1]);ssz=ssz+float(pm[2])
            for i in range(len(faces)):stringf=stringf+strings[i]
            stringsum='total : '+str(ssx)+'    '+str(ssy)+'    '+str(ssz)+' \n'
            stringpm=string0+stringf+stringsum
            pressureMomentTotal=[ssx,ssy,ssz]		

            strings=[];stringf=''
            string0='# -- Viscous Moment x,y,z --- \n'
            ssx=0;ssy=0;ssz=0;viscousMoment=[]
            for i in range(len(faces)):
                ss=forceDic[faces[i]];vm=ss[3]
                viscousMoment.append([vm[0],vm[1],vm[2]])
                string=faces[i]+' : '+vm[0]+'   '+vm[1]+'   '+vm[2]+' \n'
                strings.append(string)
                ssx=ssx+float(vm[0]);ssy=ssy+float(vm[1]);ssz=ssz+float(vm[2])
            for i in range(len(faces)):stringf=stringf+strings[i]
            stringsum='total : '+str(ssx)+'    '+str(ssy)+'    '+str(ssz)+' \n'
            stringvm=string0+stringf+stringsum
            viscousMomentTotal=[ssx,ssy,ssz]

            string0='# -- Total Force x,y,z --- \n'
            xForceTotal=pressureForceTotal[0]+viscousForceTotal[0]
            yForceTotal=pressureForceTotal[1]+viscousForceTotal[1]
            zForceTotal=pressureForceTotal[2]+viscousForceTotal[2]
            stringForce=str(xForceTotal)+'    '+str(yForceTotal)+'    '+str(zForceTotal)+'\n'
            stringTotalForce=string0+stringForce

            string0='# -- Total Moment x,y,z --- \n'
            xMomentTotal=pressureMomentTotal[0]+viscousMomentTotal[0]
            yMomentTotal=pressureMomentTotal[1]+viscousMomentTotal[1]
            zMomentTotal=pressureMomentTotal[2]+viscousMomentTotal[2]
            stringMoment=str(xMomentTotal)+'    '+str(yMomentTotal)+'    '+str(zMomentTotal)+'\n'
            stringTotalMoment=string0+stringMoment

            stringtot=stringpf+'\n'+stringvf+'\n'+stringpm+'\n'+stringvm+'\n'+stringTotalForce+'\n'+stringTotalMoment
            textbuffer.set_text(stringtot)

            faces=[]
            for i in range(len(bcname)):
                if onOff[i]==1:faces.append(bcname[i])
            if len(faces)!=0:
                f=open(self.caseDir+'/system/settings/forcereport','w')
                for ii in faces:f.write(ii+' ')
                f.write('\n')
                f.write(rho+' '+aref+' '+uref+' '+cofr[0]+' '+cofr[1]+' '+cofr[2]+' '+dragdir[0]+' '+dragdir[1]+' '+dragdir[2]+' '+liftdir[0]+' '+liftdir[1]+' '+liftdir[2]+'\n')
                f.close()
            else:
                os.system('rm '+self.caseDir+'/system/settings/forcereport')

        def forceCoeff(widget):
            rho=rhoEntry.get_text();aref=arefEntry.get_text();uref=urefEntry.get_text()
            cofrx=cxEntry.get_text();cofry=cyEntry.get_text();cofrz=czEntry.get_text()
            dragx=dragxEntry.get_text();dragy=dragyEntry.get_text();dragz=dragzEntry.get_text()
            liftx=liftxEntry.get_text();lifty=liftyEntry.get_text();liftz=liftzEntry.get_text()
            if GEN.isNumber(rho)==False:GEN.makeDialog('Error!!! rho is not number')            
            if GEN.isNumber(aref)==False:GEN.makeDialog('Error!!! aref is not number')            
            if GEN.isNumber(uref)==False:GEN.makeDialog('Error!!! uref is not number')            
            if GEN.isNumber(cofrx)==False:GEN.makeDialog('Error!!! cofrx is not number')            
            if GEN.isNumber(cofry)==False:GEN.makeDialog('Error!!! cofry is not number')            
            if GEN.isNumber(cofrz)==False:GEN.makeDialog('Error!!! cofrz is not number')            
            if GEN.isNumber(dragx)==False:GEN.makeDialog('Error!!! dragx is not number')            
            if GEN.isNumber(dragy)==False:GEN.makeDialog('Error!!! dragy is not number')            
            if GEN.isNumber(dragz)==False:GEN.makeDialog('Error!!! dragz is not number')            
            if GEN.isNumber(liftx)==False:GEN.makeDialog('Error!!! liftx is not number')            
            if GEN.isNumber(lifty)==False:GEN.makeDialog('Error!!! lifty is not number')            
            if GEN.isNumber(liftz)==False:GEN.makeDialog('Error!!! liftz is not number')
            
            cofr=[cofrx,cofry,cofrz];dragdir=[dragx,dragy,dragz];liftdir=[liftx,lifty,liftz]

            onOff=[]
            for i in range(len(bcname)):
                vvv=checkBCs[i].get_active();onOff.append(vvv)

            orix=cxEntry.get_text()
            if GEN.isNumber(orix)==False:GEN.makeDialog('Error!!! orix is not number')
            oriy=cyEntry.get_text()
            if GEN.isNumber(oriy)==False:GEN.makeDialog('Error!!! oriy is not number')
            oriz=czEntry.get_text()
            if GEN.isNumber(oriz)==False:GEN.makeDialog('Error!!! oriz is not number')
            
            forceCoeffList=self.reportForceCoeff(bcname,onOff,rho,cofr,dragdir,liftdir,uref,aref)

            string0='# -- Force Coefficients --- \n'
            string1='\n'+'A_ref : '+aref+'\n'
            string2='U_ref : '+uref+'\n'
            string3='rho_ref : '+rho+'\n'
            string4='center of rotation : ('+cofr[0]+' '+cofr[1]+' '+cofr[2]+')\n'
            cdstr='\n'+'Cd = '+forceCoeffList[0]+'\n';clstr='Cl = '+forceCoeffList[1]+'\n'
            cmstr='Cm = '+forceCoeffList[2]+'\n'
            string=string0+string1+string2+string3+string4+cdstr+clstr+cmstr
            textbuffer.set_text(string)

            faces=[]
            for i in range(len(bcname)):
                if onOff[i]==1:faces.append(bcname[i])
            if len(faces)!=0:
                f=open(self.caseDir+'/system/settings/forcereport','w')
                for ii in faces:f.write(ii+' ')
                f.write('\n')
                f.write(rho+' '+aref+' '+uref+' '+cofr[0]+' '+cofr[1]+' '+cofr[2]+' '+dragdir[0]+' '+dragdir[1]+' '+dragdir[2]+' '+liftdir[0]+' '+liftdir[1]+' '+liftdir[2]+'\n')
                f.close()
            else:
                os.system('rm '+self.caseDir+'/system/settings/forcereport')
        
        def closewin(widget):
            mwin.destroy()
            self.openSubWindow=False

        GEN=generalClass(self.caseDir)

        if self.openSubWindow==False:  

            onOff=[]
            for i in range(len(bcname)):onOff.append(0)

            mwin=gtk.Window(gtk.WINDOW_TOPLEVEL);mwin.set_border_width(10);mwin.set_title('Force report')
            mwin.set_position(gtk.WIN_POS_CENTER);mwin.set_transient_for(mainwindow)
            mainbox=gtk.HBox(False,0)
            leftbox=gtk.VBox(False,5)
            rightbox=gtk.VBox(False,10);rightbox.set_size_request(500,600);rightbox.set_border_width(10)
            bcbox=gtk.VBox(False,5)	  
            mwin.add(mainbox);mainbox.pack_start(leftbox,False,False,0);mainbox.pack_start(rightbox,False,False,0)

            swbc = gtk.ScrolledWindow();swbc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC);swbc.set_size_request(250,300)
            swbc.add_with_viewport(bcbox)

            leftbox.pack_start(swbc,True,True,5)

            bclabel=gtk.Label('Select patches for forces')
            bcbox.pack_start(bclabel,False,False,5)
            checkBCs=[]
            for i in range(len(bcname)):
                checkBCs.append(gtk.CheckButton(bcname[i]))
                bcbox.pack_start(checkBCs[i],False,False,5)

            rhobox=gtk.HBox(False,5);leftbox.pack_start(rhobox,True,True,2)
            label=gtk.Label('Density [kg/m3]')
            rhoEntry=gtk.Entry();rhoEntry.set_size_request(60,28);rhoEntry.set_text('1.2')
            rhobox.pack_start(label,False,False,5);rhobox.pack_end(rhoEntry,False,False,5)

            arefbox=gtk.HBox(False,5);leftbox.pack_start(arefbox,False,False,2)
            label=gtk.Label('Reference area [m2]')
            arefEntry=gtk.Entry();arefEntry.set_size_request(60,28);arefEntry.set_text('1')
            arefbox.pack_start(label,False,False,5);arefbox.pack_end(arefEntry,False,False,5)

            urefbox=gtk.HBox(False,5);leftbox.pack_start(urefbox,False,False,2)
            label=gtk.Label('Reference velocity [m/s]')
            urefEntry=gtk.Entry();urefEntry.set_size_request(60,28);urefEntry.set_text('1')
            urefbox.pack_start(label,False,False,5);urefbox.pack_end(urefEntry,False,False,5)

            cofrbox=gtk.HBox(False,5);leftbox.pack_start(cofrbox,False,False,2)
            label=gtk.Label('CofR')
            cxEntry=gtk.Entry();cxEntry.set_size_request(60,28);cxEntry.set_text('0')
            cyEntry=gtk.Entry();cyEntry.set_size_request(60,28);cyEntry.set_text('0')
            czEntry=gtk.Entry();czEntry.set_size_request(60,28);czEntry.set_text('0')
            cofrbox.pack_start(label,False,False,5);cofrbox.pack_end(czEntry,False,False,5)
            cofrbox.pack_end(cyEntry,False,False,0);cofrbox.pack_end(cxEntry,False,False,5)

            dragdirbox=gtk.HBox(False,5);leftbox.pack_start(dragdirbox,False,False,2)
            label=gtk.Label('Drag dir.')
            dragxEntry=gtk.Entry();dragxEntry.set_size_request(60,28);dragxEntry.set_text('1')
            dragyEntry=gtk.Entry();dragyEntry.set_size_request(60,28);dragyEntry.set_text('0')
            dragzEntry=gtk.Entry();dragzEntry.set_size_request(60,28);dragzEntry.set_text('0')
            dragdirbox.pack_start(label,False,False,5);dragdirbox.pack_end(dragzEntry,False,False,5)
            dragdirbox.pack_end(dragyEntry,False,False,0);dragdirbox.pack_end(dragxEntry,False,False,5)

            liftdirbox=gtk.HBox(False,5);leftbox.pack_start(liftdirbox,False,False,2)
            label=gtk.Label('Lift dir.')
            liftxEntry=gtk.Entry();liftxEntry.set_size_request(60,28);liftxEntry.set_text('0')
            liftyEntry=gtk.Entry();liftyEntry.set_size_request(60,28);liftyEntry.set_text('0')
            liftzEntry=gtk.Entry();liftzEntry.set_size_request(60,28);liftzEntry.set_text('1')
            liftdirbox.pack_start(label,False,False,5);liftdirbox.pack_end(liftzEntry,False,False,5)
            liftdirbox.pack_end(liftyEntry,False,False,0);liftdirbox.pack_end(liftxEntry,False,False,5)

            buttonbox=gtk.VBox(False,5);leftbox.pack_start(buttonbox,False,False,2)

            forceB=gtk.Button(label='force');forceB.connect('clicked',force);forceB.set_size_request(90,28)
            closebutton=gtk.Button(label='close');closebutton.connect('clicked',closewin)
            coeffB=gtk.Button(label='forceCoeff');coeffB.connect('clicked',forceCoeff)

            bubox1=gtk.HBox(False,5);buttonbox.pack_start(bubox1,False,False,5)
            bubox2=gtk.HBox(False,5);buttonbox.pack_start(bubox2,False,False,5)
            bubox1.pack_start(forceB,True,True,5);bubox1.pack_end(coeffB,True,True,5)
            bubox2.pack_end(closebutton,True,True,5)

            sw = gtk.ScrolledWindow();sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            textview = gtk.TextView()
            textbuffer = textview.get_buffer()
            sw.add(textview)
            rightbox.pack_start(sw,True,True,5)
            textbuffer.set_text('#-------------- force report---------------')

            aa=glob.glob(self.caseDir+'/system/settings/forcereport')
            if len(aa)!=0:
                f=open(aa[0],'r');ff=f.readlines();f.close()
                patchs=ff[0][:-2].split(' ')
                for i in range(len(patchs)):
                    bb=bcname.index(patchs[i]);checkBCs[bb].set_active(1)
                rr=ff[1][:-1].split(' ')
                rhoEntry.set_text(rr[0]);arefEntry.set_text(rr[1]);urefEntry.set_text(rr[2])
                cxEntry.set_text(rr[3]);cyEntry.set_text(rr[4]);czEntry.set_text(rr[5])
                dragxEntry.set_text(rr[6]);dragyEntry.set_text(rr[7]);dragzEntry.set_text(rr[8])
                liftxEntry.set_text(rr[9]);liftyEntry.set_text(rr[10]);liftzEntry.set_text(rr[11])

            mwin.show_all()
            
            self.openSubWindow=True
    #-------------------------------------------------------------------------------------
    def reportForce(self,bcname,onOff,rho,cofr,dragdir,liftdir,Uref,Aref):

        def kEpsilonRASProperties():
            f=open(self.caseDir+'/constant/RASProperties','w')
            f.write('FoamFile\n')
            f.write('{\n')
            f.write('    version     2.0;\n')
            f.write('    format      ascii;\n')
            f.write('    class       dictionary;\n')
            f.write('    location    "constant";\n')
            f.write('    object      RASProperties;\n')
            f.write('}\n')
            f.write('RASModel        kEpsilon;\n')
            f.write('turbulence      on;\n')
            f.write('printCoeffs     on;\n')
            f.close()
            
        faces=[]
        for i in range(len(bcname)):
            if onOff[i]==1:faces.append(bcname[i])

        forceList={}

        f=open(self.caseDir+'/system/execFlowFunctionDict','w')
        f.write('FoamFile \n')
        f.write('{ \n')
        f.write('    version    2.0; \n')
        f.write('    format     ascii; \n')
        f.write('    class      dictionary; \n')
        f.write('    object     execFlowFunctionDict; \n')
        f.write('} \n')
        f.write(' \n')
        f.write('functions \n')
        f.write('{ \n')
        f.write('    forces \n')
        f.write('    { \n')
        f.write('	     type          forces; \n')
        f.write('	     functionObjectLibs ( "libforces.so" ); \n')
        f.write('	     outputControl timeStep; \n')
        f.write('	     outputInterval 1; \n')
        f.write('	     patches \n')
        f.write('	     ( \n')
        for ii in faces:f.write('	     '+ii+' \n')
        f.write('	     ); \n')
        f.write('	     rhoName   rhoInf; \n')
        f.write('	     UName     U; \n')
        f.write('	     pName     p; \n')
        f.write('	     log       true; \n')
        f.write('	     rhoInf    '+rho+'; \n')
        f.write('	     CofR      ('+cofr[0]+' '+cofr[1]+' '+cofr[2]+'); \n')
        f.write('    } \n')
        f.write(' \n')
        f.write('} \n')
        f.close()

        pp=glob.glob(self.caseDir+'/processor*')
                
        if self.turmodel=='kOmegaSST' or self.turmodel=='laminar':
            if pp==[]:
                os.system('execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionDict -latestTime > '+self.caseDir+'/forceLog ')
            else:
                os.system('mpirun -np '+str(len(pp))+' execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionDict -latestTime -parallel > '+self.caseDir+'/forceLog ')
        else:        
            os.system('cp '+self.caseDir+'/constant/RASProperties '+self.caseDir+'/constant/oriRASProperties') # temporary 
            kEpsilonRASProperties() # temporary
            if pp==[]:
                os.system('execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionDict -latestTime > '+self.caseDir+'/forceLog ')
            else:
                os.system('mpirun -np '+str(len(pp))+' execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionDict -latestTime -parallel > '+self.caseDir+'/forceLog ')            
            os.system('cp '+self.caseDir+'/constant/oriRASProperties '+self.caseDir+'/constant/RASProperties') # temporary 
	
        f=open(self.caseDir+'/forceLog','r');ff=f.readlines();f.close()
        for j in range(len(ff)):
            aa=ff[j].strip()
            if aa[:13]=='sum of forces':
        	pforce=ff[j+1];vforce=ff[j+2]
            if aa[:14]=='sum of moments':
        	pmoment=ff[j+1];vmoment=ff[j+2]

        pf1=pforce.strip();pf2=pf1.split(':')[1];pf3=pf2.strip()[1:-1]
        pressureforce=pf3.split(' ')
        vf1=vforce.strip();vf2=vf1.split(':')[1];vf3=vf2.strip()[1:-1]
        viscousforce=vf3.split(' ')
        pm1=pmoment.strip();pm2=pm1.split(':')[1];pm3=pm2.strip()[1:-1]
        pressuremoment=pm3.split(' ')
        vm1=vmoment.strip();vm2=vm1.split(':')[1];vm3=vm2.strip()[1:-1]
        viscousmoment=vm3.split(' ')

        for ii in faces:forceList[ii]=[pressureforce,viscousforce,pressuremoment,viscousmoment]

        return forceList
    #-------------------------------------------------------------------------------------
    def reportForceCoeff(self,bcname,onOff,rho,cofr,dragdir,liftdir,Uref,Aref):

        def kEpsilonRASProperties():
            f=open(self.caseDir+'/constant/RASProperties','w')
            f.write('FoamFile\n')
            f.write('{\n')
            f.write('    version     2.0;\n')
            f.write('    format      ascii;\n')
            f.write('    class       dictionary;\n')
            f.write('    location    "constant";\n')
            f.write('    object      RASProperties;\n')
            f.write('}\n')
            f.write('RASModel        kEpsilon;\n')
            f.write('turbulence      on;\n')
            f.write('printCoeffs     on;\n')
            f.close()
            
        faces=[]
        for i in range(len(bcname)):
            if onOff[i]==1:faces.append(bcname[i])

        forceCoeffList=[]
        f=open(self.caseDir+'/system/execFlowFunctionsDictCoeff','w')
        f.write('FoamFile \n')
        f.write('{ \n')
        f.write('    version    2.0; \n')
        f.write('    format     ascii; \n')
        f.write('    class      dictionary; \n')
        f.write('    object     execFlowFunctionsDict; \n')
        f.write('} \n')
        f.write(' \n')
        f.write('functions \n')
        f.write('{ \n')
        f.write('forcesCoeffs \n')
        f.write('    { \n')
        f.write('        type          forceCoeffs; \n')
        f.write('        functionObjectLibs ( "libforces.so" ); \n')
        f.write('        outputControl timeStep; \n')
        f.write('        outputInterval 1; \n')
        f.write('        patches \n')
        f.write('        ( \n')
        for ii in faces:f.write('            '+ii+' \n')
        f.write('        ); \n')
        f.write('        rhoName     rhoInf; \n')
        f.write('        log         true; \n')
        f.write('        rhoInf      '+rho+'; \n')
        f.write('	 CofR        ('+cofr[0]+' '+cofr[1]+' '+cofr[2]+'); \n')
        f.write('        liftDir     ('+liftdir[0]+' '+liftdir[1]+' '+liftdir[2]+'); \n')
        f.write('        dragDir     ('+dragdir[0]+' '+dragdir[1]+' '+dragdir[2]+'); \n')
        f.write('        pitchAxis   (0 1 0); \n')
        f.write('        magUInf     '+Uref+'; \n')
        f.write('        lRef        1; \n')
        f.write('        Aref        '+Aref+'; \n')
        f.write('    } \n')
        f.write('} \n')
        f.close()

        pp=glob.glob(self.caseDir+'/processor*')
                
        if self.turmodel=='kOmegaSST' or self.turmodel=='laminar':
            if pp==[]:
                os.system('execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionsDictCoeff -latestTime > '+self.caseDir+'/forceCoeffLog ')
            else:
                os.system('mpirun -np '+str(len(pp))+' execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionsDictCoeff -latestTime -parallel > '+self.caseDir+'/forceCoeffLog ')            
        else:        
            os.system('cp '+self.caseDir+'/constant/RASProperties '+self.caseDir+'/constant/oriRASProperties') # temporary 
            kEpsilonRASProperties() # temporary
            if pp==[]:	
                os.system('execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionsDictCoeff -latestTime > '+self.caseDir+'/forceCoeffLog ')
            else:
                os.system('mpirun -np '+str(len(pp))+' execFlowFunctionObjects -case '+self.caseDir+' -dict '+self.caseDir+'/system/execFlowFunctionsDictCoeff -latestTime -parallel > '+self.caseDir+'/forceCoeffLog ')            
            os.system('cp '+self.caseDir+'/constant/oriRASProperties '+self.caseDir+'/constant/RASProperties') # temporary 
	
        f=open(self.caseDir+'/forceCoeffLog','r');ff=f.readlines();f.close()
        for j in range(len(ff)):
            if ff[j][:5]=='force':
                cd=ff[j+2];cl=ff[j+3];cm=ff[j+1]
        cds=cd.strip();cdst=cds.split();cdstr=cdst[2]
        cls=cl.strip();clst=cls.split();clstr=clst[2]
        cms=cm.strip();cmst=cms.split();cmstr=cmst[2]
        forceCoeffList=[cdstr,clstr,cmstr]

        return forceCoeffList
    #----------------------------------------------------------------------------------------------
    def patchIntegrate(self,mainwindow):

        def applythis(widget):
            patch=patchCombo.child.get_text()
            var=varCombo.child.get_text()
            
            aa=glob.glob(self.caseDir+'/processor*')
            if aa==[]:
                os.system('patchIntegrate -latestTime -case '+self.caseDir+' '+var+' '+patch+' >'+self.caseDir+'/log.patchIntegrate ')
            else:           
                os.system('mpirun -np '+str(len(aa))+' patchIntegrate -parallel -latestTime -case '+self.caseDir+' '+var+' '+patch+' >'+self.caseDir+'/log.patchIntegrate ')
                
            f=open(self.caseDir+'/log.patchIntegrate','r');ff=f.readlines();f.close()
            res=ff[24:-2];string=''
            for ii in res:string=string+ii
            textbuffer.set_text(string)	

        def closethis(widget):
            mwin.destroy()
            self.openSubWindow=False

        GEN=generalClass(self.caseDir);bcname=GEN.getBCName()
        
        if self.openSubWindow==False:  

            mwin=gtk.Window(gtk.WINDOW_TOPLEVEL);mwin.set_border_width(10);mwin.set_title('patch integrate')
            mwin.set_position(gtk.WIN_POS_CENTER);mwin.set_transient_for(mainwindow)
            mainbox=gtk.VBox(False,0);mwin.add(mainbox)

            hbox1=gtk.HBox(False,5);mainbox.pack_start(hbox1,False,False,5)
            hbox2=gtk.HBox(False,5);mainbox.pack_start(hbox2,False,False,5)
            hbox3=gtk.HBox(False,5);hbox3.set_size_request(650,200);mainbox.pack_start(hbox3,False,False,5)

            label=gtk.Label('patch to integrate');label.set_size_request(160,28)
            patchCombo=gtk.combo_box_entry_new_text();patchCombo.set_size_request(140,28)
            for ii in bcname:patchCombo.append_text(ii);patchCombo.set_active(0)
            hbox1.pack_start(label,False,False,5);hbox1.pack_start(patchCombo,False,False,5)

            label=gtk.Label('variable to integrate');label.set_size_request(160,28)
            varCombo=gtk.combo_box_entry_new_text();varCombo.set_size_request(140,28)
            if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':variables=['p','U','k','epsilon','omega']
            else:variables=['p_rgh','p','U','T','k','epsilon','omega']
            for ii in variables:varCombo.append_text(ii);varCombo.set_active(0)
            hbox2.pack_start(label,False,False,5);hbox2.pack_start(varCombo,False,False,5)	
	
            sw=gtk.ScrolledWindow();sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            textview = gtk.TextView()
            textbuffer = textview.get_buffer()
            sw.add(textview)
            hbox3.pack_start(sw,True,True,5)

            buttonbox=gtk.HBox(False,5);mainbox.pack_start(buttonbox,False,False,5)
            closeB=gtk.Button(label='close');closeB.connect('clicked',closethis);closeB.set_size_request(140,28)
            applyB=gtk.Button(label='Calculate');applyB.connect('clicked',applythis);applyB.set_size_request(140,28)
            buttonbox.pack_end(closeB,False,False,5);buttonbox.pack_end(applyB,False,False,5)

            mwin.show_all()
            
            self.openSubWindow=True
    #----------------------------------------------------------------------------------------------
    def patchAverage(self,mainwindow):

        def applythis(widget):
            patch=patchCombo.child.get_text()
            var=varCombo.child.get_text()
            
            aa=glob.glob(self.caseDir+'/processor*')
            if aa==[]:
                os.system('patchAverage -latestTime -case '+self.caseDir+' '+var+' '+patch+' >'+self.caseDir+'/log.patchAverage ')
            else:           
                os.system('mpirun -np '+str(len(aa))+' patchAverage -parallel -latestTime -case '+self.caseDir+' '+var+' '+patch+' >'+self.caseDir+'/log.patchAverage ')
                
            f=open(self.caseDir+'/log.patchAverage','r');ff=f.readlines();f.close()
            res=ff[24:-2];string=''
            for ii in res:string=string+ii
            textbuffer.insert_at_cursor(string)
            
        def closethis(widget):
            mwin.destroy()
            self.openSubWindow=False
       
        GEN=generalClass(self.caseDir);bcname=GEN.getBCName()

        if self.openSubWindow==False:  
        
            mwin=gtk.Window(gtk.WINDOW_TOPLEVEL);mwin.set_border_width(10);mwin.set_title('patch Average')
            mwin.set_position(gtk.WIN_POS_CENTER);mwin.set_transient_for(mainwindow)
            mainbox=gtk.VBox(False,0);mwin.add(mainbox)

            hbox1=gtk.HBox(False,5);mainbox.pack_start(hbox1,False,False,5)
            hbox2=gtk.HBox(False,5);mainbox.pack_start(hbox2,False,False,5)
            hbox3=gtk.HBox(False,5);hbox3.set_size_request(650,200);mainbox.pack_start(hbox3,False,False,5)

            label=gtk.Label('patch to average');label.set_size_request(160,28)
            patchCombo=gtk.combo_box_entry_new_text();patchCombo.set_size_request(140,28)
            for ii in bcname:patchCombo.append_text(ii);patchCombo.set_active(0)
            hbox1.pack_start(label,False,False,5);hbox1.pack_start(patchCombo,False,False,5)

            label=gtk.Label('variable to average');label.set_size_request(160,28)
            varCombo=gtk.combo_box_entry_new_text();varCombo.set_size_request(140,28)
            if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':variables=['p','U','k','epsilon','omega']
            else:variables=['p_rgh','p','U','T','k','epsilon','omega']
            for ii in variables:varCombo.append_text(ii);varCombo.set_active(0)
            hbox2.pack_start(label,False,False,5);hbox2.pack_start(varCombo,False,False,5)	
	
            sw=gtk.ScrolledWindow();sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            textview = gtk.TextView()
            textbuffer = textview.get_buffer()
            sw.add(textview)
            hbox3.pack_start(sw,True,True,5)

            buttonbox=gtk.HBox(False,5);mainbox.pack_start(buttonbox,False,False,5)
            closeB=gtk.Button(label='close');closeB.connect('clicked',closethis);closeB.set_size_request(140,28)
            applyB=gtk.Button(label='Calculate');applyB.connect('clicked',applythis);applyB.set_size_request(140,28)
            buttonbox.pack_end(closeB,False,False,5);buttonbox.pack_end(applyB,False,False,5)

            mwin.show_all()
        
            self.openSubWindow=True
    #----------------------------------------------------------------------------------------------
    def patchSummary(self):

        def closethis(widget):mwin.destroy()

        os.system('patchSummary -expand -time 0 -case '+self.caseDir+' > '+self.caseDir+'/patchSummary')

        mwin=gtk.Window(gtk.WINDOW_TOPLEVEL);mwin.set_border_width(10)
        mwin.set_title('Patch Summary');mwin.set_position(gtk.WIN_POS_CENTER)
        mainbox=gtk.VBox(False,0);mwin.add(mainbox)

        hbox=gtk.HBox(False,5);hbox.set_size_request(400,600);mainbox.pack_start(hbox,True,True,5)

        sw=gtk.ScrolledWindow();sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        textview=gtk.TextView()
        textbuffer=textview.get_buffer()
        sw.add(textview)
        hbox.pack_start(sw,True,True,5)

        f=open(self.caseDir+'/patchSummary','r');ff=f.readlines();f.close()
        res=ff[24:-2];string=''
        for ii in res:string=string+ii
        textbuffer.set_text(string)

        buttonbox=gtk.HBox(False,5);mainbox.pack_start(buttonbox,False,False,5)
        closeB=gtk.Button(label='close');closeB.connect('clicked',closethis);closeB.set_size_request(140,28)
        buttonbox.pack_end(closeB,False,False,5)

        mwin.show_all()
    #----------------------------------------------------------------------------------------------
    def probeLocation(self,mainwindow,gvtk,notebook):

        def start(widget):
            aa=self.varCombo.child;variable=aa.get_text()
            aa=self.numCombo.child;number=int(aa.get_text())

            xyz=[]    
            for i in range(number):
                xx=self.xCoords[i].get_text();yy=self.yCoords[i].get_text();zz=self.zCoords[i].get_text()
                xyz.append('('+xx+' '+yy+' '+zz+')')

            f=open(self.caseDir+'/system/probesDict','w')
            f.write('FoamFile\n');f.write('{\n');f.write('    version         2.0;\n')
            f.write('    format          ascii;\n');f.write('    class           dictionary;\n')
            f.write('    location        system;\n');f.write('    object          probesDict;\n')
            f.write('}\n')
            f.write('fields\n')
            f.write('(\n')
            f.write('    '+variable+'\n')
            f.write(');\n')
            f.write('probeLocations\n')
            f.write('(\n')
            for ii in xyz:f.write('    '+ii+'\n')
            f.write(');\n')	    
            f.close()

            aa=glob.glob(self.caseDir+'/processor*')
            if aa==[]:
                os.system('probeLocations -latestTime -noFunctionObjects -case '+self.caseDir+' > '+self.caseDir+'/log.probeLocation')
            else:
                os.system('mpirun -np '+str(len(aa))+' probeLocations -latestTime -noFunctionObjects -case '+self.caseDir+' -parallel > '+self.caseDir+'/log.probeLocation')

            aa=glob.glob(self.caseDir+'/postProcessing/probes/*')
            times=[]
            for ii in aa:
                bb=ii.split('/');times.append(bb[-1])
            times.sort()
            if times!=[]:lastTime=times[-1]
            else:lastTime='0'

            aa=glob.glob(self.caseDir+'/postProcessing/probes/'+lastTime+'/'+variable)
            string=''
            if aa!=[]:
                f=open(aa[0],'r');ff=f.readlines();f.close()
                for ii in ff:string=string+'  '+ii
                textbuffer.set_text(string)
            else:
                textbuffer.set_text('There is no data')

            os.system('rm -r '+self.caseDir+'/postProcessing/probes/'+lastTime)
            
            f=open(self.caseDir+'/system/settings/probeLocations','w')                       
            f.write('field;'+variable+'\n')
            f.write('number;'+str(number)+'\n')
            for i in range(len(xyz)):f.write('coord;'+xyz[i][1:-1]+'\n')
            f.close()

        def stop(widget,data=None):
            self.Win.destroy()
            self.openSubWindow=False

        def setEntry(widget):
            old=self.itembox.get_children();oldnumber=len(old)
            oldx=[];oldy=[];oldz=[]
            for i in range(oldnumber):
                oldx.append(self.xCoords[i].get_text());oldy.append(self.yCoords[i].get_text());oldz.append(self.zCoords[i].get_text())

            aa=self.numCombo.child;pointnumber=int(aa.get_text())
            self.xCoords=[];self.yCoords=[];self.zCoords=[];hboxs=[];self.displayB=[]

            for i in range(len(old)):self.itembox.remove(old[i])
            for i in range(pointnumber):
                self.xCoords.append(gtk.Entry());self.yCoords.append(gtk.Entry());self.zCoords.append(gtk.Entry())
                self.displayB.append(gtk.Button('Display'));self.displayB[i].connect('clicked',displaypoint)		
                hboxs.append(gtk.HBox(False,5))
                self.itembox.pack_start(hboxs[i],False,False,5)
                label=gtk.Label('point'+str(i+1))
                self.xCoords[i].set_size_request(60,28);self.yCoords[i].set_size_request(60,28);self.zCoords[i].set_size_request(60,28)
                self.displayB[i].set_size_request(80,28)
                hboxs[i].pack_start(label,False,False,5)
                hboxs[i].pack_end(self.displayB[i],False,False,5)
                hboxs[i].pack_end(self.zCoords[i],False,False,5);hboxs[i].pack_end(self.yCoords[i],False,False,0)
                hboxs[i].pack_end(self.xCoords[i],False,False,5)                
            self.Win.show_all()

            if oldnumber > pointnumber:
                for i in range(pointnumber):
                    self.xCoords[i].set_text(oldx[i]);self.yCoords[i].set_text(oldy[i]);self.zCoords[i].set_text(oldz[i])
            else:
                for i in range(oldnumber):
                    self.xCoords[i].set_text(oldx[i]);self.yCoords[i].set_text(oldy[i]);self.zCoords[i].set_text(oldz[i])

        def displaypoint(widget):
            i=self.displayB.index(widget)
            xc=self.xCoords[i].get_text();yc=self.yCoords[i].get_text();zc=self.zCoords[i].get_text()            
            VTK=VTKStuff(self.caseDir,self.rr,self.gg,self.bb);renderer=VTK.showPoint(xc,yc,zc)
            gvtk.GetRenderWindow().AddRenderer(renderer)
            gvtk.Initialize();gvtk.Start()
            
            notebook.set_current_page(1)
            
        def setSavedValue():
            aa=glob.glob(self.caseDir+'/system/settings/probeLocations')
            if len(aa)!=0:
                GEN=generalClass(self.caseDir)
                dic=GEN.makeDictFromFile(aa[0])
                self.varCombo.set_active(variables.index(dic['field']))
                self.numCombo.set_active(numList.index(dic['number']))
                for i in range(int(dic['number'])):
                    aa=dic['coord'].split(' ')
                    self.xCoords[i].set_text(aa[0]);self.yCoords[i].set_text(aa[1]);self.zCoords[i].set_text(aa[2])
                     
        #--------------------------------------------            
        if self.openSubWindow==False:
       
            self.Win=gtk.Window(gtk.WINDOW_TOPLEVEL);self.Win.connect('delete_event',self.delete_event);self.Win.connect('destroy',self.destroy)
            self.Win.set_border_width(10);self.Win.set_title('Probe Location');self.Win.set_transient_for(mainwindow)
            self.Win.set_position(gtk.WIN_POS_CENTER)

            self.hbox=gtk.HBox(False,5);self.Win.add(self.hbox)
            self.vbox=gtk.VBox(False,5);self.resultbox=gtk.VBox(True,5)
            self.hbox.pack_start(self.vbox,False,False,5);self.hbox.pack_start(self.resultbox,True,True,5)
            self.regbox=gtk.HBox(False,5)
            self.vbox.pack_start(self.regbox,False,False,5)
	
            label=gtk.Label('Select Field')
            self.varCombo=gtk.combo_box_entry_new_text()
            self.varCombo.set_size_request(140,28)
            if self.solver=='simpleNFoam' or self.solver=='pimpleNFoam':variables=['p','U','k','epsilon','omega']
            else: variables=['p','p_rgh','U','T','k','epsilon','omega']
            for ii in variables:self.varCombo.append_text(ii)
            self.varCombo.set_active(0)
            self.varBox=gtk.HBox(False,5);self.vbox.pack_start(self.varBox,False,False,5)
            self.varBox.pack_start(label,False,False,5);self.varBox.pack_end(self.varCombo,False,False,5)
            #------------------------------
            self.locFrame=gtk.Frame()
            self.locFrame.set_label('Probe Locations');self.locFrame.set_label_align(0.0,1.0)
            self.locFrame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)

            self.locBox=gtk.VBox(False,5)        
            self.vbox.pack_start(self.locFrame,False,False,5)
            self.locFrame.add(self.locBox)

            self.nlabel=gtk.Label('Number of Points')
            self.numCombo=gtk.combo_box_entry_new_text()
            self.numCombo.set_size_request(100,28)
            numList=['0','1','2','3','4','5','6','7','8','9','10']
            for jj in numList:self.numCombo.append_text(jj)
            self.numCombo.set_active(0)
            self.numCombo.connect('changed',setEntry)
    
            self.npointbox=gtk.HBox(False,5)
            self.locBox.pack_start(self.npointbox,False,False,5)	
            self.npointbox.pack_start(self.nlabel,False,False,5);self.npointbox.pack_end(self.numCombo,False,False,5)

            self.swin=gtk.ScrolledWindow();self.swin.set_border_width(5);self.swin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
            self.locBox.pack_start(self.swin)
            self.itembox=gtk.VBox(False,0)
            self.swin.add_with_viewport(self.itembox)
            self.swin.set_size_request(400,200)
            #-----------------------------------
            self.buttonBox=gtk.HBox(False,5)
            self.vbox.pack_start(self.buttonBox,False,False,5)
            self.appB=gtk.Button('Apply');self.closeB=gtk.Button('Close')
            self.appB.set_size_request(100,28);self.closeB.set_size_request(100,28)
            self.appB.connect('clicked',start);self.closeB.connect('clicked',stop)
            self.buttonBox.pack_end(self.closeB,False,False,5);self.buttonBox.pack_end(self.appB,False,False,5)
            #------------------------------
            self.resFrame=gtk.Frame();self.resFrame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
            self.resultbox.pack_start(self.resFrame,True,True,5)

            sw=gtk.ScrolledWindow();sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            sw.set_size_request(300,300)
            textview = gtk.TextView()
            textbuffer = textview.get_buffer()
            sw.add(textview)
            self.resFrame.add(sw)

            setSavedValue()
            
            self.Win.show_all()
            
            self.openSubWindow=True
    #----------------------------------------------------------------------------------
    def main(self):
        gtk.main()

if __name__=='__main__':
    solverwin=Win()
    solverwin.main()
