    Info<< "Reading thermophysical properties\n" << endl;

    autoPtr<rhoThermo> pThermo(rhoThermo::New(mesh));
    rhoThermo& thermo = pThermo();
    thermo.validate(args.executable(), "h", "e");

    volScalarField rho
    (
        IOobject
        (
            "rho",
            runTime.timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        thermo.rho()
    );
    rho.write();

    dimensionedScalar initialMass = fvc::domainIntegrate(rho);
    dimensionedScalar totalVolume = sum(mesh.V());

    IOdictionary operatingConditions
    (
        IOobject
        (
            "operatingConditions",
            runTime.constant(),
            mesh,
            IOobject::MUST_READ_IF_MODIFIED,
            IOobject::NO_WRITE
        )
    );

    dimensionedScalar Op(operatingConditions.lookup("Op"));

    dimensionedScalar rho0("rho0", initialMass/totalVolume);

    bool rho0Specified
    (
        operatingConditions.readIfPresent("rho0", rho0)
    ); 
    volScalarField& p = thermo.p();
//    const volScalarField& psi = thermo.psi();

    Info<< "Reading field U\n" << endl;
    volVectorField U
    (
        IOobject
        (
            "U",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

    Info<< "Reading/calculating face flux field phi\n" << endl;
    surfaceScalarField phi
    (
        IOobject
        (
            "phi",
            runTime.timeName(),
            mesh,
            IOobject::READ_IF_PRESENT,
            IOobject::AUTO_WRITE
        ),
        fvc::interpolate(rho*U) & mesh.Sf()
    );

    Info<< "Creating turbulence model\n" << endl;
    autoPtr<compressible::RASModel> turbulence
    (
        compressible::RASModel::New
        (
            rho,
            U,
            phi,
            thermo
        )
    );

    Info<< "Calculating field g.h\n" << endl;
    volScalarField gh("gh", g & mesh.C());

    Info<< "Reading field p_rgh\n" << endl;
    volScalarField p_rgh
    (
        IOobject
        (
            "p_rgh",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

    // For post-processing
    volScalarField Umag
    (
        IOobject
        (
            "Umag",
            runTime.timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mag(U)
    );
    Umag.write();

    // User-Defined Scalars
    IOdictionary scalarTransportProperties
    (
        IOobject
        (
            "scalarTransportProperties",
            runTime.time().constant(),
            runTime.db(),
            IOobject::READ_IF_PRESENT,
            IOobject::NO_WRITE
        )
    );

    scalar scalars = 0;

    if (scalarTransportProperties.headerOk())
    {
        scalars = readScalar
        (
            scalarTransportProperties.lookup("numberOfScalars")
        );
    }

    PtrList<volScalarField> scalarFluid(scalars);
    PtrList<scalarTransportModel> scalarTransportFluid(scalars);

    forAll(scalarFluid, i)
    {
        OStringStream buffer;
        buffer << i;

        scalarFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    scalarTransportProperties.subDict
                    (
                        "scalar" + buffer.str()
                    ).lookup("name"),
                    runTime.timeName(),
                    mesh,
                    IOobject::MUST_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh
            )
        );

        scalarTransportFluid.set
        (
            i,
            new scalarTransportModel
            (
                "scalar" + buffer.str(),
                scalarFluid[i]
            )
        );
    }
