    // Initialise fluid field pointer lists
    PtrList<rhoThermo> thermoFluid(fluidRegions.size());
    PtrList<volScalarField> rhoFluid(fluidRegions.size());
    PtrList<dimensionedScalar> initialMassFluid(fluidRegions.size());
    PtrList<dimensionedScalar> totalVolumeFluid(fluidRegions.size());
    PtrList<IOdictionary> operatingConditionsFluid(fluidRegions.size());
    PtrList<dimensionedScalar> OpFluid(fluidRegions.size());
    PtrList<dimensionedScalar> rho0Fluid(fluidRegions.size());
    PtrList<bool> rho0SpecifiedFluid(fluidRegions.size());

    PtrList<volVectorField> UFluid(fluidRegions.size());
    PtrList<surfaceScalarField> phiFluid(fluidRegions.size());
    PtrList<compressible::RASModel> turbulenceFluid
    ( 
        fluidRegions.size()
    );

    PtrList<uniformDimensionedVectorField> gFluid(fluidRegions.size());
    PtrList<volScalarField> ghFluid(fluidRegions.size());
    PtrList<volScalarField> p_rghFluid(fluidRegions.size());

    PtrList<radiation::radiationModel> radiationFluid(fluidRegions.size());

    PtrList<fv::IOoptionList> fluidFvOptions(fluidRegions.size());

//    PtrList<volScalarField> massImbalanceFluid(fluidRegions.size());

    // Populate fluid field pointer lists
    forAll(fluidRegions, i)
    {
        Info<< "*** Reading fluid mesh thermophysical properties for region "
            << fluidRegions[i].name() << nl << endl;

        Info<< "    Adding to thermoFluid\n" << endl;
        thermoFluid.set
        (
            i,
            rhoThermo::New(fluidRegions[i]).ptr()
        );

        Info<< "    Adding to rhoFluid\n" << endl;
        rhoFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "rho",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                thermoFluid[i].rho()
            )
        );
        rhoFluid[i].write();

        initialMassFluid.set
        (
            i,
            new dimensionedScalar
            (
                fvc::domainIntegrate(rhoFluid[i])
            )
        );

        totalVolumeFluid.set
        (
            i,
            new dimensionedScalar
            (
                sum(fluidRegions[i].V())
            )
        );

        operatingConditionsFluid.set
        (
            i,
            new IOdictionary
            (
                IOobject
                (
                    "operatingConditions",
                    runTime.constant(),
                    fluidRegions[i],
                    IOobject::MUST_READ_IF_MODIFIED,
                    IOobject::NO_WRITE
                )
            )
        );

        OpFluid.set
        (
            i,
            new dimensionedScalar
            (
                operatingConditionsFluid[i].lookup("Op")
            )
        );

        rho0Fluid.set
        (
            i,
            new dimensionedScalar
            (
                "rho0",
                initialMassFluid[i]/totalVolumeFluid[i]
            )
        );

        rho0SpecifiedFluid.set
        (
            i,
            new bool
            (
                operatingConditionsFluid[i].readIfPresent
                (
                   "rho0",
                   rho0Fluid[i]
                )
            )
        );

        Info<< "    Adding to UFluid\n" << endl;
        UFluid.set
        (
            i,
            new volVectorField
            (
                IOobject
                (
                    "U",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::MUST_READ,
                    IOobject::AUTO_WRITE
                ),
                fluidRegions[i]
            )
        );

        Info<< "    Adding to phiFluid\n" << endl;
        phiFluid.set
        (
            i,
            new surfaceScalarField
            (
                IOobject
                (
                    "phi",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::READ_IF_PRESENT,
                    IOobject::AUTO_WRITE
                ),
                fvc::interpolate(rhoFluid[i]*UFluid[i])
                    & fluidRegions[i].Sf()
            )
        );

        Info<< "    Adding turbulence \n" << endl;
        turbulenceFluid.set
        (
            i,
            compressible::RASModel::New
            (
                rhoFluid[i],
                UFluid[i],
                phiFluid[i],
                thermoFluid[i]
            ).ptr()
        );

        Info<< "    Adding to gFluid\n" << endl;
        gFluid.set
        (
            i,
            new uniformDimensionedVectorField
            (
                IOobject
                (
                    "g",
                    runTime.constant(),
                    fluidRegions[i],
                    IOobject::MUST_READ,
                    IOobject::NO_WRITE
                )
            )
        );

        Info<< "    Adding to ghFluid\n" << endl;
        ghFluid.set
        (
            i,
            new volScalarField("gh", gFluid[i] & fluidRegions[i].C())
        );

        p_rghFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "p_rgh",
                    runTime.timeName(),
                    fluidRegions[i],
                    IOobject::MUST_READ,
                    IOobject::AUTO_WRITE
                ),
                fluidRegions[i]
            )
        );

        radiationFluid.set
        (
            i,
            radiation::radiationModel::New(thermoFluid[i].T())
        );

        Info<< "    Adding fvOptions\n" << endl;
        fluidFvOptions.set
        (
            i,
            new fv::IOoptionList(fluidRegions[i])
        );
    }

    // User-defined scalar
    List<PtrList<volScalarField> > scalarFluid(scalars);
    List<PtrList<bool> > solveScalarFluid(scalars);
    List<PtrList<scalarTransportModel> > scalarTransportFluid(scalars);

    forAll(uds, udsI)
    {
        OStringStream buffer;
        buffer << udsI;

        scalarFluid[udsI].setSize(fluidRegions.size());
        solveScalarFluid[udsI].setSize(fluidRegions.size());
        scalarTransportFluid[udsI].setSize(fluidRegions.size());

        forAll(fluidRegions, i)
        {
            solveScalarFluid[udsI].set
            (
                i,
                new bool
                (
                    false
                )
            );

            bool& solveScalar = solveScalarFluid[udsI][i];

            forAll (uds[udsI], j)
            {
                if (uds[udsI][j] == fluidRegions[i].name())
                {
                    solveScalar = true;
                }
            }

            if (solveScalar)
            {
                IOdictionary scalarTransportProperties
                (
                    IOobject
                    (
                        "scalarTransportProperties",
                        runTime.time().constant(),
                        fluidRegions[i],
                        IOobject::MUST_READ,
                        IOobject::NO_WRITE
                    )
                );

                scalarFluid[udsI].set
                (
                    i,
                    new volScalarField
                    (
                        IOobject
                        (
                            scalarTransportProperties.subDict
                            (
                                "scalar" + buffer.str()
                            ).lookup("name"),
                            runTime.timeName(),
                            fluidRegions[i],
                            IOobject::MUST_READ,
                            IOobject::AUTO_WRITE
                        ),
                        fluidRegions[i]
                    )
                );

                scalarTransportFluid[udsI].set
                (
                    i,
                    new scalarTransportModel
                    (
                        "scalar" + buffer.str(),
                        scalarFluid[udsI][i]
                    )
                );
            }
        }
    }

/*        // Mass Imbalance
        Info<< "    Adding to massImbalanceFluid\n" << endl;
        massImbalanceFluid.set
        (
            i,
            new volScalarField
            (
                IOobject
                (
                    "massImbalance",
                    runTime.timeName(),
                    fluidRegions[i]
                ),
                fluidRegions[i],
                dimensionedScalar
                (
                    "massImbalance",
                    dimMass/dimTime/dimVolume,
                    0.0
                )
            )
        ); 
*/
