forAll(fluidNames, i)
{
    IOobject fvSolution
    (
        "fvSolution",
        runTime.system(),
        fluidNames[i],
        runTime,
        IOobject::NO_READ,
        IOobject::NO_WRITE,
        false
    );

    Info<< "Testing:" << fvSolution.objectPath() << endl;

    if (!fvSolution.headerOk())
    {
        Info<< "Writing default " << fluidNames[i]/fvSolution.name() << endl;

        dictionary defaultDict;

        dictionary solversDict;

        // rho
        dictionary rhoDict;
        rhoDict.add("solver", "PCG", true);
        rhoDict.add("preconditioner", "DIC", true);
        rhoDict.add("tolerance", "1e-07", true);
        rhoDict.add("relTol", "0.1", true);
        rhoDict.add("minIter", "1", true);

        // rhoFinal
        dictionary rhoFinalDict;
        rhoFinalDict.add("$rho", "", true);
        rhoFinalDict.add("relTol", "0", true);

        // p_rgh
        dictionary p_rghDict;
        p_rghDict.add("solver", "GAMG", true);
        p_rghDict.add("smoother", "GaussSeidel", true);
        p_rghDict.add("cacheAgglomeration", "true", true);
        p_rghDict.add("nCellsInCoarsestLevel", "10", true);
        p_rghDict.add("agglomerator", "faceAreaPair", true);
        p_rghDict.add("mergeLevels", "1", true);
        p_rghDict.add("tolerance", "1e-07", true);
        p_rghDict.add("relTol", "0.01", true);
        p_rghDict.add("minIter", "1", true);
        p_rghDict.add("maxIter", "20", true);

        // p_rghFinal
        dictionary p_rghFinalDict;
        p_rghFinalDict.add("$p_rgh", "", true);
        p_rghFinalDict.add("relTol", "0", true);

        // "(U|h|k|epsilon|omega|R)"
        dictionary otherDict;
        otherDict.add("solver", "PBiCG", true);
        otherDict.add("preconditioner", "DILU", true);
        otherDict.add("tolerance", "1e-07", true);
        otherDict.add("relTol", "0.1", true);
        otherDict.add("minIter", "1", true);
        otherDict.add("maxIter", "20", true);

        // "(U|h|k|epsilon|omega|R)Final"
        dictionary otherFinalDict;
        otherFinalDict.add("$U", "", true);
        otherFinalDict.add("relTol", "0", true);

        solversDict.add("rho", rhoDict);
        solversDict.add("rhoFinal", rhoFinalDict);
        solversDict.add("p_rgh", p_rghDict);
        solversDict.add("p_rghFinal", p_rghFinalDict);
        solversDict.add(string("(U|h|k|epsilon|omega|R)"), otherDict);
        solversDict.add(string("(U|h|k|epsilon|omega|R)Final"), otherFinalDict);


        dictionary pimpleDict;
        pimpleDict.add("momentumPredictor", "on", true);
        pimpleDict.add("nCorrectors", "2", true);
        pimpleDict.add("nNonOrthogonalCorrectors", "1", true);

        dictionary simpleDict;
        simpleDict.add("nNonOrthogonalCorrectors", "1", true);

        dictionary relaxDict;

        dictionary fieldsRelaxDict;
        fieldsRelaxDict.add(string("rho.*"), "1.0", true);
        fieldsRelaxDict.add(string("p_rgh.*"), "0.3", true);

        dictionary eqnRelaxDict;
        eqnRelaxDict.add(string("U.*"), "0.7", true);
        eqnRelaxDict.add(string("h.*"), "1.0", true);
        eqnRelaxDict.add(string("k.*"), "0.8", true);
        eqnRelaxDict.add(string("epsilon.*"), "0.8", true);
        eqnRelaxDict.add(string("omega.*"), "0.8", true);

        relaxDict.add("fields", fieldsRelaxDict);
        relaxDict.add("equations", eqnRelaxDict);

        defaultDict.add("solvers", solversDict);
        defaultDict.add("PIMPLE", pimpleDict);
        defaultDict.add("SIMPLE", simpleDict);
        defaultDict.add("relaxationFactors", relaxDict);

        IOdictionary(fvSolution, defaultDict).regIOobject::write();
    }
}

forAll(solidNames, i)
{
    IOobject fvSolution
    (
        "fvSolution",
        runTime.system(),
        solidNames[i],
        runTime,
        IOobject::NO_READ,
        IOobject::NO_WRITE,
        false
    );

    Info<< "Testing:" << fvSolution.objectPath() << endl;

    if (!fvSolution.headerOk())
    {
        Info<< "Writing default " << solidNames[i]/fvSolution.name() << endl;

        dictionary defaultDict;

        dictionary solversDict;

        dictionary hDict;
        hDict.add("solver", "PCG", true);
        hDict.add("preconditioner", "DIC", true);
        hDict.add("tolerance", "1e-06", true);
        hDict.add("relTol", "0.1", true);
        hDict.add("minIter", "1", true);

        dictionary hFinalDict;
        hFinalDict.add("$h", "", true);
        hFinalDict.add("relTol", "0", true);

        solversDict.add("h", hDict);
        solversDict.add("hFinal", hFinalDict);

        dictionary pimpleDict;
        pimpleDict.add("nNonOrthogonalCorrectors", "0", true);

        dictionary relaxDict;

        dictionary eqnRelaxDict;
        eqnRelaxDict.add(string("h.*"), "1.0", true);

        relaxDict.add("equations", eqnRelaxDict);

        defaultDict.add("solvers", solversDict);
        defaultDict.add("PIMPLE", pimpleDict);
        defaultDict.add("relaxationFactors", relaxDict);

        IOdictionary(fvSolution, defaultDict).regIOobject::write();
    }
}
