/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "constantDiffusivity.H"
#include "addToRunTimeSelectionTable.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace diffusivityModels
{
    defineTypeNameAndDebug(constantDiffusivity, 0);
    addToRunTimeSelectionTable(diffusivityModel, constantDiffusivity, dictionary);
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::diffusivityModels::constantDiffusivity::constantDiffusivity
(
    const word& name,
    const dictionary& diffusivityProperties,
    const volScalarField& scalar
)
:
    diffusivityModel(name, diffusivityProperties, scalar),
    d0_("d", dimensionSet(1, -1, -1, 0, 0), diffusivityProperties_.lookup("d")),
    d_
    (
        IOobject
        (
            name,
            scalar_.time().timeName(),
            scalar_.db(),
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        scalar_.mesh(),
        d0_
    )
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

bool Foam::diffusivityModels::constantDiffusivity::read
(
    const dictionary& diffusivityProperties
)
{
    diffusivityModel::read(diffusivityProperties);

    diffusivityProperties_.lookup("d") >> d0_;
    d_ = d0_;

    return true;
}


// ************************************************************************* //
