/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "polynomialDiffusivity.H"
#include "addToRunTimeSelectionTable.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace diffusivityModels
{
    defineTypeNameAndDebug(polynomialDiffusivity, 0);
    addToRunTimeSelectionTable
    (
        diffusivityModel,
        polynomialDiffusivity,
        dictionary
    );
}
}


// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::diffusivityModels::polynomialDiffusivity::calcD() const
{
    const volScalarField& T = scalar_.db().lookupObject<volScalarField>("T");

    return A_ + B_*T + C_*pow(T, 2) + D_*pow(T, 3) + E_*pow(T, 4);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::diffusivityModels::polynomialDiffusivity::polynomialDiffusivity
(
    const word& name,
    const dictionary& diffusivityProperties,
    const volScalarField& scalar
)
:
    diffusivityModel(name, diffusivityProperties, scalar),
    polynomialDiffusivityCoeffs_
    (
        diffusivityProperties.subDict(typeName + "Coeffs")
    ),
    A_
    (
        "A",
        dimensionSet(1, -1, -1, 0, 0),
        polynomialDiffusivityCoeffs_.lookup("A")
    ),
    B_
    (
        "B",
        dimensionSet(1, -1, -1, -1, 0),
        polynomialDiffusivityCoeffs_.lookup("B")
    ),
    C_
    (
        "C",
        dimensionSet(1, -1, -1, -2, 0),
        polynomialDiffusivityCoeffs_.lookup("C")
    ),
    D_
    (
        "D",
        dimensionSet(1, -1, -1, -3, 0),
        polynomialDiffusivityCoeffs_.lookup("D")
    ),
    E_
    (
        "E",
        dimensionSet(1, -1, -1, -4, 0),
        polynomialDiffusivityCoeffs_.lookup("E")
    ),
    d_
    (
        IOobject
        (
            name,
            scalar_.time().timeName(),
            scalar_.db(),
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        calcD()
    )
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

bool Foam::diffusivityModels::polynomialDiffusivity::read
(
    const dictionary& diffusivityProperties
)
{
    diffusivityModel::read(diffusivityProperties);

    polynomialDiffusivityCoeffs_ = 
        diffusivityProperties.subDict(typeName + "Coeffs");

    polynomialDiffusivityCoeffs_.lookup("A") >> A_;
    polynomialDiffusivityCoeffs_.lookup("B") >> B_;
    polynomialDiffusivityCoeffs_.lookup("C") >> C_;
    polynomialDiffusivityCoeffs_.lookup("D") >> D_;
    polynomialDiffusivityCoeffs_.lookup("E") >> E_;

    return true;
}


// ************************************************************************* //
