/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::diffusivityModels::polynomialDiffusivity

Description
    An incompressible Bird-Carreau non-Newtonian diffusivity model.

    The Bird-Carreau-Yasuda form is also supported if the optional "a"
    coefficient is specified.  "a" defaults to 2 for the Bird-Carreau model.

SourceFiles
    polynomialDiffusivity.C

\*---------------------------------------------------------------------------*/

#ifndef polynomialDiffusivity_H
#define polynomialDiffusivity_H

#include "diffusivityModel.H"
#include "dimensionedScalar.H"
#include "volFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diffusivityModels
{

/*---------------------------------------------------------------------------*\
                           Class polynomialDiffusivity Declaration
\*---------------------------------------------------------------------------*/

class polynomialDiffusivity
:
    public diffusivityModel
{
    // Private data

        dictionary polynomialDiffusivityCoeffs_;

        dimensionedScalar A_;
        dimensionedScalar B_;
        dimensionedScalar C_;
        dimensionedScalar D_;
        dimensionedScalar E_;

        volScalarField d_;


    // Private Member Functions

        //- Calculate and return the laminar diffusivity
        tmp<volScalarField> calcD() const;


public:

    //- Runtime type information
    TypeName("polynomialDiffusivity");


    // Constructors

        //- construct from components
        polynomialDiffusivity
        (
            const word& name,
            const dictionary& diffusivityProperties,
            const volScalarField& scalar
        );


    //- Destructor
    ~polynomialDiffusivity()
    {}


    // Member Functions

        //- Return the laminar diffusivity
        tmp<volScalarField> d() const
        {
            return d_;
        }

        //- Return the laminar diffusivity for patch
        tmp<scalarField> d(const label patchi) const
        {
            return d_.boundaryField()[patchi];
        }

        //- Correct the laminar diffusivity
        void correct()
        {
            d_ = calcD();
        }

        //- Read transportProperties dictionary
        bool read(const dictionary& diffusivityProperties);
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace diffusivityModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
