/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "turbulentDiffusivity.H"
#include "addToRunTimeSelectionTable.H"
#include "surfaceFields.H"
#include "turbulenceModel.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace diffusivityModels
{
    defineTypeNameAndDebug(turbulentDiffusivity, 0);
    addToRunTimeSelectionTable
    (
        diffusivityModel, 
        turbulentDiffusivity, 
        dictionary
    );
}
}


// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::diffusivityModels::turbulentDiffusivity::getD() const
{
    const compressible::turbulenceModel& turbulence
    (
        scalar_.db().lookupObject<compressible::turbulenceModel>
        (
            "turbulenceModel"
        )
    );
 
    return 1.0*turbulence.alphat();
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::diffusivityModels::turbulentDiffusivity::turbulentDiffusivity
(
    const word& name,
    const dictionary& diffusivityProperties,
    const volScalarField& scalar
)
:
    diffusivityModel(name, diffusivityProperties, scalar),
    d_
    (
        IOobject
        (
            name,
            scalar_.time().timeName(),
            scalar_.db(),
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        getD()
    )
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

bool Foam::diffusivityModels::turbulentDiffusivity::read
(
    const dictionary& diffusivityProperties
)
{
    diffusivityModel::read(diffusivityProperties);

    return true;
}


// ************************************************************************* //
