/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::diffusivityModels::turbulentDiffusivity

Description
    An incompressible turbulentDiffusivity diffusivity model.

SourceFiles
    turbulentDiffusivity.C

\*---------------------------------------------------------------------------*/

#ifndef turbulentDiffusivity_H
#define turbulentDiffusivity_H

#include "diffusivityModel.H"
#include "dimensionedScalar.H"
#include "volFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace diffusivityModels
{

/*---------------------------------------------------------------------------*\
                           Class turbulentDiffusivity Declaration
\*---------------------------------------------------------------------------*/

class turbulentDiffusivity
:
    public diffusivityModel
{
    // Private data

        volScalarField d_;


    // Private Member Functions

        //- get turbulent diffusivity
        tmp<volScalarField> getD() const;


public:

    //- Runtime type information
    TypeName("turbulentDiffusivity");


    // Constructors

        //- construct from components
        turbulentDiffusivity
        (
            const word& name,
            const dictionary& diffusivityProperties,
            const volScalarField& scalar
        );


    //- Destructor
    ~turbulentDiffusivity()
    {}


    // Member Functions

        //- Return the laminar diffusivity
        tmp<volScalarField> d() const
        {
            return d_;
        }

        //- Return the laminar diffusivity for patch
        tmp<scalarField> d(const label patchi) const
        {
            return d_.boundaryField()[patchi];
        }

        //- Correct the laminar diffusivity (not appropriate, diffusivity constant)
        void correct()
        {
            d_ = getD();
        }

        //- Read transportProperties dictionary
        bool read(const dictionary& diffusivityProperties);
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace diffusivityModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
