/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "scalarTransportModel.H"
#include "diffusivityModel.H"
#include "volFields.H"


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::scalarTransportModel::scalarTransportModel
(
    const word& scalarIndex,
    const volScalarField& scalar
)
:
    IOdictionary
    (
        IOobject
        (
            "scalarTransportProperties",
            scalar.time().constant(),
            scalar.db(),
            IOobject::MUST_READ_IF_MODIFIED,
            IOobject::NO_WRITE
        )
    ),
    diffusivityModelPtr_
    (
        diffusivityModel::New
        (
            "d", 
            this->subDict(scalarIndex), 
            scalar
        )
    ),
    scalarIndex_(scalarIndex)
{}


// * * * * * * * * * * * * * * * * Destructors * * * * * * * * * * * * * * * //

Foam::scalarTransportModel::~scalarTransportModel()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::scalarTransportModel::d() const
{
    return diffusivityModelPtr_->d();
}


Foam::tmp<Foam::scalarField>
Foam::scalarTransportModel::d(const label patchi) const
{
    return diffusivityModelPtr_->d(patchi);
}


void Foam::scalarTransportModel::correct()
{
    diffusivityModelPtr_->correct();
}


bool Foam::scalarTransportModel::read()
{
    if (regIOobject::read())
    {
        return diffusivityModelPtr_->read(this->subDict(scalarIndex_));
    }
    else
    {
        return false;
    }
}


// ************************************************************************* //
