/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::scalarTransportModel

Description
    A simple single-phase transport model based on diffusivityModel.

    Used by the incompressible single-phase solvers like simpleFoam,
    turbFoam etc.

SourceFiles
    scalarTransportModel.C

\*---------------------------------------------------------------------------*/

#ifndef scalarTransportModel_H
#define scalarTransportModel_H

#include "primitiveFieldsFwd.H"
#include "volFieldsFwd.H"
#include "surfaceFieldsFwd.H"
#include "IOdictionary.H"
#include "autoPtr.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class diffusivityModel;

/*---------------------------------------------------------------------------*\
                Class scalarTransportModel Declaration
\*---------------------------------------------------------------------------*/

class scalarTransportModel
:
    public IOdictionary
{
    // Private Data

        autoPtr<diffusivityModel> diffusivityModelPtr_;

        const word scalarIndex_;


    // Private Member Functions

        //- Disallow copy construct
        scalarTransportModel(const scalarTransportModel&);

        //- Disallow default bitwise assignment
        void operator=(const scalarTransportModel&);


public:

    // Constructors

        //- Construct from components
        scalarTransportModel
        (
            const word& scalarIndex,
            const volScalarField& scalar
        );


    //- Destructor
    ~scalarTransportModel();


    // Member Functions

        //- Return the diffusivity
        tmp<volScalarField> d() const;

        //- Return the diffusivity for patch
        tmp<scalarField> d(const label patchi) const;

        //- Correct the laminar diffusivity
        void correct();

        //- Read transportProperties dictionary
        bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
