/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Description
    Macro for relax surface normal gradient correction

\*---------------------------------------------------------------------------*/

#ifndef relaxSnGradCorr_H
#define relaxSnGradCorr_H

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * /

#define QUOTE(fileName) #fileName                                            \


#define relaxSnGradCorr(eqn, Type)                                           \
                                                                             \
    if (eqn.faceFluxCorrectionPtr() && eqn##SnGradCorrUrf[0] < 1.0)          \
    {                                                                        \
        eqn##SnGradCorr.writeOpt() = IOobject::AUTO_WRITE;                   \
                                                                             \
        Type eqn##SnGradCorrOld = eqn##SnGradCorr;                           \
                                                                             \
        eqn##SnGradCorr = *(eqn.faceFluxCorrectionPtr());                    \
                                                                             \
        surfaceScalarField signChangeFlagField                               \
        (                                                                    \
            0.5*mag(sign(eqn##SnGradCorr) + sign(eqn##SnGradCorrOld))        \
        );                                                                   \
                                                                             \
        surfaceScalarField modifiedUrf                                       \
        (                                                                    \
            eqn##SnGradCorrUrf[0]                                            \
          - 0.25*eqn##SnGradCorrUrf[0]*(1.0 - signChangeFlagField)           \
        );                                                                   \
                                                                             \
        Type eqn##SnGradCorrRelaxed                                          \
        (                                                                    \
            eqn##SnGradCorrOld                                               \
          + min(modifiedUrf, 1.0)*(eqn##SnGradCorr - eqn##SnGradCorrOld)     \
        );                                                                   \
                                                                             \
        eqn.source() += mesh.V()                                             \
            *fvc::div                                                        \
            (                                                                \
                eqn##SnGradCorr - eqn##SnGradCorrRelaxed                     \
            )().internalField();                                             \
                                                                             \
        *(eqn.faceFluxCorrectionPtr()) = eqn##SnGradCorrRelaxed;             \
                                                                             \
        eqn##SnGradCorr = eqn##SnGradCorrRelaxed;                            \
                                                                             \
        if (pimple.finalInnerIter())                                         \
        {                                                                    \
            eqn##SnGradCorr                                                  \
                == dimensionedScalar("zero", dimVolume/dimTime, 0.0);        \
        }                                                                    \
    }


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
