/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    LGCDryerSetBCs

Description

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "regionProperties.H"

#include "replaceEntryValue.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    argList::noBanner();

    argList::addNote
    (
        "Replace entry value of specific boundary patch"
    );

    timeSelector::addOptions(true, true);
    argList::noParallel();
    #include "addRegionOption.H"

    argList::addOption
    (
        "field",
        "name",
        "specify a field to change boundary entry value" 
    );

    argList::addOption
    (
        "patch",
        "name",
        "specify a patch to change entry value"
    );

    argList::addOption
    (
        "entry",
        "name",
        "specify an entry to change value"
    );

    argList::addOption
    (
        "value",
        "value",
        "specify a value to replace existing entry value"
    );

    argList::addBoolOption
    (
        "parallel",
        "change entry value in decomposed case"
    );

    #include "setRootCase.H"
    //Foam::Info<< "Create time\n" << Foam::endl;
    Foam::Time runTime(Foam::Time::controlDictName, args);

    word selectedField = args.optionRead<word>("field");
    word selectedPatch = args.optionRead<word>("patch");
    word selectedEntry = args.optionRead<word>("entry");
    word value = args.optionRead<word>("value");

    word regionName(fvMesh::defaultRegion);
    args.optionReadIfPresent("region", regionName);

    const bool parallel = args.optionFound("parallel");

    if (parallel)
    {
        label nProcs = 0;
        while (isDir(args.path()/(word("processor") + name(nProcs))))
        {
            ++nProcs;
        }

        // Create the processor databases
        PtrList<Time> databases(nProcs);

        forAll(databases, procI)
        {
            databases.set
            (
                procI,
                new Time
                (
                    Time::controlDictName,
                    args.rootPath(),
                    args.caseName()/fileName(word("processor") + name(procI))
                )
            );
        }

        // Use the times list from the master processor
        // and select a subset based on the command-line options
        instantList timeDirs = timeSelector::select
        (
            databases[0].times(),
            args
        );

        if
        (
            !args.optionFound("latestTime")
         && !args.optionFound("time")
        )
        {
            forAll(databases, procI)
            {
                databases[procI].setTime(instant(0, "0"), 0);
            }
        }
        else
        {
            forAll(databases, procI)
            {
                databases[procI].setTime(timeDirs[0], 0);
            }
        }

        forAll(databases, procI)
        {
            replaceEntryValue
            (
                databases[procI],
                regionName,
                databases[procI].timeName(),
                selectedField,
                selectedPatch,
                selectedEntry,
                value
            );
        }
    }
    else
    {
        instantList timeDirs = timeSelector::select
        (
            runTime.times(),
            args
        );
    
        if
        (
            !args.optionFound("latestTime")
         && !args.optionFound("time")
        )
        {
            runTime.setTime(instant(0, "0"), 0);
        }
        else
        {
            runTime.setTime(timeDirs[0], 0);
        }

        replaceEntryValue
        (
            runTime,
            regionName,
            runTime.timeName(),
            selectedField,
            selectedPatch,
            selectedEntry,
            value
        );
    }

    //Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
